/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

/**
 * This annotation can be used to specify the logging level for RPC calls. The annotation can be
 * attached to RPC interface and/or methods. An annotation at the method takes precedence if both
 * are present.
 *
 * The logging is performed using apache common logging {@link Log} with the default log level
 * defined in {@link DefaultValues#DEFAULT_LOG_LEVEL} for all calls that do not specify an explicit
 * log level using this annotation. The names for the logs used are:
 * <code>de.starface.com.rpc.client.interceptor.LoggingRequestInterceptor</code> for the client and
 * <code>de.starface.com.rpc.server.interceptor.LoggingRequestInterceptor</code> for the server.
 *
 * @author tilmann
 */
@Documented
@Inherited
@Retention(RUNTIME)
@Target({METHOD, TYPE})
public @interface RpcLogging {
	// Default value in nested interface due to JavaC Bug
	// http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6857918
	public static interface DefaultValues {
		/**
		 * The default log level used for calls not specifying an {@link RpcLogging} annotation.
		 * Change this value to alter the log level for these calls.
		 */
		public static RpcLoggingLevel DEFAULT_LOG_LEVEL = RpcLoggingLevel.DEBUG;
	}

	/**
	 * Identifies the log level being used to log the RPC call(s). If the value is not specified in
	 * the annotation the level {@link RpcLoggingLevel#DEBUG} is used.
	 */
	RpcLoggingLevel value() default RpcLoggingLevel.DEBUG;
}
