/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */

package de.starface.com.rpc.client;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcThreadFactory;
import de.starface.com.rpc.common.RpcTransportToken;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executor;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

public class AsynchronousExecutor {
	private final Logger log = LogManager.getLogger(getClass());
	private ConcurrentHashMap<String, Object> syncObjects = new ConcurrentHashMap<>();
	private ConcurrentHashMap<RpcTransportToken, Executor> executors = new ConcurrentHashMap<>();
	private ConcurrentHashMap<RpcTransportToken, Integer> executorSizes = new ConcurrentHashMap<>();

	private Executor getExecutor(RpcTransportToken transportToken) {
		Executor executor = executors.get(transportToken);
		if (executor == null) {
			executorSizes.put(transportToken, 100);
			executor = createExecutor(transportToken.getIdentifierString(), 100);
			executors.put(transportToken, executor);
		}
		return executor;
	}

	private Executor createExecutor(String name, int size) {
		// This means for each AsynchronousExecutorProvider all asynchronous calls will be executed
		// one after the other in a single thread and new calls exceeding 100 will be rejected
		return new ThreadPoolExecutor(0, 1, 30, TimeUnit.SECONDS, new LinkedBlockingQueue<>(
			size),
			new RpcThreadFactory("AsynchronousExecutor[" + name + "]"));
	}

	public void execute(Runnable runnable, RpcTransportToken transportToken) throws RpcException {
		Object syncObject = syncObjects.computeIfAbsent(transportToken.getIdentifierString(), k -> new Object());
		synchronized (syncObject) {
			ThreadPoolExecutor executor = (ThreadPoolExecutor) getExecutor(transportToken);
			try {
				if (queueMustBeReset(executor, transportToken)) {
					executor = exchangeExecutor(transportToken, 100, false);
					log.info("Client/Network reset to 100 TransportToken: " + transportToken.getIdentifierString() +
						" QueueSize: " + executor.getQueue().size());
				}
				executor.execute(runnable);
				if (log.isDebugEnabled()) {
					log.debug("TransportToken: " + transportToken.getIdentifierString() + " QueueSize: " +
						executor.getQueue().size() + " ActiveCount: " + executor.getActiveCount() +
						" Registered size: " + executorSizes.get(transportToken));
				}
			} catch (RejectedExecutionException e) {
				Integer size = executorSizes.get(transportToken);
				if (size == null) {
					size = 100;
				}
				if (size >= 1000) {
					log.warn(
						"Client/Network maybe to slow TransportToken: " + transportToken.getIdentifierString() +
							" current elements " +
							executor.getQueue().size() + " client will be unregistered");
					exchangeExecutor(transportToken, 100, true);
					throw new RpcException(RpcException.Codes.Client.ASYNC_CALL_BUFFER_OVERFLOW,
						"[" + Thread.currentThread().getName() +
							"] Too many asynchronous calls awaiting execution for this RPC client. ( 1000 Tasks )",
						e);
				} else {
					ThreadPoolExecutor createdExecutor = exchangeExecutor(transportToken, size + 100, false);
					createdExecutor.execute(runnable);
					log.warn(
						"Client/Network maybe to slow TransportToken: " + transportToken.getIdentifierString() +
							" new QueueSize: " + size + " current elements " + createdExecutor.getQueue().size());
				}
			}
		}
	}

	private ThreadPoolExecutor exchangeExecutor(RpcTransportToken transportToken, int newSize, boolean removeAllTasks) {
		executorSizes.put(transportToken, newSize);
		ThreadPoolExecutor executor = (ThreadPoolExecutor) executors.remove(transportToken);
		List<Runnable> runnables = shutdownExecutor(executor);
		ThreadPoolExecutor createdExecutor =
			(ThreadPoolExecutor) createExecutor(transportToken.getIdentifierString(), newSize);
		executors.put(transportToken, createdExecutor);
		if (!removeAllTasks) {
			createdExecutor.getQueue().addAll(runnables);
		}
		return createdExecutor;
	}

	private List<Runnable> shutdownExecutor(ThreadPoolExecutor executorService) {
		executorService.shutdown();
		ArrayList<Runnable> elements = new ArrayList<>();
		executorService.getQueue().drainTo(elements);
		try {
			if (!executorService.awaitTermination(250, TimeUnit.MILLISECONDS)) {
				elements.addAll(executorService.shutdownNow());
		    } 
		} catch (InterruptedException e) {
			elements.addAll(executorService.shutdownNow());
		}
		return elements;
	}

	private boolean queueMustBeReset(ThreadPoolExecutor executor, RpcTransportToken transportToken) {
		return executor.getQueue().isEmpty() && executorSizes.containsKey(transportToken) &&
			executorSizes.get(transportToken) != 100;
	}

}
