/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.client;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.client.interceptor.OutgoingRequestInterceptor;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter;
import de.starface.com.rpc.services.RpcAgent;

/**
 * An RpcClient can be used to create RPC proxies and agents to interact with an RPC server and set
 * some default settings for the created instances.
 *
 * <p>
 * In order to call remote procedures it is essential to create a client-side proxy object. All
 * method calls to this proxy are then redirected to the remote server via RPC. The corresponding
 * server for each proxy is determined by a specific {@link RpcTransportToken} which defines the
 * transport simply by its type and the server address via this transport by its properties. For
 * convenience it is possible to create proxies with an {@link RpcAuthToken} when a corresponding
 * {@link RpcAuthTokenConverter} has been registered before.
 * </p>
 *
 * <p>
 * Furthermore this factory supports the concept of agents which provide a far higher level of
 * functionality than objects. Agents encapsulate a series of remote procedure calls in order to
 * provide higher level functionality.
 * </p>
 *
 * @author tilmann
 *
 * @param <TransportToken>
 *        the transport token type of this RpcClient depending on the underlying transport protocol
 */
public interface RpcClient<TransportToken extends RpcTransportToken> {

	/**
	 * Retrieve the default server transport token of this RpcClient.
	 *
	 * @return the default server transport token of this RpcClient
	 * @throws RpcException
	 *         if a problem occurs during construction of the default server transport token
	 */
	public TransportToken getDefaultServerTransportToken() throws RpcException;

	/**
	 * Sets the default server transport token used during proxy creation if no explicit server
	 * transport or auth token has been specified.
	 *
	 * @param defaultServerTransportToken
	 *        the new default server transport token to use
	 */
	public void setDefaultServerTransportToken(TransportToken defaultServerTransportToken);

	/**
	 * Retrieve the default server auth token of this RpcClient.
	 *
	 * @return the default server auth token of this RpcClient
	 */
	public RpcAuthToken getDefaultServerAuthToken();

	/**
	 * Sets the default server auth token used during proxy creation if no explicit server transport
	 * or auth token has been specified. A {@link RpcAuthTokenConverter} that can convert the auth
	 * token to a {@link RpcTransportToken} must also be registered with
	 * {@link #registerAuthTokenConverter(RpcAuthTokenConverter)} for this to work.
	 *
	 * @param defaultServerAuthToken
	 *        the new default server auth token to use
	 */
	public void setDefaultServerAuthToken(RpcAuthToken defaultServerAuthToken);

	/**
	 * Registers a {@link RpcAuthTokenConverter} for use to convert a auth token to the
	 * corresponding transport token when creating a proxy
	 *
	 * @param <AuthToken>
	 *        Type of the auth token
	 * @param converter
	 *        The {@link RpcAuthTokenConverter} to register
	 */
	public <AuthToken extends RpcAuthToken> void registerAuthTokenConverter(RpcAuthTokenConverter<TransportToken, AuthToken> converter);

	/**
	 * Removes a {@link RpcAuthTokenConverter} from the list of known converter
	 *
	 * @param <AuthToken>
	 *        Type of the auth token
	 * @param converter
	 *        The {@link RpcAuthTokenConverter} to unregister
	 */
	public <AuthToken extends RpcAuthToken> void unregisterAuthTokenConverter(RpcAuthTokenConverter<TransportToken, AuthToken> converter);

	/**
	 * Creates a new proxy instance of the given interface for the server represented by the given
	 * AuthToken.
	 *
	 * @param <RpcInterface>
	 *        Type the proxy should implement
	 * @param rpcInterface
	 *        The class of the interface to use for RPC-Calls
	 * @param serverTransportToken
	 *        the transport token of the server to connect to
	 * @return The created proxy instance
	 *
	 * @throws IllegalArgumentException
	 *         If one of the arguments is {@code null} or if the given interfaceClass is not an
	 *         interface
	 * @throws RpcException
	 *         If the required transport has not been registered for XML-RPC (code -2) or the RPC
	 *         proxy cannot be created for another reason (code -1) with more details available in
	 *         this exceptions cause.
	 */
	public <RpcInterface> RpcInterface createRpcProxy(Class<RpcInterface> rpcInterface,
													  TransportToken serverTransportToken)
		throws IllegalArgumentException, RpcException;

	/**
	 * Creates a new proxy instance of the given interface for the server represented by the given
	 * AuthToken.
	 *
	 * @param <RpcInterface>
	 *        Type the proxy should implement
	 * @param <AuthToken>
	 *        Type of the authentication used
	 * @param rpcInterface
	 *        The class of the interface to use for RPC-Calls
	 * @param serverAuthToken
	 *        The auth token of the server to connect to
	 * @return The created proxy instance
	 * @throws RpcException
	 *         If the required transport has not been registerd (code -2) for XML-RPC or an auth
	 *         token converter for the auth token type has not been registered (code -3) the
	 *         transport token for the given auth token can not be obtained from the auth token
	 *         converter (code -4) or the RPC proxy cannot be created for another reason (code -1)
	 *         with more details available in the exception's cause.
	 * @throws IllegalArgumentException
	 *         If one of the arguments is {@code null} or if the given interfaceClass is not an
	 *         interface
	 */
	public <RpcInterface, AuthToken extends RpcAuthToken> RpcInterface createRpcProxy(Class<RpcInterface> rpcInterface,
																					  AuthToken serverAuthToken)
		throws IllegalArgumentException, RpcException;

	/**
	 * Creates a new proxy instance of the given interface for the server represented by the default
	 * server auth token if present or default server transport token.
	 *
	 * @param <RpcInterface>
	 *        Type the proxy should implement
	 * @param rpcInterface
	 *        The class of the interface to use for RPC-Calls
	 * @return The created proxy instance
	 * @throws IllegalArgumentException
	 *         If one of the arguments is {@code null} or if the given interfaceClass is not an
	 *         interface
	 * @throws RpcException
	 *         If not default auth or transport token has been set (code -5), if the required
	 *         transport has not been registerd (code -2) for XML-RPC or an auth token converter for
	 *         the auth token type has not been registered (code -3) the transport token for the
	 *         given auth token can not be obtained from the auth token converter (code -4) or the
	 *         RPC proxy cannot be created for another reason (code -1) with more details available
	 *         in the exception's cause.
	 */
	public <RpcInterface> RpcInterface createRpcProxy(Class<RpcInterface> rpcInterface)
		throws IllegalArgumentException, RpcException;

	/**
	 * Creates a new agent instance of the class provided and initializes it.
	 *
	 * @param <RpcInterface>
	 *        Type of the processor the agent uses
	 * @param <X>
	 *        Type of the agent to create
	 * @param serviceName
	 *        Name of the corresponding service on the server. Used as prefix for remote procedure
	 *        (or ignored when {@code null})
	 * @param agentClass
	 *        The Class of the agent to create and initialize (must not be {@code null})
	 * @param serverTransportToken
	 *        A {@link RpcTransportToken} determining the corresponding XML-RPC server
	 * @return The created agent
	 * @throws RpcException
	 *         If the agent can not be instantiated (code -6) or if the required transport has not
	 *         been registered for XML-RPC (code -2) or the RPC agent cannot be created for another
	 *         reason (code -1) with more details available in this exceptions cause.
	 * @throws IllegalArgumentException
	 *         If agent class or server token are null
	 *
	 * @see RpcAgent
	 */
	public <RpcInterface, X extends RpcAgent<RpcInterface>> X createRpcAgent(String serviceName,
																			 Class<X> agentClass,
																			 TransportToken serverTransportToken)
		throws IllegalArgumentException, RpcException;

	/**
	 * Creates a new agent instance of the class provided and initializes it.
	 *
	 * @param <RpcInterface>
	 *        Type of the processor the agent uses
	 * @param <Agent>
	 *        Type of the agent to create
	 * @param <AuthToken>
	 * @param serviceName
	 *        Name of the corresponding service on the server. Used as prefix for remote procedure
	 *        (or ignored when {@code null})
	 * @param agentClass
	 *        The Class of the agent to create and initialize (must not be {@code null})
	 * @param serverAuthToken
	 *        A {@link RpcAuthToken} determining the corresponding XML-RPC server which is used to
	 *        resolve the corresponding transport token when a corresponding
	 *        {@link RpcAuthTokenConverter} is registered!
	 * @return The created agent
	 *
	 * @throws IllegalArgumentException
	 *         If agent class or server token are null
	 *
	 * @throws RpcException
	 *         If the agent can not be instantiated (code -6) or if the required transport has not
	 *         been registerd (code -2) for XML-RPC or an auth token converter for the auth token
	 *         type has not been registered (code -3) the transport token for the given auth token
	 *         can not be obtained from the auth token converter (code -4) or the RPC proxy cannot
	 *         be created for another reason (code -1) with more details available in the
	 *         exception's cause.
	 * @see RpcAgent
	 * @see RpcAuthTokenConverter
	 * @see #registerAuthTokenConverter(RpcAuthTokenConverter)
	 */
	public <RpcInterface, Agent extends RpcAgent<RpcInterface>, AuthToken extends RpcAuthToken> Agent createRpcAgent(String serviceName,
																													 Class<Agent> agentClass,
																													 AuthToken serverAuthToken)
		throws IllegalArgumentException, RpcException;

	/**
	 * Creates a new agent instance of the class provided and initializes it. For communication the
	 * default server auth token if present or default server transport token is used.
	 *
	 * @param <RpcInterface>
	 *        Type of the processor the agent uses
	 * @param <X>
	 *        Type of the agent to create
	 * @param serviceName
	 *        Name of the corresponding service on the server. Used as prefix for remote procedure
	 *        (or ignored when {@code null})
	 * @param agentClass
	 *        The Class of the agent to create and initialize (must not be {@code null})
	 * @return The created agent
	 * @throws IllegalArgumentException
	 *         If the given agentClass is null
	 * @throws RpcException
	 *         If not default auth or transport token has been set (code -5), if the agent can not
	 *         be instantiated (code -6) or if the required transport has not been registerd (code
	 *         -2) for XML-RPC or an auth token converter for the auth token type has not been
	 *         registered (code -3) the transport token for the given auth token can not be obtained
	 *         from the auth token converter (code -4) or the RPC proxy cannot be created for
	 *         another reason (code -1) with more details available in the exception's cause.
	 */
	public <RpcInterface, X extends RpcAgent<RpcInterface>> X createRpcAgent(String serviceName,
																			 Class<X> agentClass)
		throws IllegalArgumentException, RpcException;

	/**
	 * Adds the given {@link OutgoingRequestInterceptor} to the processing chain of all requests
	 * called through proxies/agents created by this RpcClient.
	 *
	 * @param interceptor
	 *        the interceptor to be added
	 */
	public void addRpcRequestInterceptor(OutgoingRequestInterceptor<? super TransportToken> interceptor);

	/**
	 * Removes the given {@link OutgoingRequestInterceptor} from the processing chain of all
	 * requests called through proxies/agents created by this RpcClient.
	 *
	 * @param interceptor
	 *        the interceptor to be removed
	 */
	public void removeRpcRequestInterceptor(OutgoingRequestInterceptor<? super TransportToken> interceptor);

	/**
	 * Waits till all pending requests called through proxies/agents created by this client have
	 * completed.
	 *
	 * @throws InterruptedException
	 *         if the current thread was interrupted while waiting
	 */
	public void waitForPendingRequests() throws InterruptedException;

	/**
	 * Waits till all pending requests called through proxies/agents created by this client have
	 * completed or the given time has passed - whatever occurs first.
	 *
	 * @param millis
	 *        the max time to wait
	 *
	 * @throws InterruptedException
	 *         if the current thread was interrupted while waiting
	 */
	public void waitForPendingRequests(long millis) throws InterruptedException;

	/**
	 * Sets the {@link RpcExceptionConverter} that is used to convert exceptions forwarded to the
	 * proxy users of this client.
	 *
	 * @param exceptionConverter
	 *        the converter to be used for exception conversion
	 */
	public void setExceptionConverter(RpcExceptionConverter<? super TransportToken> exceptionConverter);

	/**
	 * Retrieves the {@link RpcExceptionConverter} that is used to convert exceptions forwarded to
	 * the proxy users of this client.
	 *
	 * @return the converter to be used for exception conversion
	 */
	public RpcExceptionConverter<? super TransportToken> getExceptionConverter();
}
