/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.client;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.client.interceptor.OutgoingRequestInterceptor;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter;
import de.starface.com.rpc.services.RpcAgent;

import org.apache.commons.lang3.Validate;

import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * This is the base class for all {@link RpcClient} implementations.
 *
 * @author may, tilmann
 *
 * @param <TransportToken>
 *        the transport token type of this RpcClient depending on the underlying transport protocol
 */
public abstract class RpcClientBase<TransportToken extends RpcTransportToken> implements RpcClient<TransportToken> {
	private final AsynchronousExecutor asynchronousExecutor = new AsynchronousExecutor();

	private final Map<Class<? extends RpcAuthToken>, RpcAuthTokenConverter<TransportToken, ? extends RpcAuthToken>> _converter = new HashMap<>();

	private final List<OutgoingRequestInterceptor<? super TransportToken>> requestInterceptors = new ArrayList<>();

	private TransportToken defaultServerTransportToken = null;

	private RpcAuthToken defaultServerAuthToken = null;

	private RpcExceptionConverter<? super TransportToken> exceptionConverter;

	private final Set<OutgoingRequest<TransportToken>> currentRequests = new HashSet<>();

	private final RpcRequestCaller.OutgoingRequestEventListener<TransportToken> requestCallerEventListener = new RpcRequestCaller.OutgoingRequestEventListener<TransportToken>() {
		@Override
		public void requestCreated(OutgoingRequest<TransportToken> request) {
			addRequest(request);
		}

		@Override
		public void requestSending(OutgoingRequest<TransportToken> request) {
			// Nothing to do
		}

		@Override
		public void requestReturned(OutgoingRequest<TransportToken> request) {
			removeRequest(request);
		}

		@Override
		public void requestError(OutgoingRequest<TransportToken> request) {
			removeRequest(request);
		}

		@Override
		public void requestFinished(OutgoingRequest<TransportToken> request) {
			// Nothing to do
		}
	};

	/**
	 * This method may be overriden by subclasses to provide a predefined default server transport
	 * token. It is executed when a proxy is constructed and no server auth/transport token has been
	 * given and no default server auth/transport token has been set to create a default server
	 * transport token. This implementation simply returns <code>null</code> meaning the
	 * construction of the proxy will fail with a {@link RpcException} .
	 *
	 * @return a transport token that will be used as the default server transport in the future
	 * @throws RpcException
	 *         if a problem occurs during construction of the default server transport token
	 */
	protected TransportToken createDefaultServerTransportToken() throws RpcException {
		return null;
	}

	/**
	 * Retrieve the default server transport token of this RPC Client trying to create one with the
	 * method {@link #createDefaultServerTransportToken()} if none has been set before.
	 *
	 * @see de.starface.com.rpc.client.RpcClient#getDefaultServerTransportToken()
	 */
	@Override
	public TransportToken getDefaultServerTransportToken() throws RpcException {
		// Implemented through double checked singleton gives a certain security
		// that createDefaultServerTransportToken() will not be executed twice
		// although this cannot be guaranteed because of the nature of java byte code

		if (defaultServerTransportToken == null) {
			synchronized (this) {
				if (defaultServerTransportToken == null) {
					defaultServerTransportToken = createDefaultServerTransportToken();
				}
			}
		}
		return defaultServerTransportToken;
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#setDefaultServerTransportToken(de.starface.com.rpc.common.RpcTransportToken)
	 */
	@Override
	public void setDefaultServerTransportToken(TransportToken defaultServerTransportToken) {
		this.defaultServerTransportToken = defaultServerTransportToken;
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#getDefaultServerAuthToken()
	 */
	@Override
	public RpcAuthToken getDefaultServerAuthToken() {
		return defaultServerAuthToken;
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#setDefaultServerAuthToken(de.starface.com.rpc.common.authentication.RpcAuthToken)
	 */
	@Override
	public void setDefaultServerAuthToken(RpcAuthToken defaultServerAuthToken) {
		this.defaultServerAuthToken = defaultServerAuthToken;
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#registerAuthTokenConverter(de.starface.com.rpc.common.authentication.RpcAuthTokenConverter)
	 */
	@Override
	public <AuthToken extends RpcAuthToken> void registerAuthTokenConverter(RpcAuthTokenConverter<TransportToken, AuthToken> converter) {
		_converter.put(converter.getAuthType(), converter);
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#unregisterAuthTokenConverter(de.starface.com.rpc.common.authentication.RpcAuthTokenConverter)
	 */
	@Override
	public <AuthToken extends RpcAuthToken> void unregisterAuthTokenConverter(RpcAuthTokenConverter<TransportToken, AuthToken> converter) {
		_converter.remove(converter.getAuthType());
	}

	/**
	 * Creates a new proxy instance of a given interface. This proxy instance will call remote
	 * methods on a XML-RPC server which is identified by the given TransportToken.
	 *
	 * @param <RpcInterface>
	 *        Interface the proxy implements
	 * @param <T>
	 *        Type of the used transport token
	 * @param serviceName
	 *        The name used as prefix for the RPC calls
	 * @param proxyInterface
	 *        A interface the proxy should implement
	 * @param serverTransportToken
	 *        A TransportToken to identify the server method calls are redirected to
	 * @param requestCaller
	 *        The matching {@link OutgoingRequestProcessor} for the used transport Token
	 * @return A proxy instance of the given interface
	 * @throws ProxyCreationFailedException
	 *         with attached cause if the proxy could not be created
	 * @throws IllegalArgumentException
	 *         if the given proxyInterface class is not an interface
	 */
	@SuppressWarnings("unchecked")
	private <RpcInterface> RpcInterface createRpcProxy(String serviceName, Class<RpcInterface> rpcInterface,
													   TransportToken serverTransportToken)
		throws IllegalArgumentException, RpcException {
		if (!rpcInterface.isInterface()) {
			throw new IllegalArgumentException(rpcInterface.getName() + " is not an interface!");
		}

		try {
			ClassLoader classLoader = rpcInterface.getClassLoader();
			if (classLoader == null) {
				classLoader = Thread.currentThread().getContextClassLoader();
			}

			RpcRequestCaller<TransportToken> rpcRequestCaller = new RpcRequestCaller<>(this,
				serviceName, serverTransportToken, getRequestProcessorFactory(), rpcInterface);

			rpcRequestCaller.subscribeEvents(requestCallerEventListener);

			return (RpcInterface) Proxy.newProxyInstance(classLoader, new Class<?>[]{rpcInterface},
				rpcRequestCaller);
		} catch (Exception e) {
			throw new RpcException(RpcException.Codes.Client.PROXY_CREATION_FAILED,
				"Could not create proxy Instance for interface " + rpcInterface.getName(), e);
		}
	}

	private OutgoingRequestProcessorFactory<TransportToken> getRequestProcessorFactory() {
		return new OutgoingRequestProcessorFactory<TransportToken>() {
			@Override
			public OutgoingRequestProcessor<TransportToken> createOutgoingRequestProcessor() {
				return RpcClientBase.this.createOutgoingRequestProcessor();
			}
		};
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#createRpcProxy(Class,
	 *      de.starface.com.rpc.common.RpcTransportToken)
	 */
	@Override
	public <RpcInterface> RpcInterface createRpcProxy(Class<RpcInterface> rpcInterface,
													  TransportToken serverTransportToken)
		throws IllegalArgumentException, RpcException {
		Validate.notNull(rpcInterface, "rpcInterface=null");
		Validate.notNull(serverTransportToken, "serverTransportToken=null");

		return createRpcProxy(null, rpcInterface, serverTransportToken);
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#createRpcProxy(Class,
	 *      de.starface.com.rpc.common.authentication.RpcAuthToken)
	 */
	@Override
	public <RpcInterface, AuthToken extends RpcAuthToken> RpcInterface createRpcProxy(Class<RpcInterface> rpcInterface,
																					  AuthToken serverAuthToken)
		throws IllegalArgumentException, RpcException {
		Validate.notNull(rpcInterface, "rpcInterface=null");
		Validate.notNull(serverAuthToken, "serverAuthToken=null");

		TransportToken serverTransportToken = convertAuthToken(serverAuthToken);

		return createRpcProxy(rpcInterface, serverTransportToken);
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#createRpcProxy(Class)
	 */
	@Override
	public <RpcInterface> RpcInterface createRpcProxy(Class<RpcInterface> rpcInterface)
		throws IllegalArgumentException, RpcException {
		if (defaultServerAuthToken != null) {
			return createRpcProxy(rpcInterface, defaultServerAuthToken);
		}

		TransportToken defaultServerTransportToken = getDefaultServerTransportToken();
		if (defaultServerTransportToken != null) {
			return createRpcProxy(rpcInterface, defaultServerTransportToken);
		}

		throw new RpcException(RpcException.Codes.Client.DEFAULT_SERVER_TOKEN_MISSING,
			"No defaultServerAuthToken or no defaultServerTransportToken have been set.");
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#createRpcAgent(String, Class,
	 *      de.starface.com.rpc.common.RpcTransportToken)
	 */
	@Override
	public <RpcInterface, X extends RpcAgent<RpcInterface>> X createRpcAgent(String serviceName,
																			 Class<X> agentClass,
																			 TransportToken serverTransportToken)
		throws IllegalArgumentException, RpcException

	{
		Validate.notNull(agentClass, "agent must not be null");
		Validate.notNull(serverTransportToken, "serverTransportToken=null");

		X agent;
		try {
			agent = agentClass.newInstance();
		} catch (Exception e) {
			throw new RpcException(RpcException.Codes.Client.AGENT_CREATION_FAILED, "Cannot instantiate agent", e);
		}

		final RpcInterface rpcProxy = createRpcProxy(serviceName, agent.getProxyType(), serverTransportToken);

		agent.setProxy(rpcProxy);

		return agent;
	}

	/**
	 * This method is to be implemented by subclasses to provide the request processors for their
	 * transport protocol implementation.
	 *
	 * @return a request processor that can be used to process outgoing requests
	 */
	protected abstract OutgoingRequestProcessor<TransportToken> createOutgoingRequestProcessor();

	/**
	 * @see de.starface.com.rpc.client.RpcClient#createRpcAgent(String, Class,
	 *      de.starface.com.rpc.common.authentication.RpcAuthToken)
	 */
	@Override
	public <RpcInterface, Agent extends RpcAgent<RpcInterface>, AuthToken extends RpcAuthToken> Agent createRpcAgent(String serviceName,
																													 Class<Agent> agentClass,
																													 AuthToken serverAuthToken)
		throws IllegalArgumentException, RpcException {
		Validate.notNull(agentClass, "agentClass=null");
		Validate.notNull(serverAuthToken, "serverAuthToken=null");

		TransportToken serverTransportToken = convertAuthToken(serverAuthToken);

		return this.createRpcAgent(serviceName, agentClass, serverTransportToken);
	}

	private <AuthToken extends RpcAuthToken> TransportToken convertAuthToken(AuthToken authToken)
		throws RpcException {
		TransportToken transportToken;
		@SuppressWarnings("unchecked")
		RpcAuthTokenConverter<TransportToken, AuthToken> converter = (RpcAuthTokenConverter<TransportToken, AuthToken>) _converter.get(authToken.getClass());

		if (converter != null) {
			try {
				transportToken = converter.convert(authToken);
			} catch (Exception e) {
				throw new RpcException(RpcException.Codes.Client.AUTH_CONVERTER_EXCEPTION,
					"The registered auth token converter threw an exception during execution.", e);
			}
		} else {
			throw new RpcException(RpcException.Codes.Client.AUTH_CONVERTER_MISSING,
				"Auth token converter not registered for the server auth token type "
					+ authToken.getClass().getCanonicalName());
		}
		if (transportToken == null) {
			throw new RpcException(RpcException.Codes.Client.AUTH_UNKNOWN,
				"The registered auth token converter did not convert the server auth token.");
		}

		return transportToken;
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#createRpcAgent(String, Class)
	 */
	@Override
	public <RpcInterface, X extends RpcAgent<RpcInterface>> X createRpcAgent(String serviceName,
																			 Class<X> agentClass)
		throws IllegalArgumentException, RpcException {
		if (defaultServerAuthToken != null) {
			return createRpcAgent(serviceName, agentClass, defaultServerAuthToken);
		}

		TransportToken defaultServerTransportToken = getDefaultServerTransportToken();
		if (defaultServerTransportToken != null) {
			return createRpcAgent(serviceName, agentClass, defaultServerTransportToken);
		}

		throw new RpcException(RpcException.Codes.Client.DEFAULT_SERVER_TOKEN_MISSING,
			"No defaultServerAuthToken or no defaultServerTransportToken have been set.");
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#addRpcRequestInterceptor(de.starface.com.rpc.client.interceptor.OutgoingRequestInterceptor)
	 */
	@Override
	public void addRpcRequestInterceptor(OutgoingRequestInterceptor<? super TransportToken> interceptor) {
		if (!requestInterceptors.contains(interceptor)) {
			requestInterceptors.add(0, interceptor);
		}
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#removeRpcRequestInterceptor(de.starface.com.rpc.client.interceptor.OutgoingRequestInterceptor)
	 */
	@Override
	public void removeRpcRequestInterceptor(OutgoingRequestInterceptor<? super TransportToken> interceptor) {
		requestInterceptors.remove(interceptor);
	}

	public List<OutgoingRequestInterceptor<? super TransportToken>> getRequestInterceptors() {
		return requestInterceptors;
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#getExceptionConverter()
	 */
	@Override
	public RpcExceptionConverter<? super TransportToken> getExceptionConverter() {
		return exceptionConverter;
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#setExceptionConverter(de.starface.com.rpc.client.RpcExceptionConverter)
	 */
	@Override
	public void setExceptionConverter(RpcExceptionConverter<? super TransportToken> exceptionConverter) {
		this.exceptionConverter = exceptionConverter;
	}

	private void addRequest(OutgoingRequest<TransportToken> request) {
		synchronized (currentRequests) {
			currentRequests.add(request);
		}
	}

	private void removeRequest(OutgoingRequest<TransportToken> request) {
		synchronized (currentRequests) {
			currentRequests.remove(request);
			if (currentRequests.isEmpty()) {
				currentRequests.notifyAll();
			}
		}
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#waitForPendingRequests()
	 */
	@Override
	public void waitForPendingRequests() throws InterruptedException {
		synchronized (currentRequests) {
			while (!currentRequests.isEmpty()) {
				currentRequests.wait();
			}
		}
	}

	/**
	 * @see de.starface.com.rpc.client.RpcClient#waitForPendingRequests(long)
	 */
	@Override
	public void waitForPendingRequests(long millis) throws InterruptedException {
		synchronized (currentRequests) {
			if (!currentRequests.isEmpty()) {
				currentRequests.wait(millis);
			}
		}
	}

	public AsynchronousExecutor getAsynchronousExecutor() {
		return asynchronousExecutor;
	}
}
