/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.client;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.RpcRuntimeException;
import de.starface.com.rpc.client.interceptor.AsynchronousRequestInterceptor;
import de.starface.com.rpc.client.interceptor.LoggingRequestInterceptor;
import de.starface.com.rpc.client.interceptor.OutgoingRequestInterceptor;
import de.starface.com.rpc.client.interceptor.ValueTranslationRequestInterceptor;
import de.starface.com.rpc.common.ReflectionUtils;
import de.starface.com.rpc.common.RpcRequest;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.events.EventsProviderSupport;
import de.starface.com.rpc.common.interceptor.RequestInterceptorChain;
import de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink;
import de.starface.com.rpc.services.filetransfer.FileTransferEventListener;

import org.apache.commons.lang3.Validate;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * The RpcRequestCaller contains the pre and post processing logic for outgoing RPC calls. It
 * creates the outgoing RPC request, sets up and invokes the request intercepter chain followed by
 * the actual request processor. Afterwards it performs exception handling and returns the RPC
 * results to the client implementation.
 *
 * This class also provides the possibility to influence the call processing by adding/removing
 * request interceptors and setting an exception converter and to add listeners to be informed about
 * the events of {@link OutgoingRequestEventListener} and {@link FileTransferEventListener}. To do
 * one of these the client implementation must first obtain the RpcRequestCaller instance in use by
 * invoking {@link RpcRequestCaller#getRequestCallerFromProxy(Object)} for their proxy object.
 *
 * @author tilmann
 *
 * @param <TransportToken>
 */
public class RpcRequestCaller<TransportToken extends RpcTransportToken> implements InvocationHandler {
	private static final OutgoingRequestInterceptor<RpcTransportToken> loggingRequestInterceptor = new LoggingRequestInterceptor();

	@SuppressWarnings({"unchecked", "rawtypes"})
	private final EventsProviderSupport<OutgoingRequestEventListener<TransportToken>> eventsSupport = new EventsProviderSupport(
		OutgoingRequestEventListener.class);

	private final TransportToken targetTransportToken;
	private final String rpcNamePrefix;
	private final OutgoingRequestProcessorFactory<TransportToken> processorFactory;
	private final Class<?> rpcInterface;
	private final RpcClientBase<TransportToken> rpcClient;
	private final List<OutgoingRequestInterceptor<? super TransportToken>> requestInterceptors = new ArrayList<>();
	private final EventsRequestInterceptor eventsRequestInterceptor = new EventsRequestInterceptor();

	private final EventsProviderSupport<FileTransferEventListener> downloadEventsSupport = new EventsProviderSupport<>(
		FileTransferEventListener.class);
	private final EventsProviderSupport<FileTransferEventListener> uploadEventsSupport = new EventsProviderSupport<>(
		FileTransferEventListener.class);

	private RpcExceptionConverter<? super TransportToken> exceptionConverter;

	public RpcRequestCaller(RpcClientBase<TransportToken> rpcClient, String rpcNamePrefix,
							TransportToken targetTransportToken,
							OutgoingRequestProcessorFactory<TransportToken> processorFactory, Class<?> rpcInterface) {
		Validate.notNull(rpcClient, "rpcClient=null");
		Validate.notNull(targetTransportToken, "targetTransportToken=null");
		Validate.notNull(processorFactory, "processorFactory=null");
		Validate.notNull(rpcInterface, "rpcInterface=null");
		this.targetTransportToken = targetTransportToken;
		this.rpcNamePrefix = rpcNamePrefix;
		this.processorFactory = processorFactory;
		this.rpcInterface = rpcInterface;
		this.rpcClient = rpcClient;
	}

	/**
	 * This method is invoked from the proxies generated by the corresponding RpcClient
	 *
	 * @see InvocationHandler#invoke(Object, Method,
	 *      Object[])
	 */
	@Override
	public Object invoke(Object proxy, Method rpcMethod, Object[] prarameters) throws Throwable {
		// redirect object calls to the local VM
		if (rpcMethod.getDeclaringClass().equals(Object.class)) {
			return rpcMethod.invoke(proxy, prarameters);
		}

		OutgoingRequest<TransportToken> request =
				OutgoingRequest.createRequestWithPrefix(rpcNamePrefix, rpcInterface, rpcMethod, prarameters,
						targetTransportToken, rpcClient);
		try {
			eventsSupport.getEventsDistributor().requestCreated(request);

			RequestInterceptorChain<OutgoingRequest<TransportToken>> interceptorChain =
					prepareRequestInterceptorChain(request);

			interceptorChain.processRequest();
			validateReturnValueType(request);
			eventsSupport.getEventsDistributor().requestFinished(request);
			return request.getReturnValue();
		} catch (Exception originalException) {
			eventsSupport.getEventsDistributor().requestError(request);
			Throwable toBeThrown = unpackApplicationException(originalException, rpcMethod);
			toBeThrown = invokeExceptionConverter(toBeThrown, request);

			if (!canBeThrown(toBeThrown, rpcMethod)) {
				if (canBeThrown(originalException, rpcMethod)) {
					toBeThrown = originalException;
				} else {
					toBeThrown = createRpcRuntimeException(originalException, toBeThrown);
				}
			}
			throw toBeThrown;
		}
	}

	@SuppressWarnings({"unchecked", "rawtypes"})
	private RequestInterceptorChain<OutgoingRequest<TransportToken>> prepareRequestInterceptorChain(OutgoingRequest<TransportToken> request)
		throws RpcException {
		return new RequestInterceptorChain<>(request,
			processorFactory.createOutgoingRequestProcessor(), (List) prepareRequestInterceptors(request));
	}

	private Throwable invokeExceptionConverter(Throwable original, OutgoingRequest<TransportToken> request) {
		RpcExceptionConverter<TransportToken> converter = findExceptionConverter();
		Throwable converted = original;

		if (converter != null) {
			try {
				converted = converter.convertExceptionToBeThrownOnProxy(original, request);

				if (converted == null) {
					converted = new RpcException(RpcException.Codes.Client.EXCEPTION_CONVERTER_ERROR,
						"RpcExceptionConverter returned null instead of a valid Throwable.", original);
				} else if (!canBeThrown(converted, request.getRpcMethod())) {
					converted = new RpcException(RpcException.Codes.Client.EXCEPTION_CONVERTER_ERROR,
						"Throwable returned by RpcExceptionConverter cannot be thrown from proxy method.",
						converted);
				}
			} catch (Exception e) {
				converted = new RpcException(RpcException.Codes.Client.EXCEPTION_CONVERTER_ERROR,
					"RpcExceptionConverter did throw an exception.", original);
			}
		}
		return converted;
	}

	private Throwable createRpcRuntimeException(Exception original, Throwable toBeThrown) {
		if (toBeThrown instanceof RpcException) {
			toBeThrown = new RpcRuntimeException((RpcException) toBeThrown);
		}
		if (original instanceof RpcException) {
			toBeThrown = new RpcRuntimeException((RpcException) original);
		}
		return toBeThrown;
	}

	private List<OutgoingRequestInterceptor<? super TransportToken>> prepareRequestInterceptors(OutgoingRequest<TransportToken> request)
		throws RpcException {
		List<OutgoingRequestInterceptor<? super TransportToken>> interceptors = new LinkedList<>();
		interceptors.addAll(rpcClient.getRequestInterceptors());
		interceptors.addAll(requestInterceptors);
		interceptors.add(AsynchronousRequestInterceptor.createInterceptorForRequest(request));
		interceptors.add(ValueTranslationRequestInterceptor.createInterceptorForRequest(request,
			downloadEventsSupport.getEventsDistributor(), uploadEventsSupport.getEventsDistributor()));
		interceptors.add(eventsRequestInterceptor);
		interceptors.add(loggingRequestInterceptor);
		return interceptors;
	}

	private static Throwable unpackApplicationException(Exception e, Method rpcMethod) throws Throwable {
		if (e instanceof RpcException) {
			RpcException rpcEx = (RpcException) e;

			// If application exception try to throw cause
			if (rpcEx.getCode() == RpcException.Codes.Server.APPLICATION_EXCEPTION
				|| rpcEx.getCode() == RpcException.Codes.Server.PREPARED_CLIENT_THROWABLE
				|| rpcEx.getCode() == RpcException.Codes.Client.PREPARED_CLIENT_THROWABLE) {
				Throwable t = rpcEx.getCause();
				if (t != null) {
					return t;
				}
			}
		}
		return e;
	}

	private static boolean canBeThrown(Throwable t, Method method) {
		if (t instanceof RuntimeException) {
			return true;
		}
		Class<? extends Throwable> tClass = t.getClass();
		for (Class<?> c : method.getExceptionTypes()) {
			if (c.isAssignableFrom(tClass)) {
				return true;
			}
		}
		return false;
	}

	private static void validateReturnValueType(RpcRequest<?> request) throws RpcException {
		Class<?> exptectedReturnValueClass = ReflectionUtils.getClassOfType(request.getRpcMethod().getGenericReturnType());

		if (request.getReturnValue() == null) {
			if (exptectedReturnValueClass.isPrimitive() && exptectedReturnValueClass != void.class) {
				throw new RpcException(RpcException.Codes.Client.RETURN_TYPE_MISMATCH,
					"RPC server returned null instead of a primitive of type: "
						+ exptectedReturnValueClass.getName());
			}
		} else {
			Class<?> returnValueClass = request.getReturnValue().getClass();

			if (!ReflectionUtils.isActualClassCompatibleToExpectedClass(exptectedReturnValueClass,
				returnValueClass)) {
				throw new RpcException(RpcException.Codes.Client.RETURN_TYPE_MISMATCH,
					"RPC server returned wrong result type: " + returnValueClass.getName());
			}
		}
	}

	/**
	 * Adds the given {@link OutgoingRequestInterceptor} to the processing chain of all requests
	 * called through this request caller.
	 *
	 * @param interceptor
	 *        the interceptor to be added
	 */
	public void addRpcRequestInterceptor(OutgoingRequestInterceptor<? super TransportToken> interceptor) {
		Validate.notNull(interceptor, "interceptor=null");
		if (!requestInterceptors.contains(interceptor)) {
			requestInterceptors.add(0, interceptor);
		}
	}

	/**
	 * Removes the given {@link OutgoingRequestInterceptor} from the processing chain of all
	 * requests called through this request caller.
	 *
	 * @param interceptor
	 *        the interceptor to be added
	 */
	public void removeRpcRequestInterceptor(OutgoingRequestInterceptor<? super TransportToken> interceptor) {
		Validate.notNull(interceptor, "interceptor=null");
		requestInterceptors.remove(interceptor);
	}

	/**
	 * Retrieves the RpcRequestCaller associated with the given RPC proxy object.
	 *
	 * @param rpcProxy
	 *        an RPC proxy instance generated by an {@link RpcClient}
	 * @return the RpcRequestCaller associated with the given RPC proxy
	 * @throws IllegalArgumentException
	 *         if the given object is not an RPC proxy generated by an {@link RpcClient}
	 *         implementation
	 */
	public static RpcRequestCaller<?> getRequestCallerFromProxy(Object rpcProxy) throws IllegalArgumentException {
		Validate.notNull(rpcProxy, "rpcProxy=null");
		InvocationHandler invocationHandler = Proxy.getInvocationHandler(rpcProxy);
		if (!(invocationHandler instanceof RpcRequestCaller)) {
			throw new IllegalArgumentException("The given object is not an RPC proxy.");
		}
		return (RpcRequestCaller<?>) invocationHandler;
	}

	public RpcExceptionConverter<? super TransportToken> getExceptionConverter() {
		return exceptionConverter;
	}

	public void setExceptionConverter(RpcExceptionConverter<? super TransportToken> exceptionConverter) {
		this.exceptionConverter = exceptionConverter;
	}

	@SuppressWarnings("unchecked")
	private RpcExceptionConverter<TransportToken> findExceptionConverter() {
		if (exceptionConverter != null) {
			return (RpcExceptionConverter<TransportToken>) exceptionConverter;
		}
		return (RpcExceptionConverter<TransportToken>) rpcClient.getExceptionConverter();
	}

	/**
	 * This interface defines a listener that can be sent events about ongoing RPC requests
	 *
	 * @author tilmann
	 *
	 * @param <TransportToken>
	 *        the transport token type of the requests
	 */
	public interface OutgoingRequestEventListener<TransportToken extends RpcTransportToken> {
		/**
		 * A new request has been created.
		 *
		 * @param request
		 *        the newly created request.
		 */
		public void requestCreated(OutgoingRequest<TransportToken> request);

		/**
		 * The request has finished preprocessing and is now being sent to the RPC server
		 *
		 * @param request
		 *        the request being sent
		 */
		public void requestSending(OutgoingRequest<TransportToken> request);

		/**
		 * The request has returned from RPC server and is now starting post processing
		 *
		 * @param request
		 *        the returned request
		 */
		public void requestReturned(OutgoingRequest<TransportToken> request);

		/**
		 * Postprocessing of the request finished and the result is now being returned to the proxy
		 * caller
		 *
		 * @param request
		 *        the finished request
		 */
		public void requestFinished(OutgoingRequest<TransportToken> request);

		/**
		 * An error has occurred during processing of the request.
		 *
		 * @param request
		 *        the erroneous request
		 */
		public void requestError(OutgoingRequest<TransportToken> request);
	}

	/**
	 * Add an OutgoingRequestEventListener to this request callers listeners
	 *
	 * @param listener
	 *        the listener to be added
	 */
	public void subscribeEvents(OutgoingRequestEventListener<TransportToken> listener) {
		eventsSupport.subscribeEvents(listener);
	}

	/**
	 * Remove an OutgoingRequestEventListener from this request callers listeners
	 *
	 * @param listener
	 *        the listener to be removed
	 */
	public void unsubscribeEvents(OutgoingRequestEventListener<TransportToken> listener) {
		eventsSupport.unsubscribeEvents(listener);
	}

	/**
	 * Add an FileTransferEventListener to this request callers listeners
	 *
	 * @param listener
	 *        the listener to be added
	 */
	public void subscribeAutomaticFileUploadEvents(FileTransferEventListener listener) {
		uploadEventsSupport.subscribeEvents(listener);
	}

	/**
	 * Remove an FileTransferEventListener from this request callers listeners
	 *
	 * @param listener
	 *        the listener to be removed
	 */
	public void unsubscribeAutomaticFileUploadEvents(FileTransferEventListener listener) {
		uploadEventsSupport.unsubscribeEvents(listener);
	}

	/**
	 * Add an FileTransferEventListener to this request callers listeners
	 *
	 * @param listener
	 *        the listener to be added
	 */
	public void subscribeAutomaticFileDownloadEvents(FileTransferEventListener listener) {
		downloadEventsSupport.subscribeEvents(listener);
	}

	/**
	 * Remove an FileTransferEventListener from this request callers listeners
	 *
	 * @param listener
	 *        the listener to be removed
	 */
	public void unsubscribeAutomaticFileDownloadEvents(FileTransferEventListener listener) {
		downloadEventsSupport.unsubscribeEvents(listener);
	}

	private class EventsRequestInterceptor implements OutgoingRequestInterceptor<TransportToken> {
		@Override
		public void interceptRequest(OutgoingRequest<TransportToken> request,
									 RequestInterceptorChainLink nextChainLink) throws RpcException {
			eventsSupport.getEventsDistributor().requestSending(request);
			try {
				nextChainLink.processRequest();
			} finally {
				eventsSupport.getEventsDistributor().requestReturned(request);
			}
		}
	}
}
