/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.client.interceptor;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.client.AsynchronousExecutor;
import de.starface.com.rpc.client.OutgoingRequest;
import de.starface.com.rpc.common.ReflectionUtils;
import de.starface.com.rpc.common.RpcRequest;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.lang.reflect.Method;

/**
 * This request interceptor implements the aspect of asynchronous RPC requests.
 *
 * @author tilmann
 */
public class AsynchronousRequestInterceptor implements OutgoingRequestInterceptor<RpcTransportToken> {
	private static final Logger log = LogManager.getLogger(AsynchronousRequestInterceptor.class);

	private AsynchronousExecutor executor;

	private AsynchronousRequestInterceptor(AsynchronousExecutor executor) {
		this.executor = executor;
	}

	public static OutgoingRequestInterceptor<RpcTransportToken> createInterceptorForRequest(OutgoingRequest<? extends RpcTransportToken> request) {
		boolean asynchronous = getAsynchronousForMethod(request.getRpcInterface(), request.getRpcMethod());

		if (!asynchronous) {
			return NoOperationRequestInterceptor.INSTANCE;
		}

		return new AsynchronousRequestInterceptor(request.getRpcClient().getAsynchronousExecutor());
	}

	/**
	 * @see de.starface.com.rpc.common.interceptor.RpcRequestInterceptor#interceptRequest(de.starface.com.rpc.common.RpcRequest,
	 *      de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink)
	 */
	@Override
	public void interceptRequest(OutgoingRequest<RpcTransportToken> request,
								 final RequestInterceptorChainLink interceptorChain) throws RpcException {
		assertReturnTypeIsVoid(request);

		Runnable runnable = () -> {
			try {
				interceptorChain.processRequest();
			} catch (Throwable t) {
				if (request != null) {
					log.info("processChainAsynchronously: Error during asynchronous RPC request(" +
						request.getTargetTransportToken() + "/" + request.getRpcMethodName() + ")." + t.getMessage());
				} else {
					log.info("processChainAsynchronously: Error during asynchronous RPC request." + t.getMessage());
				}
			}
		};
		executor.execute(runnable, request.getTargetTransportToken());
	}

	private static void assertReturnTypeIsVoid(RpcRequest<?> request) throws RpcException {
		if (!void.class.equals(request.getRpcMethod().getGenericReturnType())) {
			throw new RpcException(RpcException.Codes.Client.RETURN_TYPE_INVALID, "The asynchronous procedure "
				+ request.getRpcMethod().getName() + " of interface " + request.getRpcInterface().getName()
				+ " must have a return type of void.");
		}
	}

	private static boolean getAsynchronousForMethod(Class<?> rpcInterface, Method rpcMethod) {
		RpcAsynchronous annotation = ReflectionUtils.findRpcAnnotation(RpcAsynchronous.class, rpcInterface,
			rpcMethod);
		if (annotation != null) {
			return annotation.value();
		} else {
			return RpcAsynchronous.DEFAULT_BEHAVIOUR;
		}
	}
}
