/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.common;

import de.starface.com.rpc.annotation.RpcAuthRequired;
import de.starface.com.rpc.common.authentication.RpcAuthToken;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

/**
 * This class contains some utility methods used to perform reflection operations on java types
 *
 * @author tilmann
 */
public class ReflectionUtils {
	private ReflectionUtils() {
		// No instances will be made
	}

	public static Class<?> getClassOfType(Type type) {
		if (type instanceof Class) {
			return (Class<?>) type;
		}
		if (type instanceof ParameterizedType) {
			Type rawType = ((ParameterizedType) type).getRawType();
			if (rawType instanceof Class) {
				return (Class<?>) rawType;
			}
		}
		throw new IllegalArgumentException("The raw type of given parameterized type " + type + " is not a class.");
	}

	/**
	 * Checks if two given types are compatible i.e. are assignable. Checks equality, autoboxing
	 * behavior and assignability. Note: this method is not transitive!
	 *
	 * @param expected
	 *        The type to which is assigned to
	 * @param actual
	 *        The type to assign
	 * @return true if the type on the left side can be used instead of the type on the right side
	 */
	public static boolean isActualClassCompatibleToExpectedClass(Class<?> expected, Class<?> actual) {
		// if void is expected nothing is returned
		if (expected.equals(void.class)) {
			return true;
		}

		// simple equality
		if (expected.equals(actual)) {
			return true;
		}

		// right site can be assigned to left side
		if (expected.isAssignableFrom(actual)) {
			return true;
		}

		// left side primitive, right side not
		if (expected.equals(int.class) && actual.equals(Integer.class)) {
			return true;
		}
		if (expected.equals(boolean.class) && actual.equals(Boolean.class)) {
			return true;
		}
		if (expected.equals(float.class) && actual.equals(Float.class)) {
			return true;
		}
		if (expected.equals(double.class) && actual.equals(Double.class)) {
			return true;
		}
		if (expected.equals(long.class) && actual.equals(Long.class)) {
			return true;
		}
		if (expected.equals(byte.class) && actual.equals(Byte.class)) {
			return true;
		}
		if (expected.equals(short.class) && actual.equals(Short.class)) {
			return true;
		}
		if (expected.equals(char.class) && actual.equals(Character.class)) {
			return true;
		}

		// other way around
		if (expected.equals(Integer.class) && actual.equals(int.class)) {
			return true;
		}
		if (expected.equals(Boolean.class) && actual.equals(boolean.class)) {
			return true;
		}
		if (expected.equals(Float.class) && actual.equals(float.class)) {
			return true;
		}
		if (expected.equals(Double.class) && actual.equals(double.class)) {
			return true;
		}
		if (expected.equals(Long.class) && actual.equals(long.class)) {
			return true;
		}
		if (expected.equals(Byte.class) && actual.equals(byte.class)) {
			return true;
		}
		if (expected.equals(Short.class) && actual.equals(short.class)) {
			return true;
		}
		if (expected.equals(Character.class) && actual.equals(char.class)) {
			return true;
		}

		return false;
	}

	public static <AnnotationType extends Annotation> AnnotationType findRpcAnnotation(Class<AnnotationType> annotationClass,
																					   Class<?> rpcInterface,
																					   Method rpcProcedure) {
		AnnotationType annotation = findRpcAnnotationFromMethod(annotationClass, rpcProcedure);
		if (annotation != null) {
			return annotation;
		} else {
			return findRpcAnnotationFromInterface(annotationClass, rpcInterface);
		}
	}

	public static <AnnotationType extends Annotation> AnnotationType findRpcAnnotationFromMethod(Class<AnnotationType> annotationClass,
																								 Method rpcMethod) {
		if (rpcMethod == null) {
			return null;
		}

		if (!rpcMethod.isAnnotationPresent(annotationClass)) {
			return null;
		}

		return rpcMethod.getAnnotation(annotationClass);
	}

	public static <AnnotationType extends Annotation> AnnotationType findRpcAnnotationFromInterface(Class<AnnotationType> annotationClass,
																									Class<?> rpcInterface) {
		if (rpcInterface == null) {
			return null;
		}

		if (!rpcInterface.isAnnotationPresent(annotationClass)) {
			return null;
		}

		return rpcInterface.getAnnotation(annotationClass);
	}

	public static Class<? extends RpcAuthToken> determineAuthTokenTypeOfRpcInstanceOrInterface(Object rpcInstance,
																							   Class<?> rpcInterface) {
		Class<? extends RpcAuthToken> authTokenType = null;

		authTokenType = determineAuthTokenTypeOfClass(rpcInstance.getClass());

		if (authTokenType == null) {
			authTokenType = determineAuthTokenTypeOfClass(rpcInterface);
		}

		return authTokenType;
	}

	private static Class<? extends RpcAuthToken> determineAuthTokenTypeOfClass(Class<?> annotatedClass) {
		RpcAuthRequired annotation = annotatedClass.getAnnotation(RpcAuthRequired.class);

		if (annotation != null) {
			return annotation.value();
		} else {
			return null;
		}
	}

	public static Class<? extends RpcAuthToken> determineAuthTokenTypeOfMethod(Method rpcMethod) {
		RpcAuthRequired annotation = rpcMethod.getAnnotation(RpcAuthRequired.class);
		if (annotation != null) {
			return annotation.value();
		} else {
			return null;
		}
	}
}
