/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.common;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.lang.Thread.UncaughtExceptionHandler;
import java.util.concurrent.ThreadFactory;

/**
 * A thread factory that spawn all new threads created by this RPC API
 *
 * @author may
 */
public class RpcThreadFactory implements ThreadFactory {
	private static final Logger log = LogManager.getLogger(RpcThreadFactory.class);

	private final ThreadGroup group;
	private final UncaughtExceptionHandler exceptionHandler;

	public RpcThreadFactory(String threadGroupName, UncaughtExceptionHandler exceptionHandler) {
		this.group = new ThreadGroup(threadGroupName);
		if (exceptionHandler != null) {
			this.exceptionHandler = exceptionHandler;
		} else {
			this.exceptionHandler = new RpcUncaughtExceptionHandler();
		}
	}

	public RpcThreadFactory(String threadGroupName) {
		this(threadGroupName, null);
	}

	public CharSequence printStackTracesOfLiveThreads() {
		StringBuilder sb = new StringBuilder("'" + group.getName() + "' Threads: " + group.activeCount() + "\n");
		final Thread[] threads = new Thread[group.activeCount() + 2];
		int count = group.enumerate(threads, false);
		for (; count > 0; ) {
			final Thread thread = threads[--count];
			printStackTrace(thread, sb);
		}
		return sb;
	}

	private void printStackTrace(Thread thread, StringBuilder sb) {
		if (thread == null) {
			return;
		}
		sb.append("====== " + thread.getName() + "======\n");
		for (StackTraceElement element : thread.getStackTrace()) {
			sb.append(element.toString());
			sb.append("\n");
		}
		sb.append("==============");
	}
	/**
	 * @see ThreadFactory#newThread(Runnable)
	 */
	@Override
	public Thread newThread(Runnable runnable) {
		Thread thread = new Thread(group, runnable);

		thread.setName(group.getName() + "-" + thread.getId());
		thread.setUncaughtExceptionHandler(exceptionHandler);
		thread.setDaemon(true);

		return thread;
	}

	private static class RpcUncaughtExceptionHandler implements UncaughtExceptionHandler {
		/**
		 * @see UncaughtExceptionHandler#uncaughtException(Thread,
		 *      Throwable)
		 */
		@Override
		public void uncaughtException(Thread t, Throwable e) {
			log.error("uncaughtException: Uncaught Exception in thread " + t.getName(), e);
			try {
				t.interrupt();
			} catch (Exception ex) {
				// at least we tried!
			}
		}
	}
}
