/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.common.authentication;

import de.starface.com.rpc.annotation.RpcAuthRequired;
import de.starface.com.rpc.server.RpcObjectRegistry;
import de.starface.com.rpc.server.RpcServer;

/**
 * This class provides the implementors of RPC services using the {@link RpcServer} and
 * {@link RpcObjectRegistry} the possibility to access the current authentication token during an
 * RPC call.
 *
 * @author tilmann
 */
public class RpcAuth {
	private static final ThreadLocal<RpcAuthToken> authTokenPerThread = new ThreadLocal<>();

	/**
	 * Get the authentication token associated with the current RPC call. This method will only
	 * return a meaningful value when called from a thread currently executing an RPC call on the
	 * server side that specifies required authentication using the {@link RpcAuthRequired}
	 * annotation.
	 *
	 * @param <AuthToken>
	 *        the type of authentication token requested
	 * @return the current authentication token
	 * @throws IllegalStateException
	 *         if this method is called from a thread that is not part of an RPC service execution
	 *         or from an RPC service execution that is not defined to require authentication using
	 *         the {@link RpcAuthRequired} annotation.
	 */
	public static <AuthToken extends RpcAuthToken> AuthToken getCurrentAuthToken() throws IllegalStateException {
		@SuppressWarnings("unchecked")
		AuthToken currentAuthToken = (AuthToken) authTokenPerThread.get();

		if (currentAuthToken == null) {
			authTokenPerThread.remove();
			throw new IllegalStateException("Authentication information is not available for current execution.");
		}
		return currentAuthToken;
	}

	/**
	 * Sets the authentication token associated with the current RPC call.
	 *
	 * @param authToken
	 *        the authentication token to be set
	 */
	public static void setCurrentAuthToken(RpcAuthToken authToken) {
		authTokenPerThread.set(authToken);
	}

	/**
	 * Removes the authentication token associated with the current RPC call if set.
	 */
	public static void removeCurrentAuthToken() {
		authTokenPerThread.remove();
	}
}
