/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.common.interceptor;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RequestProcessor;
import de.starface.com.rpc.common.RpcRequest;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

/**
 * The RequestInterceptorChain is a collection of {@link RpcRequestInterceptor}s that are queued one
 * after each other to do the pre/post processing of an RPC request
 *
 * @author tilmann
 *
 * @param <Request>
 *        the type of request this chain can hold interceptors for
 */
public class RequestInterceptorChain<Request extends RpcRequest<?>> {
	private static final Logger log = LogManager.getLogger(RequestInterceptorChainLink.class);

	private final Request request;

	private final List<RpcRequestInterceptor<Request>> interceptors;

	private final RequestProcessor<Request> finalProcessor;

	public RequestInterceptorChain(Request request, RequestProcessor<Request> finalProcessor,
								   List<RpcRequestInterceptor<Request>> interceptors) {
		this.interceptors = new ArrayList<>(interceptors);
		this.request = request;
		this.finalProcessor = finalProcessor;
	}

	/**
	 * Start processing the request with the interceptors in this chain
	 *
	 * @throws RpcException
	 *         if an error occurs during interceptor or request processing
	 */
	public void processRequest() throws RpcException {
		if (log.isTraceEnabled()) {
			log.trace("processRequest: Processing interceptor chain for request " + request.getRpcMethodName()
				+ "...");
		}
		new ChainLink().processRequest();
		if (log.isTraceEnabled()) {
			log.trace("processRequest: Finished processing of interceptor chain for request "
				+ request.getRpcMethodName() + "...");
		}
	}

	private class ChainLink implements RequestInterceptorChainLink {
		private int nextInterceptorIndex = 0;

		/**
		 * @see de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink#processRequest()
		 */
		@Override
		public void processRequest() throws RpcException {
			if (nextInterceptorIndex < interceptors.size()) {
				RpcRequestInterceptor<Request> interceptor = interceptors.get(nextInterceptorIndex);

				if (log.isTraceEnabled()) {
					log.trace("processRequest: Invoking request interceptor " + interceptor.getClass().getName()
						+ " for request " + request.getRpcMethodName() + "...");
				}
				nextInterceptorIndex += 1;
				interceptor.interceptRequest(request, this);
				if (log.isTraceEnabled()) {
					log.trace("processRequest: Finished invokation of request interceptor "
						+ interceptor.getClass().getName() + ".");
				}
			} else {
				finalProcessor.processRequest(request);
			}
		}
	}
}
