/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.common.valuetranslation;

import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.com.rpc.annotation.RpcValueType;
import de.starface.com.rpc.common.ReflectionUtils;
import de.starface.com.rpc.common.RpcRequest;
import de.starface.com.rpc.services.filetransfer.RpcFile;
import de.starface.com.rpc.xmlrpc.XmlRpcObjectParser;
import de.starface.com.rpc.xmlrpc.XmlRpcObjectSerializer;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.File;
import java.lang.reflect.Array;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * This class is used to translate between Java values and RPC values in both directions. The
 * translation methods throw {@link IllegalArgumentException} if there is an error during
 * translation.
 */
public class ValueTranslator {
	private static final Set<Class<?>> classesRequiringNoTranslationOnAutomatic;
	private static final Logger log = LogManager.getLogger(ValueTranslator.class);
	private static final Class<?> byteArrayClass = new byte[0].getClass();

	private static boolean requiresNoTranslationOnAutomataic(Type type) {
		return classesRequiringNoTranslationOnAutomatic.contains(type);
	}

	static {
		Set<Class<?>> classSet = new HashSet<>();
		classSet.add(void.class);
		classSet.add(String.class);
		classSet.add(Integer.class);
		classSet.add(int.class);
		classSet.add(Boolean.class);
		classSet.add(boolean.class);
		classSet.add(Double.class);
		classSet.add(double.class);
		classSet.add(Date.class);
		classSet.add(byteArrayClass);

		classesRequiringNoTranslationOnAutomatic = classSet;
	}

	public static ValueTranslator createValueTranslatorForRequest(RpcRequest<?> request,
																  FileTranslator fileTranslator) {
		if (request.getRpcMethod() == null) {
			//Dynamische Methode, hat keine kompilierte Java-Implementation und daher braucht auch keine Transformationen
			return null;
		}
		RpcValueTranslation annotation = ReflectionUtils.findRpcAnnotation(RpcValueTranslation.class,
			request.getRpcInterface(), request.getRpcMethod());
		if (annotation != null) {
			if (annotation.value()) {
				if (annotation.automaticFileTransfer()) {
					Validate.notNull(fileTranslator,
						"A FileTranslator must be given for an RPC call with automatic file transfer.");
				} else {
					fileTranslator = null;
				}
				ValueTranslator valueTranslator = new ValueTranslator(annotation.version());
				valueTranslator.setFileTranslator(fileTranslator);
				return valueTranslator;
			}
		} else {
			if (RpcValueTranslation.DEFAULT_BEHAVIOUR) {
				return new ValueTranslator(
					RpcValueTranslation.DEFAULT_VERSION);
			}
		}
		return null;
	}

	private final int version;

	private FileTranslator fileTranslator;

	public ValueTranslator() {
		this.version = RpcValueTranslation.DEFAULT_VERSION;
	}

	public ValueTranslator(int version) {
		this.version = version;
	}

	@SuppressWarnings("unchecked")
	public <RpcType> RpcType javaValueToRpcValue(Object javaValue, Type javaValueType) {
		return (RpcType) javaValueToRpcValue(javaValue, javaValueType, RpcValueType.Automatic);
	}

	public Object javaValueToRpcValue(Object javaValue, Type javaValueType, RpcValueType rpcValueType) {
		Validate.notNull(rpcValueType, "No Rpc target type has been given for this translation.");
		Validate.notNull(javaValueType, "No Java source type has been given for this translation.");

		ParameterizedType javaParameterizedType = null;
		if (javaValueType instanceof ParameterizedType) {
			javaParameterizedType = (ParameterizedType) javaValueType;
			javaValueType = javaParameterizedType.getRawType();
		}

		Class<?> javaClass = null;
		if (javaValueType instanceof Class) {
			javaClass = (Class<?>) javaValueType;
			if (javaValue != null && !javaClass.isInstance(javaValue) && !javaClass.isPrimitive()) {
				throw new IllegalArgumentException("Given object of type " + javaValue.getClass().getName()
					+ " does not match the advertised type " + javaClass.getName());
			}
		}

		if (javaClass != null) {
			if (javaValue == null) {
				if (isRpcValueObject(javaClass)) {
					try {
						javaValue = javaClass.newInstance();
					} catch (Exception e) {
						log.warn("unable to create a default instance of " + javaClass.getSimpleName());
					}
				} else {
					Object rpcDefaultValue = getDefaultValue(javaClass);
					if (rpcDefaultValue != null) {
						log.debug("XML-RPC does not support null-values. Null-value will be replaced by default '" +
							rpcDefaultValue + "' for type " + javaClass.getSimpleName());
						return rpcDefaultValue;
					}
				}
			}

			switch (rpcValueType) {
				case Automatic:
					return javaValueToRpcAutomatic(javaValue, javaValueType, javaParameterizedType, javaClass);
				case String:
					return javaValueToRpcString(javaValue, javaClass);
				case Integer:
					return objectToJavaInteger(javaValue);
				case Boolean:
					return objectToJavaBoolean(javaValue);
				case Double:
					return objectToJavaDouble(javaValue);
				case DateTime:
					return objectToJavaDate(javaValue);
				case Base64:
					return objectToJavaByteArray(javaValue);
				case Struct:
					return javaValueToRpcMap(javaValue, javaParameterizedType, javaClass);
				case Array:
					return javaValueToRpcList(javaValue, javaParameterizedType, javaClass);
			}
		}

		throw new IllegalArgumentException("The value translation to RPC type " + rpcValueType.name()
			+ " of java type " + javaValueType + " is not supported.");
	}

	Object getDefaultValue(final Class<? extends Object> javaValueType) {
		if (String.class.isAssignableFrom(javaValueType)) {
			return "";
		} else if (Long.class.isAssignableFrom(javaValueType)) {
			return String.valueOf(0L);
		} else if (Boolean.class.isAssignableFrom(javaValueType)) {
			return false;
		} else if (Integer.class.isAssignableFrom(javaValueType)) {
			return 0;
		} else if (Double.class.isAssignableFrom(javaValueType)) {
			return 0D;
		} else if (byte[].class.isAssignableFrom(javaValueType)) {
			return new byte[0];
		} else if (List.class.isAssignableFrom(javaValueType)) {
			return Collections.emptyList();
		} else if (Map.class.isAssignableFrom(javaValueType)) {
			return Collections.emptyMap();
		} else if (Date.class.isAssignableFrom(javaValueType)) {
			return new Date(0);
		} else if (Enum.class.isAssignableFrom(javaValueType)) {
			return "";
		} else if (File.class.isAssignableFrom(javaValueType)) {
			return javaFileToRpcString(null);
		} else if (RpcFile.class.isAssignableFrom(javaValueType)) {
			return javaRpcFileToRpcString(null);
		}
		return null;
	}

	private String javaValueToRpcString(Object javaValue, Class<?> javaClass) {
		// Additional support for Enum
		if (Enum.class.isAssignableFrom(javaClass)) {
			return javaEnumToRpcString((Enum<?>) javaValue);
		}

		return objectToJavaString(javaValue);
	}

	private String objectToJavaString(Object javaValue) {
		if (javaValue instanceof Date) {
			return XmlRpcObjectSerializer.serializeDate((Date) javaValue);
		}

		if (javaValue != null) {
			return javaValue.toString();
		}
		return null;
	}

	private Object javaValueToRpcAutomatic(Object javaValue, Type javaValueType,
										   ParameterizedType javaParameterizedType, Class<?> javaClass) {
		// Pass through primitives supported by XML-RPC
		if (requiresNoTranslationOnAutomataic(javaValueType)) {
			return javaValue;
		}

		// Additional support for Java Long
		if (Long.class.equals(javaValueType) || long.class.equals(javaValueType)) {
			return objectToJavaString(javaValue);
		}

		if (Map.class.isAssignableFrom(javaClass)) {
			return javaMapToRpcMap((Map<?, ?>) javaValue, getMapValueTypeParameter(javaParameterizedType));
		}

		if (List.class.isAssignableFrom(javaClass)) {
			return javaListToRpcList((List<?>) javaValue, getListValueTypeParameter(javaParameterizedType));
		}

		if (javaClass.isArray()) {
			return javaArrayToRpcList((Object[]) javaValue, javaClass.getComponentType());
		}

		// Additional support for Enum
		if (Enum.class.isAssignableFrom(javaClass)) {
			return javaEnumToRpcString((Enum<?>) javaValue);
		}

		if (File.class.equals(javaClass)) {
			return javaFileToRpcString((File) javaValue);
		}
		if (RpcFile.class.equals(javaClass)) {
			return javaRpcFileToRpcString((RpcFile) javaValue);
		} else if (isRpcValueObject(javaClass)) {
			return javaObjectToRpcMap(javaValue, javaClass);
		}

		// Support polymorphic types such as Object or Number
		if (javaValue != null) {
			Class<?> actualClass = javaValue.getClass();
			if (actualClass != javaClass && javaClass.isAssignableFrom(actualClass)) {
				return javaValueToRpcAutomatic(javaValue, actualClass, javaParameterizedType, actualClass);
			}
		}

		throw new IllegalArgumentException("The value translation to RPC type Automatic of java type "
			+ javaValueType + " is not supported.");
	}

	private Map<String, Object> javaValueToRpcMap(Object javaValue, ParameterizedType javaParameterizedType,
												  Class<?> javaClass) {
		if (Map.class.isAssignableFrom(javaClass)) {
			return javaMapToRpcMap((Map<?, ?>) javaValue, getMapValueTypeParameter(javaParameterizedType));
		} else if (isRpcValueObject(javaClass)) {
			return javaObjectToRpcMap(javaValue, javaClass);
		}
		throw new IllegalArgumentException("The value translation to RPC type Struct of java type " + javaClass
			+ " is not supported.");
	}

	private List<?> javaValueToRpcList(Object javaValue, ParameterizedType javaParameterizedType,
									   Class<?> javaClass) {
		if (List.class.isAssignableFrom(javaClass)) {
			return javaListToRpcList((List<?>) javaValue, getListValueTypeParameter(javaParameterizedType));
		}
		if (javaClass.isArray()) {
			return javaArrayToRpcList((Object[]) javaValue, javaClass.getComponentType());
		}
		throw new IllegalArgumentException("The value translation to RPC type Array of java type " + javaClass
			+ " is not supported.");
	}

	private static String javaEnumToRpcString(Enum<?> javaEnumValue) {
		if (javaEnumValue != null) {
			return (javaEnumValue).name();
		} else {
			return "";
		}
	}

	private String javaFileToRpcString(File javaFile) {
		if (javaFile == null) {
			return javaRpcFileToRpcString(null);
		} else {
			return javaRpcFileToRpcString(new RpcFile(javaFile));
		}
	}

	private String javaRpcFileToRpcString(RpcFile rpcFile) {
		if (fileTranslator != null) {
			return fileTranslator.translateFileToFileId(rpcFile);
		}
		throw new IllegalArgumentException(
			"Automatic file transfer is not enabled for this RPC call. Are you missing an RpcFileTransfer annotation.");
	}

	public <RpcType, JavaType> Map<String, RpcType> javaMapToRpcMap(Map<?, JavaType> javaMap,
																	Class<JavaType> javaMapValueType) {
		Validate.notNull(javaMapValueType, "No Java map value type has been given for this translation.");

		return javaMapToRpcMap(javaMap, (Type) javaMapValueType);
	}

	private <RpcType> Map<String, RpcType> javaMapToRpcMap(Map<?, ?> javaMap, Type javaMapValueType) {
		Map<String, RpcType> resultMap = new HashMap<>();

		if (javaMap != null) {
			for (Map.Entry<?, ?> entry : javaMap.entrySet()) {
				resultMap.put(entry.getKey().toString(),
					this.<RpcType>javaValueToRpcValue(entry.getValue(), javaMapValueType));
			}
		}

		return resultMap;
	}

	public <RpcType, JavaType> List<RpcType> javaListToRpcList(List<JavaType> javaList,
															   Class<JavaType> javaListElementType) {
		Validate.notNull(javaListElementType, "No Java element type has been given for this translation.");

		return javaListToRpcList(javaList, (Type) javaListElementType);
	}

	private <RpcType> List<RpcType> javaListToRpcList(List<?> javaList, Type javaListElementType) {
		List<RpcType> resultList = new LinkedList<>();

		if (javaList != null) {
			for (Object javaItem : javaList) {
				resultList.add(this.<RpcType>javaValueToRpcValue(javaItem, javaListElementType));
			}
		}

		return resultList;
	}

	private <RpcType> List<RpcType> javaArrayToRpcList(Object[] javaArray, Type componentType) {
		List<RpcType> resultList = new LinkedList<>();

		if (javaArray != null) {
			for (Object javaItem : javaArray) {
				resultList.add(this.<RpcType>javaValueToRpcValue(javaItem, componentType));
			}
		}

		return resultList;
	}

	Map<String, Object> javaObjectToRpcMap(Object javaObject, Class<?> javaObjectType) {
		Map<String, Object> resultMap = new HashMap<>();
		if (javaObject != null) {
			List<ValueField> rpcValueFields = ValueField.findRpcValueFieldsForVersion(javaObjectType, version,
				false);
			for (ValueField field : rpcValueFields) {
				Object javaValue = field.getValueOfInstance(javaObject);
				Object translatedValue = javaValueToRpcValue(javaValue, field.getGenericType(), field.getRpcType());
				resultMap.put(field.getRpcName(), translatedValue);
			}
		}
		return resultMap;
	}

	@SuppressWarnings("unchecked")
	public <JavaType> JavaType rpcValueToJavaValue(Object rpcValue, Class<JavaType> javaValueType) {
		return (JavaType) rpcValueToJavaValue(rpcValue, (Type) javaValueType);
	}

	public Object rpcValueToJavaValue(Object rpcValue, Type javaValueType) {
		Validate.notNull(javaValueType, "No Java target type has been given for this translation.");

		if (void.class.equals(javaValueType)) {
			return null;
		}
		if (rpcValue == null) {
			return null;
		}

		if (String.class.equals(javaValueType)) {
			return objectToJavaString(rpcValue);
		}
		if (Integer.class.equals(javaValueType) || Integer.TYPE.equals(javaValueType)) {
			return objectToJavaInteger(rpcValue);
		}
		if (Boolean.class.equals(javaValueType) || Boolean.TYPE.equals(javaValueType)) {
			return objectToJavaBoolean(rpcValue);
		}
		if (Double.class.equals(javaValueType) || Double.TYPE.equals(javaValueType)) {
			return objectToJavaDouble(rpcValue);
		}
		if (Date.class.equals(javaValueType)) {
			return objectToJavaDate(rpcValue);
		}
		if (byteArrayClass.equals(javaValueType)) {
			return objectToJavaByteArray(rpcValue);
		}
		if (Long.class.equals(javaValueType) || Long.TYPE.equals(javaValueType)) {
			return objectToJavaLong(rpcValue);
		}
		if (File.class.equals(javaValueType)) {
			return objectToJavaFile(rpcValue);
		}
		if (RpcFile.class.equals(javaValueType)) {
			return objectToJavaRpcFile(rpcValue);
		}
		if (Number.class.equals(javaValueType)) {
			return objectToJavaNumber(rpcValue);
		}

		ParameterizedType javaParameterizedType = null;
		if (javaValueType instanceof ParameterizedType) {
			javaParameterizedType = (ParameterizedType) javaValueType;
			javaValueType = javaParameterizedType.getRawType();
		}

		if (List.class.equals(javaValueType) && rpcValue instanceof List) {
			if (javaParameterizedType != null) {
				return rpcListToJavaList((List<?>) rpcValue, getListValueTypeParameter(javaParameterizedType));
			} else {
				return rpcValue;
			}
		}

		if (Map.class.equals(javaValueType) && rpcValue instanceof Map) {
			if (javaParameterizedType != null) {
				return rpcMapToJavaMap((Map<?, ?>) rpcValue, getMapKeyTypeParameter(javaParameterizedType),
					getMapValueTypeParameter(javaParameterizedType));
			} else {
				return rpcValue;
			}
		}

		if (javaValueType instanceof Class) {
			Class<?> clazz = (Class<?>) javaValueType;
			if (Enum.class.isAssignableFrom(clazz)) {
				return rpcValueToJavaEnum(rpcValue, clazz);
			} else if (isRpcValueObject(clazz) && (rpcValue instanceof Map)) {
				return rpcMapToJavaObject((Map<?, ?>) rpcValue, clazz);
			} else if (clazz.isArray() && rpcValue instanceof List) {
				return rpcListToJavaArray((List<?>) rpcValue, clazz.getComponentType());
			}
			// Allow polymorphic translation of values
			else if (clazz.isInstance(rpcValue)) {
				return rpcValue;
			}
		}
		throw new IllegalArgumentException("The value translation of RPC type " + rpcValue.getClass().getName()
			+ " to Java type " + javaValueType + " is not supported.");
	}

	private static Double objectToJavaDouble(Object value) {
		if (value == null) {
			return null;
		}
		if (value instanceof Double) {
			return (Double) value;
		}
		if (value instanceof Number) {
			return ((Number) value).doubleValue();
		}
		if (value instanceof String) {
			return Double.parseDouble((String) value);
		}
		throw new IllegalArgumentException("Cannot convert value of type " + value.getClass().getName()
			+ " to Double.");
	}

	private static Boolean objectToJavaBoolean(Object value) {
		if (value == null) {
			return null;
		}
		if (value instanceof Boolean) {
			return (Boolean) value;
		}
		if (value instanceof String) {
			String valueString = (String) value;
			if (StringUtils.isEmpty(valueString)) {
				return false;
			} else {
				return !"false".equalsIgnoreCase(valueString);
			}
		}
		if (value instanceof Number) {
			return ((Number) value).doubleValue() != 0.0;
		}
		throw new IllegalArgumentException("Cannot convert value of type " + value.getClass().getName()
			+ " to Boolean.");
	}

	private static Number objectToJavaNumber(Object value) {
		if (value == null) {
			return null;
		}
		if (value instanceof Number) {
			return (Number) value;
		}
		if (value instanceof String) {
			try {
				return Integer.parseInt((String) value);
			} catch (NumberFormatException e) {
				// Try next format.
			}
			try {
				return Long.parseLong((String) value);
			} catch (NumberFormatException e) {
				// Try next format.
			}
			try {
				return Double.parseDouble((String) value);
			} catch (NumberFormatException e) {
				// No more formats to try :/
			}
		}
		throw new IllegalArgumentException("Cannot convert value of type " + value.getClass().getName()
			+ " to Integer.");
	}

	private static Integer objectToJavaInteger(Object value) {
		if (value == null) {
			return null;
		}
		if (value instanceof Integer) {
			return (Integer) value;
		}
		if (value instanceof Number) {
			return ((Number) value).intValue();
		}
		if (value instanceof String) {
			return Integer.parseInt((String) value);
		}
		throw new IllegalArgumentException("Cannot convert value of type " + value.getClass().getName()
			+ " to Integer.");
	}

	private static Date objectToJavaDate(Object value) {
		if (value == null) {
			return null;
		}
		if (value instanceof Date) {
			return (Date) value;
		}
		if (value instanceof String) {
			try {
				return XmlRpcObjectParser.parseDate((String) value);
			} catch (ParseException e) {
				throw new NumberFormatException("Failed to parse value " + value.toString() + " to Date.");
			}
		}
		throw new IllegalArgumentException("Cannot convert value of type " + value.getClass().getName()
			+ " to Date.");
	}

	private static byte[] objectToJavaByteArray(Object value) {
		if (value == null) {
			return null;
		}
		if (byteArrayClass.isInstance(value)) {
			return (byte[]) value;
		}
		throw new IllegalArgumentException("Cannot convert value of type " + value.getClass().getName()
			+ " to byte[].");
	}

	private static Long objectToJavaLong(Object value) {
		if (value == null) {
			return null;
		}
		if (value instanceof Number) {
			return ((Number) value).longValue();
		}
		if (value instanceof String) {
			return Long.parseLong((String) value);
		}
		throw new IllegalArgumentException("Cannot convert value of type " + value.getClass().getName()
			+ " to Long.");
	}

	private File objectToJavaFile(Object value) {
		RpcFile rpcFile = objectToJavaRpcFile(value);
		if (rpcFile == null) {
			return null;
		} else {
			return rpcFile.getFile();
		}
	}

	private RpcFile objectToJavaRpcFile(Object value) {
		if (value == null) {
			return null;
		}
		if (value instanceof String) {
			if (fileTranslator != null) {
				return fileTranslator.translateFileIdToFile((String) value);
			}
			throw new IllegalArgumentException(
				"Automatic file transfer is not enabled for this RPC call. Are you missing an RpcFileTransfer annotation.");
		}
		throw new IllegalArgumentException("Cannot convert value of type " + value.getClass().getName()
			+ " to File.");
	}

	@SuppressWarnings({"unchecked", "rawtypes"})
	private static Enum rpcValueToJavaEnum(Object rpcValue, Class<?> type) {
		if (rpcValue == null) {
			return null;
		}
		if (rpcValue instanceof String) {
			if ("".equals(rpcValue)) {
				return null;
			}

			Class<? extends Enum> enymType = (Class<? extends Enum>) type;
			return Enum.valueOf(enymType, (String) rpcValue);
		} else {
			throw new IllegalArgumentException("Cannot convert value of type " + rpcValue.getClass().getName()
				+ " to enum type " + type.getName() + ".");
		}
	}

	private <T> T rpcMapToJavaObject(Map<?, ?> rpcMap, Class<T> javaObjectType) {
		if (rpcMap.isEmpty()) {
			return null;
		}
		final T instance;
		try {
			instance = javaObjectType.newInstance();
		} catch (Exception e) {
			throw new IllegalArgumentException("The type " + javaObjectType.getName()
				+ " must have a public constructor without parameters to be used in value translation.");
		}
		List<ValueField> rpcValueFields = ValueField.findRpcValueFieldsForVersion(javaObjectType, version, true);
		for (ValueField field : rpcValueFields) {
			Object value = rpcMap.get(field.getRpcName());
			if (value != null) {
				value = rpcValueToJavaValue(value, field.getGenericType());
			}
			if (value != null) {
				field.setValueOfInstance(instance, value);
			}
		}
		return instance;
	}

	public <JavaKeyType, JavaValueType> Map<JavaKeyType, JavaValueType> rpcMapToJavaMap(Map<?, ?> rpcMap,
																						Class<JavaKeyType> javaMapKeyType,
																						Class<JavaValueType> javaMapValueType) {
		Validate.notNull(javaMapValueType, "No Java map value type has been given for this translation.");
		Validate.notNull(javaMapKeyType, "No Java map key type has been given for this translation.");
		Validate.notNull(rpcMap, "No Java map has been given for this translation.");

		return rpcMapToJavaMap(rpcMap, javaMapKeyType, (Type) javaMapValueType);
	}

	@SuppressWarnings("unchecked")
	private <JavaKeyType, JavaValueType> Map<JavaKeyType, JavaValueType> rpcMapToJavaMap(Map<?, ?> rpcMap,
																						 Type javaMapKeyType,
																						 Type javaMapValueType) {
		Map<JavaKeyType, JavaValueType> resultMap = new HashMap<>();
		for (Map.Entry<?, ?> entry : rpcMap.entrySet()) {
			JavaKeyType key = (JavaKeyType) rpcValueToJavaValue(entry.getKey(), javaMapKeyType);
			JavaValueType value = (JavaValueType) rpcValueToJavaValue(entry.getValue(), javaMapValueType);
			resultMap.put(key, value);
		}
		return resultMap;
	}

	public <JavaType> List<JavaType> rpcListToJavaList(List<?> rpcList, Class<JavaType> javaListElementType) {
		Validate.notNull(javaListElementType, "No Java element type has been given for this translation.");
		Validate.notNull(rpcList, "No Rpc list has been given for this translation.");

		return rpcListToJavaList(rpcList, (Type) javaListElementType);
	}

	@SuppressWarnings("unchecked")
	private <JavaType> List<JavaType> rpcListToJavaList(List<?> rpcList, Type javaListElementType) {
		List<JavaType> resultList = new ArrayList<>(rpcList.size());
		for (Object rpcItem : rpcList) {
			resultList.add((JavaType) rpcValueToJavaValue(rpcItem, javaListElementType));
		}
		return resultList;
	}

	@SuppressWarnings("unchecked")
	private <JavaType> JavaType[] rpcListToJavaArray(List<?> rpcList, Class<JavaType> componentType) {
		JavaType[] array = (JavaType[]) Array.newInstance(componentType, rpcList.size());
		int position = 0;
		for (Object rpcItem : rpcList) {
			array[position++] = rpcValueToJavaValue(rpcItem, componentType);
		}
		return array;
	}

	private static Type getListValueTypeParameter(ParameterizedType parameterizedType) {
		Validate.notNull(parameterizedType, "The value translation only supports parameterized list types.");
		return getTypeParameter(parameterizedType, 0);
	}

	private static Type getMapValueTypeParameter(ParameterizedType parameterizedType) {
		Validate.notNull(parameterizedType, "The value translation only supports parameterized map types.");
		return getTypeParameter(parameterizedType, 1);
	}

	private static Type getMapKeyTypeParameter(ParameterizedType parameterizedType) {
		Validate.notNull(parameterizedType, "The value translation only supports parameterized map types.");
		return getTypeParameter(parameterizedType, 0);
	}

	private static Type getTypeParameter(ParameterizedType parameterizedType, int numberOfParameter) {
		return parameterizedType.getActualTypeArguments()[numberOfParameter];
	}

	private static boolean isRpcValueObject(Class<?> javaType) {
		return javaType.isAnnotationPresent(RpcValueObject.class);
	}

	public FileTranslator getFileTranslator() {
		return fileTranslator;
	}

	public void setFileTranslator(FileTranslator fileTranslator) {
		this.fileTranslator = fileTranslator;
	}

	public static boolean isFileTranslationRequiredForType(Type type, int version) {
		ParameterizedType javaParameterizedType = null;
		if (type instanceof ParameterizedType) {
			javaParameterizedType = (ParameterizedType) type;
			type = javaParameterizedType.getRawType();
		}

		Class<?> javaClass = null;
		if (type instanceof Class) {
			javaClass = (Class<?>) type;
		}

		if (javaClass != null) {
			if (isTranslatableFileType(javaClass)) {
				return true;
			}

			if (Map.class.isAssignableFrom(javaClass)) {
				return isFileTranslationRequiredForType(getMapValueTypeParameter(javaParameterizedType), version);
			}

			if (List.class.isAssignableFrom(javaClass)) {
				return isFileTranslationRequiredForType(getListValueTypeParameter(javaParameterizedType), version);
			}

			if (javaClass.isArray()) {
				return isFileTranslationRequiredForType(javaClass.getComponentType(), version);
			}

			if (isRpcValueObject(javaClass)) {
				for (ValueField field : ValueField.findRpcValueFieldsForVersion(javaClass, version, true)) {
					if (isFileTranslationRequiredForType(field.getGenericType(), version)) {
						return true;
					}
				}
			}
		}

		return false;
	}

	private static boolean isTranslatableFileType(Class<?> type) {
		return File.class.equals(type) || RpcFile.class.equals(type);
	}
}
