/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.server;

import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.valuetranslation.FileTranslator;

import org.apache.commons.lang3.Validate;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * An implementation of {@link RpcMethod} that allows to dynamically define RPC methods at runtime.
 * DynamicRpcMethods do not have an underlying Java Method and Interface. The invocation is handeld
 * through an {@link InvocationHandler} given at construction time.
 *
 * @author tilmann
 */
public class DynamicRpcMethod implements RpcMethod {
	/**
	 * Each DynamicRpcMethod invokes an InvocationHandler when the method is invoked this handler is
	 * to be implemented by the API users.
	 *
	 * @author tilmann
	 */
	public static interface InvocationHandler {
		/**
		 * Invoke the logic of the dynamicRpcMethod with the given parameters returning the desired
		 * return value
		 *
		 * @param dynamicRpcMethod
		 *        the DynamicRpcMethod this invocation belongs to
		 * @param parameters
		 *        the parameters of the invocation
		 * @return the return value of the invocation
		 * @throws Exception
		 *         if the invocation has failed
		 */
		public Object invoke(DynamicRpcMethod dynamicRpcMethod, Object[] parameters) throws Exception;
	}

	private final String name;

	private final Class<? extends RpcAuthToken> requiredAuthType;

	private final InvocationHandler invocationHandler;

	public DynamicRpcMethod(String name, InvocationHandler invocationHandler) {
		this(name, invocationHandler, null);
	}

	public DynamicRpcMethod(String name, InvocationHandler invocationHandler,
							Class<? extends RpcAuthToken> requiredAuthType) {
		Validate.notEmpty(name, "name=empty");
		Validate.notNull(invocationHandler, "invocationHandler=null");
		this.name = name;
		this.invocationHandler = invocationHandler;
		this.requiredAuthType = requiredAuthType;
	}

	/**
	 * This property is always null
	 *
	 * @see de.starface.com.rpc.server.RpcMethod#getMethod()
	 */
	@Override
	public Method getMethod() {
		return null;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcMethod#getRequiredAuthType()
	 */
	@Override
	public Class<? extends RpcAuthToken> getRequiredAuthType() {
		return requiredAuthType;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcMethod#getName()
	 */
	@Override
	public String getName() {
		return name;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcMethod#invoke(Object[])
	 */
	@Override
	public Object invoke(Object[] parameters) throws IllegalArgumentException, IllegalAccessException,
		InvocationTargetException {
		try {
			return invocationHandler.invoke(this, parameters);
		} catch (Exception e) {
			throw new InvocationTargetException(e, "Invocation of dynamic RPC method " + name
				+ " threw exception.");
		}
	}

	/**
	 * This property is always null
	 *
	 * @see de.starface.com.rpc.server.RpcMethod#getRpcInterface()
	 */
	@Override
	public final Class<?> getRpcInterface() {
		return null;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcMethod#getFileTranslator()
	 */
	@Override
	public FileTranslator getFileTranslator() {
		return null;
	}
}
