/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.server;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RequestProcessor;
import de.starface.com.rpc.common.RpcRequest;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.lang.reflect.InvocationTargetException;

/**
 * A request processor for incoming requests that invokes the attached {@link RpcMethod} with the
 * parameters from the incoming request and stores the result or performs exception handling.
 *
 * @author tilmann
 */
public class IncomingRequestProcessor implements RequestProcessor<IncomingRequest<?>> {
	private static final Logger log = LogManager.getLogger(IncomingRequestProcessor.class);

	private RpcMethod serverMethod;

	IncomingRequestProcessor(RpcMethod serverMethod) {
		this.serverMethod = serverMethod;
	}

	@Override
	public void processRequest(IncomingRequest<?> request) throws RpcException {
		try {
			tryProcessRequest(request);
		} catch (InvocationTargetException e) {
			/*
			 * In case of an InvocationTargetException, execution failed with an (possibly declared)
			 * exception.
			 */
			Throwable t = e.getCause();
			if (t != null) {
				// Wrap application exceptions to be restored on client side
				throw new RpcException(RpcException.Codes.Server.APPLICATION_EXCEPTION,
					"The remote procedure threw an exception during execution.", t);
			} else {
				// Hopefully this should never happen because Java
				// provides a cause for any InvocationTargetException
				log.warn("The RPC server could not execute the procedure", e);
				throw new RpcException(RpcException.Codes.Server.PROCEDURE_EXECUTION_FAILED,
					"The RPC server could not execute the procedure.", e);
			}
		} catch (Exception e) {
			// Handle other unexpected exceptions
			log.warn("The RPC server could not execute the procedure", e);
			throw new RpcException(RpcException.Codes.Server.PROCEDURE_EXECUTION_FAILED,
				"The RPC server could not execute the procedure", e);
		}
	}

	private void tryProcessRequest(RpcRequest<?> request) throws IllegalArgumentException, IllegalAccessException,
		InvocationTargetException {
		Object[] parameters = request.getParameters();
		Object returnValue = serverMethod.invoke(parameters);
		request.setReturnValue(returnValue);
	}
}
