/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.server;

import de.starface.com.rpc.common.ReflectionUtils;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.valuetranslation.FileTranslator;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * This is an implementation of a static {@link RpcMethod} that is always bound to an
 * {@link RpcObject}.
 *
 * @author tilmann
 */
class ObjectBoundRpcMethod implements RpcMethod {
	private final Method method;

	private final Class<? extends RpcAuthToken> requiredAuthType;

	private final RpcObject<?> rpcOject;

	private final String name;

	private final AutomaticFileTransfer automaticFileTransfer;

	ObjectBoundRpcMethod(RpcObject<?> serverObject, Method rpcMethod, String rpcMethodNameSuffix) {
		this.method = rpcMethod;
		this.rpcOject = serverObject;
		this.name = RpcObjectRegistry.computeRpcMethodName(serverObject.getNamePrefix(),
			serverObject.getRpcInterface(), rpcMethod, rpcMethodNameSuffix);
		this.requiredAuthType = ReflectionUtils.determineAuthTokenTypeOfMethod(rpcMethod);
		this.automaticFileTransfer = AutomaticFileTransfer.createAutomaticFileTransferForCall(
			rpcOject.getRpcInterface(), rpcMethod, name, requiredAuthType);
	}

	/**
	 * @see de.starface.com.rpc.server.RpcMethod#getMethod()
	 */
	@Override
	public Method getMethod() {
		return method;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcMethod#getRequiredAuthType()
	 */
	@Override
	public Class<? extends RpcAuthToken> getRequiredAuthType() {
		if (requiredAuthType != null) {
			return requiredAuthType;
		} else {
			return rpcOject.getRequiredAuthType();
		}
	}

	/**
	 * @see de.starface.com.rpc.server.RpcMethod#getName()
	 */
	@Override
	public String getName() {
		return name;
	}

	/**
	 * @see Object#toString()
	 */
	@Override
	public String toString() {
		return "RpcMethod [name=" + name + ", method=" + method + ", requiredAuthType=" + requiredAuthType
			+ ", object=" + rpcOject + "]";
	}

	/**
	 * @see de.starface.com.rpc.server.RpcMethod#invoke(Object[])
	 */
	@Override
	public Object invoke(Object[] parameters) throws IllegalArgumentException, IllegalAccessException,
		InvocationTargetException {
		return method.invoke(rpcOject.getRpcInstance(), parameters);
	}

	/**
	 * @see de.starface.com.rpc.server.RpcMethod#getRpcInterface()
	 */
	@Override
	public Class<?> getRpcInterface() {
		return rpcOject != null ? rpcOject.getRpcInterface() : null;
	}

	/**
	 * @see de.starface.com.rpc.server.RpcMethod#getFileTranslator()
	 */
	@Override
	public FileTranslator getFileTranslator() {
		return automaticFileTransfer;
	}
}
