/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.server;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcRequest;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter;

/**
 * This interface describes an entity that is capable of executing {@link RpcRequest}s. And
 * abstracts away the subtleties of the different transport protocols through a strategy pattern.
 *
 * @author tilmann
 *
 * @param <TransportToken>
 *        The {@link RpcTransportToken} subclass this executer processes requests for.
 */
public interface RequestExecutor<TransportToken extends RpcTransportToken> {
	/**
	 * Set the {@link RpcObjectRegistry} to be used to find fitting {@link RpcMethod}s.
	 *
	 * @param rpcObjectRegistry
	 *        the {@link RpcObjectRegistry} to be used
	 */
	public void setServerObjectRegistry(RpcObjectRegistry rpcObjectRegistry);

	/**
	 * Set the {@link RpcAuthTokenConverter} to be used to authenticate the call
	 *
	 * @param authTokenConverter
	 *        the {@link RpcAuthTokenConverter} to be used
	 */
	public void setAuthTokenConverter(RpcAuthTokenConverter<? extends RpcTransportToken, ? extends RpcAuthToken> authTokenConverter);

	/**
	 * This method is used to perform the execution of the given {@link RpcRequest} including
	 * authentication and error handling.
	 *
	 * @param rpcMethodName
	 *        the name of the RPC method to be executed
	 * @param parameters
	 *        the parameters for the RPC request
	 * @param sourceTransportToken
	 *        the transport token of the caller
	 * @return the return value of the request
	 *
	 * @throws RpcException
	 *         If execution is not possible or fails
	 */
	public Object execute(String rpcMethodName, Object[] parameters, TransportToken sourceTransportToken)
		throws RpcException;

	/**
	 * Prepares an exception encountered at some time throughout the RPC request handling to an
	 * {@link RpcException} that can be returned to the calling client
	 *
	 * @param e
	 *        the exception encountered
	 * @return the {@link RpcException} to be returned to the caller
	 */
	public RpcException convertExceptionToRpcException(Exception e);
}
