/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.server;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter;
import de.starface.com.rpc.common.interceptor.RequestInterceptorChain;
import de.starface.com.rpc.server.interceptor.AuthenticationRequestInterceptor;
import de.starface.com.rpc.server.interceptor.IncomingRequestInterceptor;
import de.starface.com.rpc.server.interceptor.LoggingRequestInterceptor;
import de.starface.com.rpc.server.interceptor.ValueTranslationRequestInterceptor;

import org.apache.commons.lang3.Validate;

import java.util.ArrayList;
import java.util.List;

/**
 * A base class for the implementation of {@link RequestExecutor}s on the server side.
 *
 * @author tilmann
 *
 * @param <TransportToken>
 *        the transport token type of the requests executed
 */
public class RequestExecutorImpl<TransportToken extends RpcTransportToken> implements
	RequestExecutor<TransportToken> {
	private static final IncomingRequestInterceptor<RpcTransportToken> loggingRequestInterceptor = new LoggingRequestInterceptor();

	private RpcAuthTokenConverter<? extends RpcTransportToken, ? extends RpcAuthToken> authTokenConverter;

	private RpcObjectRegistry serverObjectRegistry;

	protected RpcMethod serverMethod;

	private RequestInterceptorChain<?> interceptorChain;

	private IncomingRequest<TransportToken> request;

	protected Object returnValue;

	/**
	 * @see de.starface.com.rpc.server.RequestExecutor#execute(String, Object[],
	 *      de.starface.com.rpc.common.RpcTransportToken)
	 */
	@Override
	public final Object execute(String rpcMethodName, Object[] parameters, TransportToken sourceTransportToken)
		throws RpcException {
		createRequest(rpcMethodName, parameters, sourceTransportToken);

		prepareRequestInterceptorChain();

		interceptorChain.processRequest();

		return request.getReturnValue();
	}

	private void createRequest(String rpcMethodName, Object[] parameters, TransportToken sourceTransportToken)
		throws RpcException {
		Validate.notEmpty(rpcMethodName, "rpcMethodName=empty");
		Validate.notNull(sourceTransportToken, "sourceTransportToken=null");

		determineServerMethod(rpcMethodName);

		request = new IncomingRequest<>(serverMethod.getName(), serverMethod.getRpcInterface(),
			serverMethod.getMethod(), parameters, sourceTransportToken);
	}

	private void determineServerMethod(String rpcMethodName) throws RpcException {
		serverMethod = null;

		if (serverObjectRegistry != null) {
			serverMethod = serverObjectRegistry.lookupRpcMethod(rpcMethodName);
		}
		if (serverMethod == null) {
			serverMethod = RpcObjectRegistry.DEFAULT_REGISTRY.lookupRpcMethod(rpcMethodName);
		}
		if (serverMethod == null) {
			throw new RpcException(RpcException.Codes.Server.PROCEDURE_NOT_AVAILABLE,
				"No processor found for the identifier " + rpcMethodName);
		}
	}

	@SuppressWarnings({"unchecked", "rawtypes"})
	private void prepareRequestInterceptorChain() throws RpcException {
		interceptorChain = new RequestInterceptorChain<>(request, new IncomingRequestProcessor(
			serverMethod), (List) prepareRequestInterceptors());
	}

	protected List<IncomingRequestInterceptor<?>> prepareRequestInterceptors() throws RpcException {
		List<IncomingRequestInterceptor<?>> interceptors = new ArrayList<>();
		interceptors.add(loggingRequestInterceptor);
		interceptors.add(new AuthenticationRequestInterceptor(serverMethod.getRequiredAuthType(),
			authTokenConverter));
		interceptors.add(ValueTranslationRequestInterceptor.createInterceptorForRequest(request,
			serverMethod.getFileTranslator()));
		return interceptors;
	}

	/**
	 * @see de.starface.com.rpc.server.RequestExecutor#setServerObjectRegistry(de.starface.com.rpc.server.RpcObjectRegistry)
	 */
	@Override
	public void setServerObjectRegistry(RpcObjectRegistry serverObjectRegistry) {
		this.serverObjectRegistry = serverObjectRegistry;
	}

	/**
	 * @see de.starface.com.rpc.server.RequestExecutor#setAuthTokenConverter(de.starface.com.rpc.common.authentication.RpcAuthTokenConverter)
	 */
	@Override
	public void setAuthTokenConverter(RpcAuthTokenConverter<? extends RpcTransportToken, ? extends RpcAuthToken> authTokenConverter) {
		this.authTokenConverter = authTokenConverter;
	}

	/**
	 * @see de.starface.com.rpc.server.RequestExecutor#convertExceptionToRpcException(Exception)
	 */
	@Override
	public RpcException convertExceptionToRpcException(Exception e) {
		if (e instanceof RpcException) {
			return (RpcException) e;
		} else {
			return new RpcException(RpcException.Codes.Server.SERVER_ERROR,
				"An unexpected error has occurred in the code of the RPC server.", e);
		}
	}
}
