/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.server.interceptor;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.authentication.RpcAuth;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter;
import de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink;
import de.starface.com.rpc.server.IncomingRequest;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * This request interceptor implements the aspect of caller authentication during RPC requests.
 *
 * @author tilmann
 */
public class AuthenticationRequestInterceptor implements IncomingRequestInterceptor<RpcTransportToken> {
	private static final Logger log = LogManager.getLogger(AuthenticationRequestInterceptor.class);

	private final Class<? extends RpcAuthToken> requiredAuthTokenType;

	private final RpcAuthTokenConverter<? extends RpcTransportToken, ? extends RpcAuthToken> authTokenConverter;

	public AuthenticationRequestInterceptor(Class<? extends RpcAuthToken> requiredAuthTokenType,
											RpcAuthTokenConverter<? extends RpcTransportToken, ? extends RpcAuthToken> authTokenConverter) {
		this.requiredAuthTokenType = requiredAuthTokenType;
		this.authTokenConverter = authTokenConverter;
	}

	/**
	 * @see de.starface.com.rpc.common.interceptor.RpcRequestInterceptor#interceptRequest(de.starface.com.rpc.common.RpcRequest,
	 *      de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink)
	 */
	@Override
	public void interceptRequest(IncomingRequest<RpcTransportToken> request,
								 RequestInterceptorChainLink interceptorChain) throws RpcException {

		if (isAuthenticationRequired()) {
			RpcAuthToken sourceAuthToken = prepareAuthentication(request);

			RpcAuth.setCurrentAuthToken(sourceAuthToken);
		}
		try {
			interceptorChain.processRequest();
		} finally {
			RpcAuth.removeCurrentAuthToken();
		}
	}

	private boolean isAuthenticationRequired() {
		return requiredAuthTokenType != null;
	}

	private RpcAuthToken prepareAuthentication(IncomingRequest<?> request) throws RpcException {
		validateAuthTokenConverter(request);

		return convertTransportTokenToAuthToken(request);
	}

	private <AuthToken extends RpcAuthToken> void validateAuthTokenConverter(IncomingRequest<?> request)
		throws RpcException {
		RpcTransportToken sourceTransportToken = request.getSourceTransportToken();
		Class<? extends RpcTransportToken> sourceTransportTokenType = sourceTransportToken.getClass();

		if (authTokenConverter == null || !authTokenConverter.getAuthType().equals(requiredAuthTokenType)
			|| !authTokenConverter.getTransportType().equals(sourceTransportTokenType)) {
			throw new RpcException(RpcException.Codes.Server.AUTH_CONVERTER_MISSING,
				"Auth token converter not registered for the client auth token type "
					+ requiredAuthTokenType.getName() + " and transport type "
					+ sourceTransportTokenType.getName());
		}
	}

	private <AuthToken extends RpcAuthToken> AuthToken convertTransportTokenToAuthToken(IncomingRequest<?> request)
		throws RpcException {
		AuthToken sourceAuthToken = null;

		RpcTransportToken sourceTransportToken = request.getSourceTransportToken();

		@SuppressWarnings("unchecked")
		RpcAuthTokenConverter<RpcTransportToken, AuthToken> converter = (RpcAuthTokenConverter<RpcTransportToken, AuthToken>) authTokenConverter;
		try {
			sourceAuthToken = converter.convert(sourceTransportToken);
		} catch (Exception e) {
			log.warn("The registered auth token converter threw an exception during execution.", e);
			throw new RpcException(RpcException.Codes.Server.AUTH_CONVERTER_EXCEPTION,
				"The registered auth token converter threw an exception during execution.", e);
		}

		if (sourceAuthToken == null) {
			throw new RpcException(RpcException.Codes.Server.AUTH_UNKNOWN,
				"The registered auth token converter did not convert the client transport token "
					+ sourceTransportToken);
		}
		return sourceAuthToken;
	}
}
