/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.services.filetransfer;

import de.starface.com.rpc.RpcRuntimeException;
import de.starface.com.rpc.client.RpcClientBase;
import de.starface.com.rpc.common.events.EventsProviderSupport;
import de.starface.com.rpc.services.RpcAgent;
import de.starface.com.rpc.services.filetransfer.FileTransfer.HashMethod;
import de.starface.com.rpc.services.filetransfer.FileTransfer.State;
import de.starface.com.rpc.services.filetransfer.common.CompleteDownloadResult;
import de.starface.com.rpc.services.filetransfer.common.ContinueDownloadResult;
import de.starface.com.rpc.services.filetransfer.common.IncomingFileTransfer;
import de.starface.com.rpc.services.filetransfer.common.InitializeDownloadResult;
import de.starface.com.rpc.services.filetransfer.common.RpcFileDownload;
import de.starface.com.rpc.services.filetransfer.common.RpcFiletransferLegacy;

import org.apache.commons.lang3.Validate;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Implementation of the agent-counterpart for the file download service. Encapsulates the whole
 * file download process in a simple method call and thus allows RPC clients to easily download
 * files from a corresponding server.
 *
 * @author tilmann
 */
public class RpcFileDownloadAgent implements RpcAgent<RpcFileDownload> {

	private static final Logger log = LogManager.getLogger(RpcFileDownloadAgent.class);

	/** The used proxy to call the server */
	private RpcFileDownload fileDownloadProxy;

	private EventsProviderSupport<FileTransferEventListener> eventsSupport = new EventsProviderSupport<>(
		FileTransferEventListener.class);

	/**
	 * Default constructor used by {@link RpcClientBase} for example.
	 */
	public RpcFileDownloadAgent() {
		// Default constructor used by XmlRpcProxyFactory
	}

	/**
	 * Contructor that lets the caller set the proxy during construction.
	 *
	 * @param proxy
	 *        the {@link RpcFiletransferLegacy} proxy to be used during the transfer.
	 */
	public RpcFileDownloadAgent(RpcFileDownload proxy) {
		setProxy(proxy);
	}

	/**
	 * Downloads the file with the given id
	 *
	 * @param fileId
	 *        the id of the file to download
	 * @return the {@link RpcFile} if the download has succeeded
	 * @throws FileTransferFailedException
	 *         if the download has failed
	 */
	public RpcFile downloadFile(String fileId) throws FileTransferFailedException {
		return downloadFile(fileId, null);
	}

	/**
	 * Downloads the file with the given id and the given hashMethod for transport reliability.
	 *
	 * @param fileId
	 *        the id of the file to download
	 * @param hashMethod
	 *        the {@link HashMethod} to be used to verify the file contents
	 * @return the {@link RpcFile} if the download has succeeded
	 * @throws FileTransferFailedException
	 *         if the download has failed
	 */
	public RpcFile downloadFile(String fileId, HashMethod hashMethod) throws FileTransferFailedException {
		Validate.notEmpty(fileId, "fileId=empty");

		try {
			InitializeDownloadResult initializeResult = fileDownloadProxy.initializeDownload(fileId, hashMethod);
			String fileTransferId = initializeResult.getFileTransferId();

			RpcFile rpcFile = new RpcFile(fileId, initializeResult.getFileInfo());
			IncomingFileTransfer incomingTransfer = new IncomingFileTransfer(fileTransferId, rpcFile, hashMethod,
				eventsSupport.getEventsDistributor(), false);

			if (State.CANCELLED == initializeResult.getFileTransferState()) {
				incomingTransfer.failAndThrow(FileTransferFailedEvent.Codes.DENIED_BY_SERVER,
					"The file transfer request was not accepted by the server.");
			}

			if (State.IN_PROGRESS != initializeResult.getFileTransferState()) {
				incomingTransfer.setIncomingServerState(initializeResult.getFileTransferState());
			}

			try {
				incomingTransfer.initializeTransfer();

				while (!incomingTransfer.allBytesTransferred() && incomingTransfer.getState() == State.IN_PROGRESS) {
					ContinueDownloadResult continueResult = fileDownloadProxy.continueDownload(fileTransferId);

					incomingTransfer.setIncomingServerState(continueResult.getFileTransferState());
					incomingTransfer.receiveBlock(continueResult.getFileBlock());
				}

				if (incomingTransfer.allBytesTransferred() && incomingTransfer.getState() == State.IN_PROGRESS) {
					CompleteDownloadResult completeResult = fileDownloadProxy.completeDownload(fileTransferId);

					incomingTransfer.setIncomingServerState(completeResult.getFileTransferState());
					incomingTransfer.completeTransfer(completeResult.getFileEnd());
				}
				return rpcFile;
			} catch (FileTransferFailedException e) {
				if (e.getCode() == FileTransferFailedEvent.Codes.ABORTED_BY_CLIENT) {
					cancelDownloadSilently(incomingTransfer);
				}
				throw e;
			} catch (RpcRuntimeException e) {
				cancelDownloadSilently(incomingTransfer);
				incomingTransfer.failAndThrow(FileTransferFailedEvent.Codes.RPC_EXCEPTION,
					"An exception occurred during an XML-RPC call.", e);

				// Just to satisfy the compiler
				return null;
			}
		} catch (RpcRuntimeException e) {
			new IncomingFileTransfer(null, null, hashMethod, eventsSupport.getEventsDistributor(), false).failAndThrow(
				FileTransferFailedEvent.Codes.RPC_EXCEPTION, "An exception occurred during an XML-RPC call.", e);

			// Just to satisfy the compiler
			return null;
		}
	}

	private void cancelDownloadSilently(IncomingFileTransfer incomingFileTransfer) {
		try {
			fileDownloadProxy.cancelDownload(incomingFileTransfer.getId());
		} catch (Exception e) {
			log.info("Exception while cancelling file download.", e);
		}
	}

	/**
	 * @see de.starface.com.rpc.services.RpcAgent#getProxyType()
	 */
	@Override
	public Class<RpcFileDownload> getProxyType() {
		return RpcFileDownload.class;
	}

	/**
	 * @see de.starface.com.rpc.services.RpcAgent#setProxy(Object)
	 */
	@Override
	public void setProxy(RpcFileDownload proxy) {
		if (fileDownloadProxy != null) {
			throw new IllegalStateException("The proxy for this RpcFileDownloadAgent has allready been set.");
		}
		fileDownloadProxy = proxy;
	}

	/**
	 * Adds a {@link FileTransferEventListener} to be informed about downloads using this agent
	 *
	 * @param listener
	 *        the listener to be added
	 */
	public void addEventListener(FileTransferEventListener listener) {
		eventsSupport.subscribeEvents(listener);
	}

	/**
	 * Removes a {@link FileTransferEventListener} from this agent's listeners
	 *
	 * @param listener
	 *        the listener to be removed
	 */
	public void removeEventListener(FileTransferEventListener listener) {
		eventsSupport.unsubscribeEvents(listener);
	}
}
