/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.services.filetransfer.common;

import de.starface.com.rpc.services.filetransfer.FileTransferEventListener;
import de.starface.com.rpc.services.filetransfer.FileTransferFailedEvent;
import de.starface.com.rpc.services.filetransfer.FileTransferFailedException;
import de.starface.com.rpc.services.filetransfer.RpcFile;

import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

/**
 * This class encapsulates the common logic for an incoming file transfer on either server or client
 * side.
 *
 * @author tilmann
 */
public class IncomingFileTransfer extends FileTransferBase {

	private IncomingFile incomingFile = new IncomingFile();

	private MessageDigest transferDigest;

	public IncomingFileTransfer(String id, RpcFile rpcFile, HashMethod hashMethod,
								FileTransferEventListener eventListener, boolean isServerFile) {
		super(id, rpcFile, hashMethod, eventListener, isServerFile);
	}

	public synchronized void initializeTransfer() throws FileTransferFailedException {
		super.initialize();
		initializeTransferDigest();
		initializeFileIO();
	}

	private void initializeTransferDigest() throws FileTransferFailedException {
		try {
			transferDigest = MessageDigest.getInstance(getHashMethod().name());
		} catch (NoSuchAlgorithmException e) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_IO_FAILURE,
				"Could not create message digest for data transfer.", e);
		}
	}

	private void initializeFileIO() throws FileTransferFailedException {
		try {
			incomingFile.openWithDigestNameAndDesiredName(getHashMethod().name(),
				getRpcFile().getFileInfo().getFileName());
			getRpcFile().setFile(incomingFile.getFile());
		} catch (Exception e) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_IO_FAILURE,
				"Could not open temporary file for writing for incoming RPC file " + getRpcFile(), e);
		}
	}

	public synchronized void receiveBlock(FileBlock fileTransferBlock) throws FileTransferFailedException {

		byte[] data = fileTransferBlock.getData();
		if (!Arrays.equals(transferDigest.digest(data), fileTransferBlock.getBlockDigest())) {
			// block digest does not match
			failAndThrow(FileTransferFailedEvent.Codes.FILE_CORRUPT, "Hash digest of incoming data is incorrect.");
		}

		long currentFileSize = incomingFile.getSize();

		if (currentFileSize == getRpcFile().getFileInfo().getFileSize()) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_CORRUPT, "Incomming excessive data.");
		}

		int dataLength = data.length;

		if (currentFileSize + dataLength > getRpcFile().getFileInfo().getFileSize()) {
			// correct block size for last block
			dataLength = (int) (getRpcFile().getFileInfo().getFileSize() - currentFileSize);
		}

		if (dataLength > 0) {
			try {
				incomingFile.write(data, dataLength);
			} catch (IOException e) {
				failAndThrow(FileTransferFailedEvent.Codes.FILE_IO_FAILURE, "Error writing to file "
					+ incomingFile.getName(), e);
			}
		}

		incrementTransferredBytes(dataLength);
		signalProgress();
	}

	public synchronized void completeTransfer(FileEnd fileTransferEnd) throws FileTransferFailedException {
		byte[] myFileDigest = null;
		try {
			myFileDigest = incomingFile.closeAndComputeDigest();
		} catch (IOException e) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_IO_FAILURE, "Error closing file.", e);
		}

		if (getTransferredBytes() < getRpcFile().getFileInfo().getFileSize()) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_CORRUPT, "File ransfer finished on incomplete file.");
		}

		if (incomingFile.getSize() != getRpcFile().getFileInfo().getFileSize()) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_CORRUPT, "Transferred file has not the expected size.");
		}

		if (!Arrays.equals(myFileDigest, fileTransferEnd.getFileDigest())) {
			failAndThrow(FileTransferFailedEvent.Codes.FILE_CORRUPT,
				"Hash digest of transferred file is incorrect.");
		}

		complete();
	}

	@Override
	protected synchronized void cleanUpOnAbort() {
		incomingFile.deleteSilently();
		getRpcFile().setFile(null);
	}
}
