/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.services.filetransfer.service;

import de.starface.com.rpc.services.filetransfer.FileTransfer;
import de.starface.com.rpc.services.filetransfer.common.FileTransferBase;

import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;

/**
 * This call represents a registry of file transfers used on the server side for both incoming and
 * outgoing file transfers. It implements a mechanism for file transfers to timeout if they have not
 * traffic for specific period of time.
 *
 * @author tilmann
 *
 * @param <FileTransferType> the type of file transfer stored in this registry
 */
public class FileTransferRegistry<FileTransferType extends FileTransferBase> {

	private static Timer timeoutTimer;
	private static final Object syncTimeoutTimer = new Object();

	private void createTimeoutTimer() {
		synchronized (syncTimeoutTimer) {
			if (timeoutTimer == null) {
				timeoutTimer = new Timer("XML-RPC File transfer timeout", true);
				// initialize with cleanup task
				timeoutTimer.schedule(new TimerTask() {
					@Override
					public void run() {
						synchronized (syncTimeoutTimer) {
							if (timeoutTimer != null) {
								timeoutTimer.purge();
							}
						}
					}
				}, 3600000L, 3600000L);
			}
		}
	}

	private void destroyTimeoutTimer() {
		synchronized (syncTimeoutTimer) {
			if (timeoutTimer != null) {
				timeoutTimer.cancel();
				timeoutTimer.purge();
				timeoutTimer = null;
			}
		}
	}

	private void createOrDestroyTimeoutTimer() {
		if (registeredFileTransfers.isEmpty()) {
			destroyTimeoutTimer();
		} else {
			createTimeoutTimer();
		}
	}

	private final Map<String, FileTransferType> registeredFileTransfers = new ConcurrentHashMap<>();

	public void registerFileTransfer(FileTransferType fileTransfer, long timeout) {
		registeredFileTransfers.put(fileTransfer.getId(), fileTransfer);
		createOrDestroyTimeoutTimer();
		createNewTimoutTask(fileTransfer, timeout);
	}

	public void unregisterFileTransfer(FileTransferType fileTransfer) {
		registeredFileTransfers.remove(fileTransfer.getId());
		createOrDestroyTimeoutTimer();
	}

	public FileTransferType getFileTransfer(String fileTransferId) {
		return registeredFileTransfers.get(fileTransferId);
	}

	private void createNewTimoutTask(FileTransferType fileTransfer, long timeout) {
		TimeoutTask timeoutTask = new TimeoutTask(fileTransfer);
		synchronized (syncTimeoutTimer) {
			if (timeoutTimer != null) {
				timeoutTimer.schedule(timeoutTask, timeout, timeout);
			}
		}
	}

	class TimeoutTask extends TimerTask {

		private final FileTransferType fileTransfer;
		private long sendBytes;

		public TimeoutTask(FileTransferType fileTransfer) {
			this.fileTransfer = fileTransfer;
			this.sendBytes = fileTransfer.getTransferredBytes();
		}

		@Override
		public void run() {
			synchronized (fileTransfer) {
				if (fileTransfer.getState() == FileTransfer.State.IN_PROGRESS) {
					long newSendBytes = fileTransfer.getTransferredBytes();
					if (newSendBytes == sendBytes) {
						fileTransfer.timeout();
						this.cancel();
						unregisterFileTransfer(fileTransfer);
					} else {
						sendBytes = newSendBytes;
					}
				} else {
					this.cancel();
					unregisterFileTransfer(fileTransfer);
				}
			}
		}
	}
}
