/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc;

import org.apache.commons.text.StringEscapeUtils;
import org.jivesoftware.smack.util.Base64;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * An object-serializer to grain a textual representation for a variety of java types as specified
 * for the XML RPC <a href="http://www.xmlrpc.com/spec">http://www.xmlrpc.com/spec</a>. The
 * corresponding deserializer part can be found in the XmlRpcPacketProvider as part of the
 * object-parsing process via a xml-pullparser.
 *
 * @author may
 */
public class XmlRpcObjectSerializer {
	/**
	 * Computes a textual representation for an given object according to <a
	 * href="http://www.xmlrpc.com/spec">http://www.xmlrpc.com/spec</a>.<br/>
	 * <b>Note: </b> Supported types so far are:
	 *
	 * <ul>
	 * <li>List</li>
	 * <li>Map</li>
	 * <li>Date</li>
	 * <li>String</li>
	 * <li>Integer</li>
	 * <li>Boolean</li>
	 * <li>Double</li>
	 * <li>byte[]</li>
	 * </ul>
	 * Other types will just cause an IllegalArgumentException
	 *
	 * @param o
	 *        The object to serialize
	 * @return The serialized representation of this object
	 * @throws IllegalArgumentException
	 *         If the given Object is not recognized
	 */
	@SuppressWarnings({"unchecked", "rawtypes"})
	public static String serialize(Object o) throws IllegalArgumentException {

		if (o instanceof List) {
			return serialize((List) o);
		}
		if (o instanceof Map) {
			return serialize((Map) o);
		}
		if (o instanceof Date) {
			return serialize((Date) o);
		}
		if (o instanceof String) {
			return serialize((String) o);
		}
		if (o instanceof Integer) {
			return serialize(((Integer) o).intValue());
		}
		if (o instanceof Boolean) {
			return serialize(((Boolean) o).booleanValue());
		}
		if (o instanceof Double) {
			return serialize(((Double) o).doubleValue());
		}
		if (o instanceof byte[]) {
			return serialize((byte[]) o);
		}

		if (o == null) {
			throw new IllegalArgumentException("null cannot be a parameter for xml-rpc yet!");
		} else {
			throw new IllegalArgumentException(o.getClass().getSimpleName()
				+ " is not supportet by XML-RPC. Maybe you forgot to activate value translation?");
		}
	}

	public static String serialize(int i) {
		StringBuilder sb = new StringBuilder();
		sb.append("<value><int>");
		sb.append(String.valueOf(i));
		sb.append("</int></value>");

		return sb.toString();
	}

	public static String serialize(boolean b) {
		StringBuilder sb = new StringBuilder();
		sb.append("<value><boolean>");
		if (b) {
			sb.append("1");
		} else {
			sb.append("0");
		}
		sb.append("</boolean></value>");

		return sb.toString();
	}

	public static String serialize(double d) {
		StringBuilder sb = new StringBuilder();
		sb.append("<value><double>");
		sb.append(String.valueOf(d));
		sb.append("</double></value>");

		return sb.toString();
	}

	static String serialize(String string) {
		StringBuilder sb = new StringBuilder();
		sb.append("<value><string>");
		sb.append(StringEscapeUtils.escapeXml10(string));
		sb.append("</string></value>");

		return sb.toString();
	}

	private static String serialize(Date date) {
		String dateString = serializeDate(date);

		StringBuilder sb = new StringBuilder();
		sb.append("<value><dateTime.iso8601>");
		sb.append(StringEscapeUtils.escapeXml10(dateString));
		sb.append("</dateTime.iso8601></value>");
		return sb.toString();
	}

	public static String serializeDate(Date date) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		return sdf.format(date);
	}

	private static String serialize(byte[] bb) {
		StringBuilder sb = new StringBuilder();
		sb.append("<value><base64>");
		sb.append(StringEscapeUtils.escapeXml10(Base64.encodeBytes(bb)));
		sb.append("</base64></value>");

		return sb.toString();
	}

	private static String serialize(List<? extends Object> a) {
		StringBuilder sb = new StringBuilder();
		sb.append("<value><array><data>");
		for (Object o : a) {
			sb.append(serialize(o));
		}
		sb.append("</data></array></value>");

		return sb.toString();
	}

	private static String serialize(Map<? extends Object, ? extends Object> m) {
		StringBuilder sb = new StringBuilder();
		sb.append("<value><struct>");
		for (Map.Entry<? extends Object, ? extends Object> e : m.entrySet()) {
			sb.append("<member>");
			sb.append("<name>");
			sb.append(StringEscapeUtils.escapeXml10(e.getKey().toString()));
			sb.append("</name>");
			sb.append(serialize(e.getValue()));
			sb.append("</member>");
		}
		sb.append("</struct></value>");

		return sb.toString();
	}
}
