/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.http;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.client.OutgoingRequest;
import de.starface.com.rpc.client.OutgoingRequestProcessor;
import de.starface.com.rpc.xmlrpc.XmlRpcOutgoingRequestProcessor;
import de.starface.com.rpc.xmlrpc.XmlRpcResponse;

import org.apache.commons.httpclient.HttpConnection;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.contrib.ssl.EasySSLProtocolSocketFactory;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.httpclient.protocol.Protocol;
import org.apache.commons.httpclient.protocol.ProtocolSocketFactory;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;

/**
 * A XmlRpcRequestCaller for the usage of a HTTP transport channel
 *
 * @author may
 *
 * @see OutgoingRequestProcessor
 */
public class HttpXmlRpcOutgoingRequestProcessor extends XmlRpcOutgoingRequestProcessor<URL> {
	public int HTTP_CONNECTION_TIMEOUT = 3000;

	private static final Logger log = LogManager.getLogger(HttpXmlRpcOutgoingRequestProcessor.class);

	private final URL callbackServerToken;

	private static final Timer timeoutTimer = new Timer("XML-RPC HTTP Timeout", true);

	private static int timeoutTimerTaskCounter = Integer.MAX_VALUE;

	private static class TimeoutTimerTask extends TimerTask {
		private HttpMethod httpMethod;

		TimeoutTimerTask(HttpMethod httpMethod) {
			this.httpMethod = httpMethod;
		}

		@Override
		public void run() {
			try {
				log.debug("Aborting XML-RPC call over HTTP due to timeout.");
				httpMethod.abort();
			} catch (Exception e) {
				log.error("Exception in TimeoutTimerTask", e);
			} finally {
				httpMethod = null;
			}
		}

		@Override
		public boolean cancel() {
			if (super.cancel()) {
				httpMethod = null;
				return true;
			}
			return false;
		}
	}

	private TimeoutTimerTask createTimeoutOnHttpMethod(HttpMethod httpMethod, long timeoutInMs) {
		TimeoutTimerTask timeoutTimerTask = new TimeoutTimerTask(httpMethod);
		timeoutTimer.schedule(timeoutTimerTask, timeoutInMs);
		if (++timeoutTimerTaskCounter % 100 == 0) {
			timeoutTimer.purge();
		}
		return timeoutTimerTask;
	}

	HttpXmlRpcOutgoingRequestProcessor(URL callbackServerToken) {
		this.callbackServerToken = callbackServerToken;
	}

	/**
	 * @see de.starface.com.rpc.client.OutgoingRequestProcessorBase#processRequest()
	 */
	@Override
	protected void processRequest() throws RpcException {
		URL targetUrl = createConnectionUrl(request);
		HttpXmlRpcRequest req = HttpXmlRpcRequest.createOutgoingRequest(request.getRpcMethodName(),
			request.getParameters());

		HttpState state = new HttpState();
		HttpConnection con = createConnection(targetUrl);

		try {
			con.open();
		} catch (IOException e) {
			throw new RpcException(RpcException.Codes.Client.TRANSPORT_FAILURE, "Could not open http connection.",
				e);
		}

		if (log.isTraceEnabled()) {
			log.trace("Sending XML-RPC call: " + req.toXml());
		}
		int statusCode;

		PostMethod method = new PostMethod(targetUrl.getPath());
		try {
			method.setQueryString(targetUrl.getParams());
			StringRequestEntity entity = new StringRequestEntity(req.toXml(), "text/xml", "UTF-8");
			method.setRequestEntity(entity);
			TimeoutTimerTask timeout = createTimeoutOnHttpMethod(method, getTimeoutForCall());
			statusCode = method.execute(state, con);
			timeout.cancel();
		} catch (Exception e) {
			log.debug("Could not send XML-RPC call over HTTP", e);
			throw new RpcException(RpcException.Codes.Client.TRANSPORT_FAILURE,
				"Could not send XML-RPC call over http", e);
		}

		XmlRpcObject result = null;
		if (statusCode != HttpStatus.SC_OK) {
			// Transport reported error
			throw new RpcException(RpcException.Codes.Client.TRANSPORT_FAILURE, "HTTP Error: " + statusCode);
		} else {
			HttpXmlRpcObjectParser parser = new HttpXmlRpcObjectParser();
			try {
				result = parser.parseStreamAsClient(method.getResponseBodyAsStream());
			} catch (IOException e) {
				throw new RpcException(RpcException.Codes.Client.TRANSPORT_FAILURE,
					"Could not parse HTTP response", e);
			}
			if (result != null && log.isTraceEnabled()) {
				log.trace("Received XML-RPC result (H): " + result.toXml());
			}
		}

		// method.releaseConnection();
		con.close();

		if (result instanceof XmlRpcResponse) {
			XmlRpcResponse response = (XmlRpcResponse) result;
			throwExceptionOnXmlRpcFault(response);
			request.setReturnValue(response.getResult());
		} else if (result == null) {
			throw new RpcException(RpcException.Codes.Client.NO_RESPONSE_FROM_SERVER,
				"No response from XML-RPC server.");
		} else {
			// the result was complete gibberish
			throw new RpcException(RpcException.Codes.Client.TRANSPORT_FAILURE, "HTTP response type "
				+ result.getClass().getName() + " unexpected.");
		}
	}

	private URL createConnectionUrl(OutgoingRequest<URL> request) {
		URL connectionUrl = request.getTargetTransportToken();
		if (callbackServerToken != null && !connectionUrl.hasCallbackUrl()) {
			connectionUrl = connectionUrl.clone();
			connectionUrl.setCallbackUrl(callbackServerToken);
		}
		return connectionUrl;
	}

	/**
	 * Creates a http(s) connection based on the known URL properties
	 *
	 * @param url
	 * @return
	 */
	private HttpConnection createConnection(URL url) {
		HttpConnection con = new HttpConnection(url.getHost(), url.getPort());
		con.getParams().setConnectionTimeout(HTTP_CONNECTION_TIMEOUT);

		if (url.getType().equals(URL.Protocol.https)) {
			Protocol https = new Protocol("easyhttps", (ProtocolSocketFactory) new EasySSLProtocolSocketFactory(),
				url.getPort());
			con.setProtocol(https);
			// set to http otherwise
		}
		return con;
	}
}
