/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.http;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.xmlrpc.XmlRpcObjectSerializer;
import de.starface.com.rpc.xmlrpc.XmlRpcResponse;

import java.util.HashMap;
import java.util.LinkedHashMap;

/**
 * A {@link XmlRpcObject} to represent a XML-RPC response (including faults) via HTTP
 *
 * @author may
 *
 */
public class HttpXmlRpcResponse implements XmlRpcObject, XmlRpcResponse {
	private final Object _result;
	private String _serializedResult = null;
	private final boolean _fault;

	private String _faultMessage;
	private int _faultCode;

	public static HttpXmlRpcResponse createOutgoingResponse(Object response) throws RpcException {
		HttpXmlRpcResponse rpcResponse = new HttpXmlRpcResponse(response);
		try {
			rpcResponse.serializeResult();
		} catch (Exception e) {
			throw new RpcException(RpcException.Codes.Server.XML_SERIALIZATION_ERROR,
				"Could not serialize return value of RPC call to XML-RPC.", e);
		}
		return rpcResponse;
	}

	public static HttpXmlRpcResponse createIncomingResponse(Object response) {
		return new HttpXmlRpcResponse(response);
	}

	public static HttpXmlRpcResponse createOutgoingFaultResponse(int faultCode, String faultString) {
		HttpXmlRpcResponse response = new HttpXmlRpcResponse(faultCode, faultString);
		response.serializeResult();
		return response;
	}

	public static HttpXmlRpcResponse createIncomingFaultResponse(int faultCode, String faultString) {
		return new HttpXmlRpcResponse(faultCode, faultString);
	}

	/**
	 * Constructor for a valid method response as specified in <a
	 * href="http://www.xmlrpc.com/spec">http://www.xmlrpc.com/spec</a>
	 *
	 * @param response
	 *        The result. See {@link XmlRpcObjectSerializer} for supported Object types
	 */
	private HttpXmlRpcResponse(Object response) {
		if (response == null) {
			throw new IllegalArgumentException("XmlRpc does not allow the response to be null!");
		}

		_result = response;
		_fault = false;

		serializeResult();
	}

	private void serializeResult() {
		if (_result != null) {
			_serializedResult = XmlRpcObjectSerializer.serialize(_result);
		}
	}

	/**
	 * Constructor for a faulty method response as specified in <a
	 * href="http://www.xmlrpc.com/spec">http://www.xmlrpc.com/spec</a>
	 *
	 * @param faultCode
	 *        The fault code
	 * @param faultString
	 *        The fault string
	 */
	private HttpXmlRpcResponse(int faultCode, String faultString) {
		if (faultString == null) {
			faultString = "";
		}
		_fault = true;
		HashMap<String, Object> faultMap = new LinkedHashMap<>(2);
		faultMap.put("faultCode", Integer.valueOf(faultCode));
		faultMap.put("faultString", faultString);
		_result = faultMap;
		_faultCode = faultCode;
		_faultMessage = faultString;

		// might throw an Illegal argument exception
		serializeResult();
	}

	/**
	 * @return The response
	 */
	@Override
	public String toXml() {
		StringBuilder sb = new StringBuilder();
		sb.append("<?xml version=\"1.0\"?>");
		sb.append("<methodResponse>");
		if (_fault) {
			if (_serializedResult == null) {
				throw new IllegalStateException("This is not an outgoing XML-RPC response.");
			}
			sb.append("<fault>");
			sb.append(_serializedResult);
			sb.append("</fault>");
		} else {
			if (_result != null) {
				if (_serializedResult == null) {
					throw new IllegalStateException("This is not an outgoing XML-RPC response.");
				}
				sb.append("<params>");
				sb.append("<param>");
				sb.append(_serializedResult);
				sb.append("</param>");
				sb.append("</params>");
			}
		}
		sb.append("</methodResponse>");
		return sb.toString();
	}

	/**
	 * @return Returns the response.
	 */
	@Override
	public Object getResult() {
		return _result;
	}

	/**
	 * @return Returns the fault.
	 */
	@Override
	public boolean isFault() {
		return _fault;
	}

	/**
	 * @return Returns the faultMessage.
	 */
	@Override
	public String getFaultMessage() {
		return _faultMessage;
	}

	/**
	 * @return Returns the faultCode.
	 */
	@Override
	public int getFaultCode() {
		return _faultCode;
	}
}
