/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.http;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.RpcException.Codes.Server;
import de.starface.com.rpc.annotation.RpcAuthRequired;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter;
import de.starface.com.rpc.common.authentication.RpcAuthTokenConverterFactory;
import de.starface.com.rpc.server.IncomingRequestProcessor;
import de.starface.com.rpc.server.RequestExecutor;
import de.starface.com.rpc.server.RpcObjectRegistry;
import de.starface.com.rpc.xmlrpc.XmlRpcRequestExecutor;
import de.starface.com.rpc.xmlrpc.http.URL.Protocol;
import de.starface.security.auth.StarfaceAuthReporter;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.net.InetAddress;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Concrete implementation of a XML-RPC server which implements a java http-servlet for transport.
 *
 * <p>
 * The implementation of the servlet follows <a
 * href="http://java.sun.com/products/servlet/2.5/docs/servlet-2_5-mr2/index.html">Sun Servlet API
 * 2.5</a> specifications.
 * </p>
 *
 * <p>
 * This implementation currently does not allow to set an explicit processor registry for call
 * processors. The request executor will use the default registry
 * {@link de.starface.com.rpc.server.RpcObjectRegistry#DEFAULT_REGISTRY} instead. Furthermore the
 * {@link RpcAuthTokenConverter} must be set via init-params in the {@code web.xml} file. If no such
 * parameter is provided, the servlet will only process requests for non-auth
 * {@link IncomingRequestProcessor}s.
 * </p>
 *
 * <p>
 * The servlet provides a {@link RpcTransportToken} in form of a {@link URL} which tries to
 * determine to identify the communication partner by a callback-URL
 * </p>
 *
 * @author may
 *
 */
public class XmlRpcServlet extends HttpServlet {
	private Logger log = LogManager.getLogger(XmlRpcServlet.class);

	private RpcAuthTokenConverter<? super URL, ? extends RpcAuthToken> authTokenConverter;

	private RpcAuthTokenConverterFactory<? super URL> authTokenConverterFactory;

	private RpcObjectRegistry registry;

	private StarfaceAuthReporter authReporter = null;

	private static final long serialVersionUID = 797591838488962107L;

	public static XmlRpcServlet createWithAuthenticationEnabled(RpcAuthTokenConverterFactory<? super URL> authTokenConverterFactory) {
		XmlRpcServlet xmlRpcServlet = new XmlRpcServlet();
		xmlRpcServlet.authTokenConverterFactory = authTokenConverterFactory;
		return xmlRpcServlet;
	}

	/**
	 * Used here to set the corresponding auth token converter
	 *
	 * @see javax.servlet.GenericServlet#init(javax.servlet.ServletConfig)
	 * @see RpcAuthTokenConverter
	 */
	@Override
	public void init(ServletConfig config) throws ServletException {
		String factoryClassName = config.getInitParameter("authConverterFactory");
		if (StringUtils.isNotBlank(factoryClassName)) {
			createAuthTokenConverterFactory(factoryClassName);
		}

		initAuthReporter(config);

		super.init(config);
	}

	private void initAuthReporter(ServletConfig config) {
		try {
			String useAuthReporter = config.getInitParameter("useAuthenticationReporter");
			if (Boolean.valueOf(useAuthReporter)) {
				authReporter = new StarfaceAuthReporter("UCI", 4444, InetAddress.getLocalHost());
			}
		} catch (Exception e) {
			log.warn(
				"Initialization of StarfaceAuthReporter failed. You are not protected against attacks via xmlrpc!",
				e);
			authReporter = null;
		}
	}

	private void reportAuthentication(HttpServletRequest request, boolean granted) {
		if (authReporter != null) {
			authReporter.sendAuthReport(request.getRemoteAddr(), request.getParameter("de.vertico.starface.user"),
				granted);
		}
	}

	/**
	 * Receive the post and perfom the xml-rpc logic
	 *
	 * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		if (log.isTraceEnabled()) {
			log.trace("URI: " + req.getRequestURI());
			log.trace("Query: " + req.getQueryString());
			log.trace("Type: " + req.getContentType());
			log.trace("Host: " + req.getRemoteHost());
			log.trace("Length: " + req.getContentLength());
		}

		final RequestExecutor<URL> executor = new XmlRpcRequestExecutor<>();
		HttpXmlRpcResponse res;

		try {
			executor.setAuthTokenConverter(getAuthTokenConverter());
			executor.setServerObjectRegistry(registry);

			HttpXmlRpcObjectParser parser = new HttpXmlRpcObjectParser();

			HttpXmlRpcRequest httpRequest = (HttpXmlRpcRequest) parser.parseStreamAsServer(req.getInputStream());

			Object returnValue = executor.execute(httpRequest.getMethodName(), httpRequest.getParameters(),
				determineUrlFromCaller(req));

			res = HttpXmlRpcResponse.createOutgoingResponse(returnValue);
			reportAuthentication(req, true);
		} catch (Exception e) {
			if (e.getCause() != null && "UcpNotConnectedException".equals(e.getCause().getClass().getSimpleName())) {
				if (log.isInfoEnabled()) {
					log.info(e.getMessage() + ": " + e.getCause().getMessage() + " Request was " + req.getRequestURI());
				}
			}  else {
				log.warn("The RPC procedure call " + req.getRequestURI() + " could not be completed.", e);
			}

			RpcException rpcException = executor.convertExceptionToRpcException(e);
			if (rpcException.getCode() == Server.AUTH_CONVERTER_EXCEPTION
				|| rpcException.getCode() == Server.AUTH_UNKNOWN) {
				reportAuthentication(req, false);
			}

			res = HttpXmlRpcResponse.createOutgoingFaultResponse(rpcException.getCode(), rpcException.getMessage());
		}
		try {
			String xmlResponse = res.toXml();
			if (log.isTraceEnabled()) {
				log.trace("Sending xml-rpc result: " + xmlResponse);
			}

			resp.setStatus(HttpServletResponse.SC_OK);
			if (StringUtils.contains(req.getHeader("Accept"), "application/xml")) {
				resp.setContentType("application/xml; charset=utf-8");
			} else {
				resp.setContentType("text/xml; charset=utf-8");
			}

			byte[] bytesToWrite = xmlResponse == null ? new byte[0] : xmlResponse.getBytes("UTF-8");

			resp.getOutputStream().write(bytesToWrite);
			resp.setContentLength(bytesToWrite.length);

			resp.getOutputStream().flush();
			resp.getOutputStream().close();
		} catch (IOException e) {
			logException("Sending response failed", e);
			throw e;
		} catch (RuntimeException e) {
			logException("Sending response failed", e);
			throw e;
		}
	}

	private static URL determineUrlFromCaller(HttpServletRequest request) {
		URL callerUrl = URL.createCallbackUrlFromRequestParameters(request.getParameterMap());
		if (StringUtils.isEmpty(callerUrl.getHost())) {
			callerUrl.setType(request.isSecure() ? Protocol.https : Protocol.http);
			callerUrl.setHost(request.getRemoteHost());
			// We do not set the port here intentionally so that authentication of clients
			// works even when they send every outgoing request on another port 
			// callerUrl.setPort(request.getRemotePort());
			callerUrl.setPath("/");
		}
		return callerUrl;
	}

	private void logException(String messagePrefix, Exception e) {
		if (log.isDebugEnabled()) {
			log.warn(messagePrefix, e);
		} else {
			log.info(messagePrefix + ": " + e.getMessage());
		}
	}

	@SuppressWarnings("unchecked")
	private void createAuthTokenConverterFactory(String className) {
		try {
			Class<?> clazz = Class.forName(className);
			authTokenConverterFactory = (RpcAuthTokenConverterFactory<URL>) clazz.newInstance();
		} catch (Exception e) {
			log.error("Could not create auth token converter factory: " + className);
		}
	}

	private RpcAuthTokenConverter<? super URL, ? extends RpcAuthToken> getAuthTokenConverter() throws RpcException {
		if (authTokenConverter != null) {
			return authTokenConverter;
		}
		if (authTokenConverterFactory != null) {
			RpcAuthTokenConverter<? super URL, ? extends RpcAuthToken> converter = authTokenConverterFactory.getConverter();

			if (converter == null) {
				throw new RpcException(RpcException.Codes.Server.AUTH_CONVERTER_MISSING,
					"No auth token converter could be obtained from factory");
			}

			// do the check which cannot be done by java generics
			if (converter.getTransportType().isAssignableFrom(URL.class)) {
				return converter;
			} else {
				throw new RpcException(RpcException.Codes.Server.AUTH_CONVERTER_MISSING,
					"The auth token converter for type " + converter.getTransportType().getName()
						+ " cannot be used to convert transport tokens of type " + URL.class.getName());
			}
		}
		return null;
	}

	/**
	 * Enables authentication for this XmlRpcServlet using the given {@link RpcAuthTokenConverter}.
	 * which converts {@link RpcTransportToken} to {@link RpcAuthToken}. Per default authentication
	 * is disabled.
	 *
	 * <p>
	 * Once authentication is enabled the underlying transport channel can be used to authenticate
	 * the caller by converting the caller-unique transport token to a {@link RpcAuthToken} which
	 * can be used for authentication using the {@link RpcAuthRequired} annotation.
	 * </p>
	 *
	 * @param authTokenConverter
	 *        The {@link RpcAuthTokenConverter} which should be used by this servlet
	 *
	 * @see RpcAuthTokenConverter#convert(RpcAuthToken)
	 */
	public void enableAuthentication(RpcAuthTokenConverter<? super URL, ? extends RpcAuthToken> authTokenConverter) {
		this.authTokenConverter = authTokenConverter;
	}

	/**
	 * Check whether this XmlRpcServlet is enabled to verify the authentication of RPC calls prior
	 * to invoking the corresponding {@link IncomingRequestProcessor}s. Per default authentication
	 * is disabled.
	 *
	 * @return <code>true</code> if authentication is enabled, <code>false</code> otherwise.
	 */
	public boolean isAuthenticationEnabled() {
		return this.authTokenConverter != null || this.authTokenConverterFactory != null;
	}

	/**
	 * Disable authentication for this XmlRpcServlet if it has previously been enabled.
	 */
	public void disableAuthentication() {
		this.authTokenConverter = null;
		this.authTokenConverterFactory = null;
	}

	/**
	 * Retrieve the {@link RpcObjectRegistry} that holds the {@link IncomingRequestProcessor}s this
	 * XmlRpcServlet is publishing. If not set the {@link RpcObjectRegistry#DEFAULT_REGISTRY} is
	 * used.
	 *
	 * @return The {@link RpcObjectRegistry} in use or <code>null</code> if not set.
	 */
	public RpcObjectRegistry getRegistry() {
		return registry;
	}

	/**
	 * Sets the {@link RpcObjectRegistry} that holds the {@link IncomingRequestProcessor}s this
	 * XmlRpcServlet is publishing. If set to <code>null</code> the
	 * {@link RpcObjectRegistry#DEFAULT_REGISTRY} is used.
	 *
	 * @param registry
	 *        The {@link RpcObjectRegistry} to use or <code>null</code>.
	 */
	public void setRegistry(RpcObjectRegistry registry) {
		this.registry = registry;
	}
}
