/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.jetty;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.common.authentication.RpcAuthTokenConverter;
import de.starface.com.rpc.server.RpcObjectRegistry;
import de.starface.com.rpc.server.RpcServer;
import de.starface.com.rpc.server.RpcServerBase;
import de.starface.com.rpc.xmlrpc.http.URL;
import de.starface.com.rpc.xmlrpc.http.URL.Protocol;
import de.starface.com.rpc.xmlrpc.http.XmlRpcServlet;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.security.SslSocketConnector;
import org.mortbay.jetty.servlet.Context;
import org.mortbay.jetty.servlet.ServletHolder;

import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * This is a {@link RpcServer} implementation that serves the registered RPC methods for to be
 * called by XML-RPC using the {@link XmlRpcServlet} and the Jetty servlet container.
 *
 * @author tilmann
 *
 */
public class HttpXmlRpcServer extends RpcServerBase<URL> {
	private XmlRpcServlet servlet = new XmlRpcServlet();

	private Server server;

	private String serverHost;

	private String serverPath;

	private int serverPort;

	private Protocol serverProtocol = Protocol.http;

	private String keyStorePath;

	private String keyStorePassword;

	private String trustStorePath;

	private String trustStorePassword;

	private HttpXmlRpcServer() {
		// Just Reducing default visibility
	}

	public static HttpXmlRpcServer createHttpServerWithHostName(String serverHostName, int serverPort,
																String serverPath) {
		Validate.notEmpty(serverHostName, "serverHostName=empty");
		if (serverPort <= 0) {
			throw new IllegalArgumentException("serverPort<=0");
		}
		if (StringUtils.isEmpty(serverPath)) {
			serverPath = "/";
		}
		HttpXmlRpcServer server = new HttpXmlRpcServer();
		server.serverHost = serverHostName;
		server.serverPort = serverPort;
		server.serverPath = serverPath;
		return server;
	}

	public static HttpXmlRpcServer createHttpServerWithAutomaticHostName(int serverPort, String serverPath)
		throws RpcException {
		String serverHostName = determinHostName();
		return createHttpServerWithHostName(serverHostName, serverPort, serverPath);
	}

	private static String determinHostName() throws RpcException {
		InetAddress addr;
		try {
			addr = InetAddress.getLocalHost();
			return addr.getCanonicalHostName();
		} catch (UnknownHostException e) {
			throw new RpcException(RpcException.Codes.Server.TRANSPORT_FAILURE, "Could not determine host name", e);
		}
	}

	public static HttpXmlRpcServer createHttpsServerWithHostName(String serverHostName, int serverPort,
																 String serverPath, String keyStorePath,
																 String keyStorePassword, String trustStorePath,
																 String trustStorePassword) {
		Validate.notEmpty(keyStorePath, "keyStorePath=empty");
		Validate.notNull(keyStorePassword, "keyStorePassword=null");
		Validate.notEmpty(trustStorePath, "trustStorePath=empty");
		Validate.notNull(trustStorePassword, "trustStorePassword=null");
		HttpXmlRpcServer server = createHttpServerWithHostName(serverHostName, serverPort, serverPath);
		server.keyStorePath = keyStorePath;
		server.keyStorePassword = keyStorePassword;
		server.trustStorePath = trustStorePath;
		server.trustStorePassword = trustStorePassword;
		server.serverProtocol = Protocol.https;
		return server;
	}

	public static HttpXmlRpcServer createHttpsServerWithAutomaticHostName(int serverPort, String serverPath,
																		  String keyStorePath,
																		  String keyStorePassword,
																		  String trustStorePath,
																		  String trustStorePassword)
		throws RpcException {
		String serverHostName = determinHostName();
		return createHttpsServerWithHostName(serverHostName, serverPort, serverPath, keyStorePath,
			keyStorePassword, trustStorePath, trustStorePassword);
	}

	/**
	 * {@inheritDoc}
	 *
	 * @see de.starface.com.rpc.server.RpcServerBase#shutdownImpl()
	 */
	@Override
	protected void shutdownImpl() throws RpcException {
		try {
			server.stop();
		} catch (Exception e) {
			throw new RpcException(RpcException.Codes.Server.TRANSPORT_FAILURE, "Could not stop Jetty server.", e);
		} finally {
			server = null;
		}
	}

	/**
	 * {@inheritDoc}
	 *
	 * @see de.starface.com.rpc.server.RpcServerBase#startupImpl()
	 */
	@Override
	protected void startupImpl() throws RpcException {
		try {
			createNewServer();
			deployRpcServlet();
			server.start();
		} catch (Exception e) {
			server = null;
			throw new RpcException(RpcException.Codes.Server.TRANSPORT_FAILURE, "Could not start Jetty server.", e);
		}
	}

	private void createNewServer() throws RpcException {
		if (serverProtocol == Protocol.https || serverProtocol == Protocol.ehttps) {
			createNewHttpsServer();
		} else {
			createNewHttpServer();
		}
	}

	private void createNewHttpServer() {
		server = new Server(serverPort);
	}

	private void createNewHttpsServer() {
		SslSocketConnector connector = new SslSocketConnector();
		connector.setPort(serverPort);
		connector.setKeystore(keyStorePath);
		connector.setKeyPassword(keyStorePassword);
		connector.setTruststore(trustStorePath);
		connector.setTrustPassword(trustStorePassword);
		server = new Server();
		server.addConnector(connector);
	}

	private void deployRpcServlet() {
		Context root = new Context(server, "/", Context.SESSIONS);
		root.addServlet(new ServletHolder(servlet), serverPath);
	}

	/**
	 * {@inheritDoc}
	 *
	 * @see de.starface.com.rpc.server.RpcServerBase#enableAuthentication(RpcAuthTokenConverter)
	 */
	@Override
	public void enableAuthentication(RpcAuthTokenConverter<? super URL, ? extends RpcAuthToken> authTokenConverter) {
		super.enableAuthentication(authTokenConverter);
		servlet.enableAuthentication(authTokenConverter);
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see de.starface.com.rpc.server.RpcServerBase#isAuthenticationEnabled()
	 */
	@Override
	public boolean isAuthenticationEnabled() {
		return servlet.isAuthenticationEnabled();
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see de.starface.com.rpc.server.RpcServerBase#disableAuthentication()
	 */
	@Override
	public void disableAuthentication() {
		super.disableAuthentication();
		servlet.disableAuthentication();
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see de.starface.com.rpc.server.RpcServerBase#getRegistry()
	 */
	@Override
	public RpcObjectRegistry getRegistry() {
		return servlet.getRegistry();
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see de.starface.com.rpc.server.RpcServerBase#setRegistry(de.starface.com.rpc.server.RpcObjectRegistry)
	 */
	@Override
	public void setRegistry(RpcObjectRegistry registry) {
		super.setRegistry(registry);
		servlet.setRegistry(registry);
	}

	/**
	 * {@inheritDoc}
	 *
	 * @see de.starface.com.rpc.server.RpcServer#getServerAddress()
	 */
	@Override
	public URL getServerAddress() {
		return new URL(serverProtocol, serverHost, serverPort, serverPath);
	}
}
