/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.smack;

import de.starface.com.rpc.common.RpcTransportToken;

import org.apache.commons.lang3.Validate;
import org.jivesoftware.smack.Connection;
import org.jivesoftware.smack.util.StringUtils;

/**
 * Implementation of a {@link RpcTransportToken} for XML-RPC via XMPP
 *
 * @author may
 *
 * @see RpcTransportToken
 */
public class JID implements RpcTransportToken {

	/** self explaining local fields */
	private String _user;
	private String _server;
	private String _resource;

	/**
	 * Creates a new JID for the given target user at the XMPP service the connection belongs to.
	 *
	 * @param xmppConnection
	 *        the to retrieve the XMPP service name from
	 * @param targetUsername
	 *        the user name of the newly created JID
	 * @param targetResource
	 *        the resource of the newly created JID
	 * @return {@code null} if the connection has never been established the JID of the target
	 *         otherwise
	 */
	public static JID createJidForTarget(Connection xmppConnection, String targetUsername, String targetResource) {
		Validate.notEmpty(targetUsername, "targetUsername=empty");
		final JID connectionJid = fetchFromConnection(xmppConnection);
		if (connectionJid != null) {
			String targetJid = targetUsername + "@" + connectionJid.getServer();
			if (!org.apache.commons.lang3.StringUtils.isEmpty(targetResource)) {
				targetJid += "/" + targetResource;
			}
			return new JID(targetJid);
		} else {
			return null;
		}
	}

	/**
	 * Retrieves the JID the of the XMPP connection
	 *
	 * @param xmppConnection
	 *        the connection to retrieve the JID for
	 * @return {@code null} if the connection has never been established the JID of the connection
	 *         otherwise
	 */
	public static JID fetchFromConnection(Connection xmppConnection) {
		Validate.notNull(xmppConnection, "xmppConnection=null");
		final String user = xmppConnection.getUser();
		if (user != null) {
			return new JID(user);
		} else {
			return null;
		}
	}

	/**
	 * Default constructor which parses a jid from the given string
	 *
	 * @param string
	 *        The string representation of a valid JID
	 *
	 * @throws IllegalArgumentException
	 *         If provided JID is invalid
	 */
	public JID(String string) {
		_user = StringUtils.parseName(string);
		_server = StringUtils.parseServer(string);
		_resource = StringUtils.parseResource(string);
		Validate.notEmpty(_server, "Invalid JID");
	}

	/**
	 * @return Returns the user.
	 */
	public String getUser() {
		return _user;
	}

	/**
	 * @param user
	 *        The user to set.
	 */
	public void setUser(String user) {
		Validate.notEmpty(user, "user=empty");
		_user = user;
	}

	/**
	 * @return Returns the service.
	 */
	public String getServer() {
		return _server;
	}

	/**
	 * @param server
	 *        The server to set.
	 */
	public void setServer(String server) {
		Validate.notEmpty(server, "server=empty");
		_server = server;
	}

	/**
	 * @return Returns the resource.
	 */
	public String getResource() {
		return _resource;
	}

	/**
	 * @param resource
	 *        The resource to set.
	 */
	public void setResource(String resource) {
		_resource = resource;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();

		if (!org.apache.commons.lang3.StringUtils.isEmpty(_user)) {
			sb.append(_user);
			sb.append("@");
		}

		sb.append(_server);

		if (!org.apache.commons.lang3.StringUtils.isEmpty(_resource)) {
			sb.append("/");
			sb.append(_resource);
		}

		return sb.toString();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((_resource == null) ? 0 : _resource.hashCode());
		result = prime * result + ((_server == null) ? 0 : _server.hashCode());
		result = prime * result + ((_user == null) ? 0 : _user.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final JID other = (JID) obj;
		if (_resource == null) {
			if (other._resource != null) {
				return false;
			}
		} else if (!_resource.equals(other._resource)) {
			return false;
		}
		if (_server == null) {
			if (other._server != null) {
				return false;
			}
		} else if (!_server.equals(other._server)) {
			return false;
		}
		if (_user == null) {
			if (other._user != null) {
				return false;
			}
		} else if (!_user.equals(other._user)) {
			return false;
		}
		return true;
	}

	@Override
	public String getIdentifierString() {
		StringBuilder sb = new StringBuilder();

		if (!org.apache.commons.lang3.StringUtils.isEmpty(_user)) {
			sb.append(_user);
			sb.append("@");
		}

		sb.append(_server);

		if (!org.apache.commons.lang3.StringUtils.isEmpty(_resource)) {
			sb.append("/");
			sb.append(_resource);
		}

		return sb.toString();
	}
}
