/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.xmlrpc.smack;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.xmlrpc.XmlRpcObjectSerializer;

import org.jivesoftware.smack.packet.IQ;

/**
 * An XMPP-XML-RPC-Packet as specified in XEP-0009 <a
 * href="http://www.xmpp.org/extensions/xep-0009.html"
 * >http://www.xmpp.org/extensions/xep-0009.html</a>
 *
 * This packet represents the method call.
 *
 * @author may
 *
 */
public class XmlRpcRequestPacket extends IQ {
	private final String _methodName;
	private final Object[] _parameters;
	private String[] _serializedParameters;

	public static XmlRpcRequestPacket createOutgoingRequest(String method, Object[] params) throws RpcException {
		XmlRpcRequestPacket packet = new XmlRpcRequestPacket(method, params);
		try {
			packet.serializeParameters();
		} catch (Exception e) {
			throw new RpcException(RpcException.Codes.Client.XML_SERIALIZATION_ERROR,
				"Could not serialize parameter values of RPC call to XML-RPC.", e);
		}
		return packet;
	}

	public static XmlRpcRequestPacket createIncomingRequest(String method, Object[] params) {
		return new XmlRpcRequestPacket(method, params);
	}

	/**
	 * Note: As for code-safety the method name and the array of parameters cannot be modified in
	 * one packet. To modify a rpc request, a new packet has to be created!
	 *
	 * @param method
	 *        the name of the remote function to call
	 * @param params
	 *        Array of Object as parameters for the function call. See
	 *        {@link XmlRpcObjectSerializer} for a list of supported types
	 * @throws IllegalArgumentException
	 *         If the a given parameter is not recognized
	 */
	private XmlRpcRequestPacket(String method, Object[] params) {
		if (method == null) {
			throw new IllegalArgumentException("method name must not be null");
		}

		_methodName = method;
		_parameters = params;
		setType(IQ.Type.SET);
	}

	private void serializeParameters() {
		if (_parameters != null) {
			_serializedParameters = new String[_parameters.length];
			for (int i = 0; i < _parameters.length; i++) {
				_serializedParameters[i] = XmlRpcObjectSerializer.serialize(_parameters[i]);
			}
		}
	}

	/**
	 * @return The query
	 */
	@Override
	public String getChildElementXML() {
		StringBuilder sb = new StringBuilder();
		sb.append("<query xmlns='jabber:iq:rpc'>");
		sb.append("<methodCall>");
		sb.append("<methodName>");
		sb.append(_methodName);
		sb.append("</methodName>");
		if (_parameters != null) {
			if (_serializedParameters == null) {
				serializeParameters();
			}
			sb.append("<params>");
			for (String s : _serializedParameters) {
				sb.append("<param>");
				sb.append(s);
				sb.append("</param>");
			}
			sb.append("</params>");
		}
		sb.append("</methodCall>");
		sb.append("</query>");

		return sb.toString();
	}

	/**
	 * @return Returns the methodName.
	 */
	public String getMethodName() {
		return _methodName;
	}

	/**
	 * @return Returns the parameters.
	 */
	public Object[] getParameters() {
		return _parameters;
	}
}
