/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.xmpp.smack;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jivesoftware.smack.Connection;
import org.jivesoftware.smack.ConnectionConfiguration;
import org.jivesoftware.smack.ConnectionConfiguration.SecurityMode;
import org.jivesoftware.smack.Roster;
import org.jivesoftware.smack.Roster.SubscriptionMode;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.XMPPException;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.util.LinkedList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

/**
 * The XmppConnector can be used to synchronously or asynchronously connect an XMPPConnection.
 * Including connection and authentication. It either connects the connection provided to the
 * constructor call or creates a new connection with the given configuration parameters. If the
 * connection cannot be established repeated connection are made with an interval of one minute till
 * the connection attempt finally succeeds. The connection can be queried from the XmppConnector.
 *
 * @author tilmann
 */
public class XmppConnector {
	/** Used for logging output */
	private static final Logger log = LogManager.getLogger(XmppConnector.class);

	/** The interval in seconds used for connection attempts */
	private static final int RECONNECT_INTERVAL = 60;

	/** The XMPPConnection to be connected */
	private Connection connection;

	/** The name of the XMPP user */
	private String xmppUserName;

	/** The resource of the XMPP user */
	private String xmppResource;

	/** The password of the XMPP user */
	private String xmppPassword;

	/** Should the connection be checked and reconnected */
	private boolean xmppAutoReconnect;

	/** The Timer used for asynchronous connection attempts */
	private Timer reconnectTimer;

	private List<XmppConnectorListener> listenerList;

	/**
	 * Creates a new XmppConnector connecting the given connection using the given xmpp user
	 * details.
	 *
	 * @param connection
	 *        the connection to be connected
	 * @param xmppUserName
	 *        the name of the xmpp user
	 * @param xmppResource
	 *        the resource of the xmpp user
	 * @param xmppPassword
	 *        the password of the xmpp user
	 * @param xmppAutoReconnect
	 *        specifies if the connection should reconnect automatically
	 */
	public XmppConnector(Connection connection, String xmppUserName, String xmppResource, String xmppPassword,
						 boolean xmppAutoReconnect) {
		init(connection, xmppUserName, xmppResource, xmppPassword, xmppAutoReconnect);
	}

	/**
	 * Creates a new XmppConnector that creates a new XMPPConnection with the given configuration
	 * parameters.
	 *
	 * @param xmppUserName
	 *        the name of the xmpp user
	 * @param xmppResource
	 *        the resource of the xmpp user
	 * @param xmppPassword
	 *        the password of the xmpp user
	 * @param xmppServerHostName
	 *        the host name of the xmpp server
	 * @param xmppServerPort
	 *        the port of the xmpp server on the given host
	 * @param xmppUseSSL
	 *        specifies if ssl connection must or must not be used
	 * @param xmppAutoReconnect
	 *        specifies if the connection should reconnect automatically
	 * @param xmppSendPresence
	 *        specifies if the connection will automatically send "availabe" presence
	 */
	public XmppConnector(String xmppUserName, String xmppResource, String xmppPassword, String xmppServerHostName,
						 int xmppServerPort, boolean xmppUseSSL, boolean xmppAutoReconnect,
						 boolean xmppSendPresence) {
		init(xmppUserName, xmppResource, xmppPassword, xmppServerHostName, xmppServerPort, xmppUseSSL,
			xmppAutoReconnect, xmppSendPresence);
	}

	/**
	 * Initialize this XmppConnector creating a new XMPPConnection with the given configuration
	 * parameters.
	 *
	 * @param xmppUserName
	 *        the name of the xmpp user
	 * @param xmppResource
	 *        the resource of the xmpp user
	 * @param xmppPassword
	 *        the password of the xmpp user
	 * @param xmppServerHostName
	 *        the host name of the xmpp server
	 * @param xmppServerPort
	 *        the port of the xmpp server on the given host
	 * @param xmppUseSSL
	 *        specifies if ssl connection must or must not be used
	 * @param xmppAutoReconnect
	 *        specifies if the connection should reconnect automatically
	 * @param xmppSendPresence
	 *        specifies if the connection will automatically send "availabe" presence
	 */
	private void init(String xmppUserName, String xmppResource, String xmppPassword, String xmppServerHostName,
					  int xmppServerPort, boolean xmppUseSSL, boolean xmppAutoReconnect, boolean xmppSendPresence) {
		Validate.notEmpty(xmppServerHostName, "xmppServerHostName=empty");
		Validate.isTrue(xmppServerPort > 0, "xmppServerPort<=0");
		ConnectionConfiguration connectionConfiguration = new ConnectionConfiguration(xmppServerHostName,
			xmppServerPort);
		if (xmppUseSSL) {
			connectionConfiguration.setSecurityMode(SecurityMode.required);
			connectionConfiguration.setSASLAuthenticationEnabled(false);
		} else {
			connectionConfiguration.setSecurityMode(SecurityMode.disabled);
			connectionConfiguration.setSASLAuthenticationEnabled(false);
		}

		// We do this because build in reconnection is not as reliable as needed
		// So we have implemented our own reconnection mechanism
		connectionConfiguration.setReconnectionAllowed(false);

		// We do not use rosters for XML-RPC
		connectionConfiguration.setRosterLoadedAtLogin(false);

		// Set if the connection will sent "available" presence automatically
		// When presence is "not available" XMPP file transfer will not work!
		connectionConfiguration.setSendPresence(xmppSendPresence);

		XMPPConnection connection = new XMPPConnection(connectionConfiguration);
		init(connection, xmppUserName, xmppResource, xmppPassword, xmppAutoReconnect);
	}

	/**
	 * Initialize this XmppConnector with the given connection using the given xmpp user details.
	 *
	 * @param connection
	 *        the connection to be connected
	 * @param xmppUserName
	 *        the name of the xmpp user
	 * @param xmppResource
	 *        the resource of the xmpp user
	 * @param xmppPassword
	 *        the password of the xmpp user
	 * @param xmppAutoReconnect
	 *        specifies if the connection should reconnect automatically
	 */
	private void init(Connection connection, String xmppUserName, String xmppResource, String xmppPassword,
					  boolean xmppAutoReconnect) {
		Validate.notEmpty(xmppUserName, "xmppUserName=empty");
		Validate.notNull(xmppPassword, "xmppPassowrd=null");
		try {
			this.xmppUserName = URLEncoder.encode(xmppUserName, "utf-8");
		} catch (UnsupportedEncodingException e) {
			this.xmppUserName = xmppUserName.replaceAll(" ", "\\\\20");
		}
		this.xmppResource = xmppResource;
		this.xmppPassword = xmppPassword;
		this.xmppAutoReconnect = xmppAutoReconnect;
		this.connection = connection;
	}

	/**
	 * Tries to connect to the XMPP server and to authenticate the user for the connection.
	 *
	 * @throws XmppConnectionException
	 *         if the connection could not be established
	 */
	private synchronized void doConnect() throws XmppConnectionException {
		if (!isConnected()) {
			// don't allow subscriptions
			Roster.setDefaultSubscriptionMode(SubscriptionMode.reject_all);

			if (!connection.isConnected()) {
				log.debug("doConnect: Trying to connect with XMPP server...");
				try {
					connection.connect();
				} catch (XMPPException e) {
					Throwable cause = e.getCause();
					if (cause == null) {
						cause = e;
					}
					final int code;
					if (cause instanceof UnknownHostException) {
						code = XmppConnectionException.Codes.UNKNOWN_HOST;
					} else {
						code = XmppConnectionException.Codes.IO_ERROR;
					}
					onConnectionFailed(cause, "", code);
				}
			}
			if (!connection.isAuthenticated()) {
				log.debug("doConnect: Trying to authenticate with XMPP server...");
				try {
					if (StringUtils.isEmpty(xmppResource)) {
						connection.login(xmppUserName, xmppPassword);
					} else {
						connection.login(xmppUserName, xmppPassword, xmppResource);
					}
				} catch (XMPPException e) {
					Throwable cause = e.getCause();
					if (cause == null) {
						cause = e;
					}
					disconnectWithoutTimerStop();
					onConnectionFailed(cause, "Authentication failed.",
						XmppConnectionException.Codes.AUTHENTICATION_FAILED);
				}
				// The following can happen if the connection has been lost straight after it has been established.
				catch (IllegalStateException e) {
					disconnectWithoutTimerStop();
					onConnectionFailed(null, "Connection has been lost straight after beeing established.",
						XmppConnectionException.Codes.IO_ERROR);
				}
			}
			if (isConnected()) {
				onConnectionSucceeded();
			}
		}
	}

	private synchronized void onConnectionSucceeded() {
		log.info("doConnect: Successfully connected to XMPP server for user " + xmppUserName);
		sendConnectionEvent(XmppConnectionEvent.Codes.CONNECTION_ESTABLISHED);
	}

	private synchronized void onConnectionFailed(Throwable cause, String message, final int code)
		throws XmppConnectionException {
		message = "Could not connect to XMPP server for user " + xmppUserName + ". " + message;

		log.warn(message, cause);

		final XmppConnectionException e2 = new XmppConnectionException(code, message, cause);

		sendConnectionFailure(e2);
		throw e2;
	}

	/**
	 * Asynchronously connect the XMPPConnection managed by this connector. This method immediately
	 * returns and connection attempts are made at an interval of one minute until the connection
	 * can be established. This method can also be used to reconnect the XMPPConnection if
	 * connection had been lost. (Apart from that the connection does reconnect automatically if the
	 * corresponding configuration parameter was set while this connector was created.)
	 */
	public synchronized void startAsyncConnects() {
		startReconnectTimer(false);
	}

	/**
	 * Cancel any outstanding asynchronous connection attempts.
	 */
	public synchronized void abortConnects() {
		stopReconnectTimer();
	}

	/**
	 * Synchronously connect the XMPPConnection managed by this connector. This method can also be
	 * used to reconnect the XMPPConnection if connection had been lost. (Apart from that the
	 * connection does reconnect automatically if the corresponding configuration parameter was set
	 * while this connector was created.)
	 *
	 * @throws XmppConnectionException
	 *         if the connection could not be established on the first try
	 *
	 */
	public synchronized void connect() throws XmppConnectionException {
		doConnect();

		if (xmppAutoReconnect) {
			startReconnectTimer(true);
		}
	}

	/**
	 * First try to synchronously connect the XMPPConnection managed by this connector and return
	 * the connection result. If the connection was not successfully established also start
	 * asynchronous connection attempts. This method can also be used to reconnect the
	 * XMPPConnection if connection had been lost. (Apart from that the connection does reconnect
	 * automatically if the corresponding configuration parameter was set while this connector was
	 * created.)
	 *
	 * @throws XmppConnectionException
	 *         if the connection could not be established on the first try
	 */
	public synchronized void startConnects() throws XmppConnectionException {
		try {
			connect();
		} finally {
			startReconnectTimer(true);
		}
	}

	/**
	 * Cancel any outstanding asynchronous connection attempts and disconnect the XMPPConnection if
	 * it is connected.
	 */
	public synchronized void disconnect() {
		stopReconnectTimer();
		disconnectWithoutTimerStop();
	}

	private synchronized void disconnectWithoutTimerStop() {
		if (connection.isConnected()) {
			log.warn("disconnect: Disconnect from XMPP server.");
			connection.disconnect();
			sendConnectionEvent(XmppConnectionEvent.Codes.CONNECTION_CLOSED);
		}
	}

	/**
	 * Retrieve the XMPPConnection managed by this XmppConnector. The connection returned may or may
	 * not be connected depending on the fact if the connection attempts conducted by this connector
	 * already succeeded and the fact that a connection once connected by this connector may have
	 * been disconnected afterwards.
	 *
	 * @return the Connection
	 */
	public Connection getConnection() {
		return connection;
	}

	/**
	 * Check whether the underlying XMPPConnection is connected to the server and the XMPP user has
	 * been successfully authenticated.
	 *
	 * @return <code>true</code> if connected and authenticated, <code>false</code> otherwise
	 */
	public boolean isConnected() {
		return connection.isConnected() && connection.isAuthenticated();
	}

	/**
	 * Starts the timer task that reconnects the xmpp connection. Optionally it uses an initial
	 * delay before the first reconnect attempt.
	 *
	 * @param initialDelay
	 *        specifies if an initial delay should be used.
	 */
	private void startReconnectTimer(boolean initialDelay) {
		if (reconnectTimer == null) {
			log.debug("startReconnectTimer: Activating reconnect timer.");
			TimerTask reconnectTimerTask = new TimerTask() {
				@Override
				public void run() {
					try {
						doConnect();

						if (!xmppAutoReconnect) {
							stopReconnectTimer();
						}
					} catch (XmppConnectionException e) {
						// info has already been logged by doConnect();
					}
					// catch all exceptions so the Timer won't die
					catch (Exception e) {
						log.error("TimerTask.run: Error in reconnect timer.", e);
					}
				}
			};

			reconnectTimer = new Timer("XmppConnector.reconnectTimer", true);
			long delay = 0;
			if (initialDelay) {
				delay = 1000 * RECONNECT_INTERVAL;
			}
			reconnectTimer.schedule(reconnectTimerTask, delay, 1000 * RECONNECT_INTERVAL);
		}
	}

	/**
	 * Stops the timer task that reconnects the xmpp connection.
	 */
	private void stopReconnectTimer() {
		if (reconnectTimer != null) {
			log.debug("stopReconnectTimer: Deactivating reconnect timer.");
			reconnectTimer.cancel();
			reconnectTimer = null;
		}
	}

	/**
	 * Distributes a connection event to all registered listeners
	 *
	 * @param code
	 *        the connection event to be signaled
	 */
	private synchronized void sendConnectionEvent(int code) {
		if (listenerList != null) {
			final XmppConnectionEvent e = new XmppConnectionEvent(code);
			for (XmppConnectorListener listener : listenerList) {
				listener.onConnectionEvent(this, e);
			}
		}
	}

	/**
	 * Distributes a connection exception to all registered listeners
	 *
	 * @param e
	 *        the exception to be signaled
	 */
	private synchronized void sendConnectionFailure(XmppConnectionException e) {
		if (listenerList != null) {
			for (XmppConnectorListener listener : listenerList) {
				listener.onConnectionFailure(this, e);
			}
		}
	}

	/**
	 * Adds an {@link XmppConnectorListener} to this connectors listener list
	 *
	 * @param listener
	 *        the listener to be added
	 */
	public synchronized void addXmppConnectorListener(XmppConnectorListener listener) {
		Validate.notNull(listener, "listener=null");
		if (listenerList == null) {
			listenerList = new LinkedList<>();
		}
		listenerList.add(listener);
	}

	/**
	 * Removes an {@link XmppConnectorListener} from this connectors listener list
	 *
	 * @param listener
	 *        the listener to be removed
	 */
	public synchronized void removeXmppConnectorListener(XmppConnectorListener listener) {
		Validate.notNull(listener, "listener=null");
		if (listenerList != null) {
			listenerList.remove(listener);
		}
	}
}
