/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.com.rpc.unittests.client;

import de.starface.com.rpc.client.OutgoingRequest;
import de.starface.com.rpc.client.OutgoingRequestProcessor;
import de.starface.com.rpc.client.OutgoingRequestProcessorFactory;
import de.starface.com.rpc.client.RpcClientBase;
import de.starface.com.rpc.client.RpcExceptionConverter;
import de.starface.com.rpc.client.RpcRequestCaller;
import de.starface.com.rpc.client.interceptor.OutgoingRequestInterceptor;
import de.starface.com.rpc.common.RpcTransportToken;
import de.starface.com.rpc.common.interceptor.RequestInterceptorChainLink;

import org.hamcrest.Matcher;
import org.hamcrest.Matchers;
import org.jmock.Expectations;
import org.jmock.Mockery;
import org.jmock.imposters.ByteBuddyClassImposteriser;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static org.hamcrest.Matchers.allOf;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertSame;
import static org.junit.jupiter.api.Assertions.assertThrows;

class RpcRequestCallerTest {
	private Mockery context;

	private RpcRequestCaller<RpcTransportToken> rpcRequestCaller;

	private RpcTransportToken targetToken = new RpcTransportToken() {

		@Override
		public String getIdentifierString() {
			return "";
		}
		// Dummy implementation
	};

	private OutgoingRequestProcessorFactory<RpcTransportToken> requestProcessorFactory;

	private OutgoingRequestProcessor<RpcTransportToken> requestProcessor;

	private OutgoingRequestInterceptor<RpcTransportToken> requestInterceptor;

	private List<OutgoingRequestInterceptor<RpcTransportToken>> requestInterceptorList = new ArrayList<>();

	Method rpcMethod;

	Object[] rpcParameters = new Object[]{321};

	private Matcher<OutgoingRequest<RpcTransportToken>> requestMatcher;

	RpcClientBase<RpcTransportToken> rpcClient;

	TestIF proxy;

	@SuppressWarnings("unchecked")
	@BeforeEach
	public void setUp() throws SecurityException, NoSuchMethodException {
		context = new Mockery() {
			{
				setImposteriser(ByteBuddyClassImposteriser.INSTANCE);
			}
		};

		requestProcessor = context.mock(OutgoingRequestProcessor.class);

		requestProcessorFactory = context.mock(OutgoingRequestProcessorFactory.class);

		rpcClient = context.mock(RpcClientBase.class);

		rpcRequestCaller = new RpcRequestCaller<>(rpcClient, "testNamePrefix", targetToken,
			requestProcessorFactory, TestIF.class);

		proxy = (TestIF) Proxy.newProxyInstance(Thread.currentThread().getContextClassLoader(),
			new Class[]{TestIF.class}, rpcRequestCaller);

		requestInterceptor = context.mock(OutgoingRequestInterceptor.class);
		requestInterceptorList.add(requestInterceptor);

		rpcMethod = TestIF.class.getDeclaredMethod("test", Object.class);

		requestMatcher = allOf(
			Matchers.<OutgoingRequest<RpcTransportToken>>hasProperty("targetTransportToken",
				Matchers.sameInstance(targetToken)),
			Matchers.<OutgoingRequest<RpcTransportToken>>hasProperty("rpcInterface",
				Matchers.sameInstance(TestIF.class)),
			Matchers.<OutgoingRequest<RpcTransportToken>>hasProperty("parameters",
				Matchers.sameInstance(rpcParameters)),
			Matchers.<OutgoingRequest<RpcTransportToken>>hasProperty("rpcMethod",
				Matchers.sameInstance(rpcMethod)));
	}

	@Test
	void getRequestCallerFromProxyReturnsCorrectRequestCaller() {
		RpcRequestCaller<?> actual = RpcRequestCaller.getRequestCallerFromProxy(proxy);
		assertEquals(rpcRequestCaller, actual);
	}

	@Test
	void getRequestCallerFromProxyThrowsIllegalArgumentExceptionOnIvalidProxy() {
		Object invalidProxy = Proxy.newProxyInstance(Thread.currentThread().getContextClassLoader(),
			new Class[]{Map.class}, new InvocationHandler() {

				@Override
				public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
					return null;
				}
			});
		assertThrows(IllegalArgumentException.class, () -> RpcRequestCaller.getRequestCallerFromProxy(invalidProxy));

	}

	@SuppressWarnings("unchecked")
	@Test
	void getAndSetExceptionConverterTest() {
		RpcExceptionConverter<RpcTransportToken> converter = context.mock(RpcExceptionConverter.class);
		RpcExceptionConverter<? super RpcTransportToken> result;

		rpcRequestCaller.setExceptionConverter(converter);
		result = rpcRequestCaller.getExceptionConverter();
		assertSame(converter, result);

		rpcRequestCaller.setExceptionConverter(null);
		result = rpcRequestCaller.getExceptionConverter();
		assertNull(result);
	}

	@SuppressWarnings("unchecked")
	@Test
	void invokeCreatesRequestProcessorUsingFactory() throws Throwable {
		context.checking(new Expectations() {
			{
				allowing(rpcClient).getRequestInterceptors();
				will(returnValue(new ArrayList<OutgoingRequestInterceptor<RpcTransportToken>>()));

				allowing(requestProcessor).processRequest(with(any(OutgoingRequest.class)));

				one(requestProcessorFactory).createOutgoingRequestProcessor();
				will(returnValue(requestProcessor));
			}
		});

		rpcRequestCaller.invoke(proxy, rpcMethod, rpcParameters);

		context.assertIsSatisfied();
	}

	@SuppressWarnings("unchecked")
	@Test
	void invokeUsesRequestInterceptorsFromClient() throws Throwable {

		context.checking(new Expectations() {
			{
				one(rpcClient).getRequestInterceptors();
				will(returnValue(requestInterceptorList));

				one(requestInterceptor).interceptRequest(with(requestMatcher),
					with(any(RequestInterceptorChainLink.class)));

				allowing(requestProcessor).processRequest(with(any(OutgoingRequest.class)));

				allowing(requestProcessorFactory).createOutgoingRequestProcessor();
				will(returnValue(requestProcessor));
			}
		});

		rpcRequestCaller.invoke(proxy, rpcMethod, rpcParameters);

		context.assertIsSatisfied();
	}

	@SuppressWarnings("unchecked")
	@Test
	void invokeUsesRequestInterceptorsFromRequestCaller() throws Throwable {

		context.checking(new Expectations() {
			{
				allowing(rpcClient).getRequestInterceptors();
				will(returnValue(new ArrayList<OutgoingRequestInterceptor<RpcTransportToken>>()));

				one(requestInterceptor).interceptRequest(with(requestMatcher),
					with(any(RequestInterceptorChainLink.class)));

				allowing(requestProcessor).processRequest(with(any(OutgoingRequest.class)));

				allowing(requestProcessorFactory).createOutgoingRequestProcessor();
				will(returnValue(requestProcessor));
			}
		});

		rpcRequestCaller.addRpcRequestInterceptor(requestInterceptor);

		rpcRequestCaller.invoke(proxy, rpcMethod, rpcParameters);

		context.assertIsSatisfied();
	}
}
