/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.examples.uci30.swing.viewmodel;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciCallListEvents;
import de.starface.integration.uci.java.v30.messages.events.UciCdrDataEvents;
import de.starface.integration.uci.java.v30.messages.events.UciGroupEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciCallListRequests;
import de.starface.integration.uci.java.v30.types.CallList;
import de.starface.integration.uci.java.v30.types.CallListEntry;
import de.starface.integration.uci.java.v30.types.CdrDataInfo;
import de.starface.integration.uci.java.v30.types.GroupSetting;
import de.starface.integration.uci.java.v30.values.CallListEntryDirection;
import de.starface.integration.uci.java.v30.values.CallListEntryProperties;
import de.starface.integration.uci.java.v30.values.CallListEntryResult;
import de.starface.integration.uci.java.v30.values.CdrUpdateType;
import de.starface.integration.uci.java.v30.values.GroupRestriction;
import de.starface.integration.uci.java.v30.values.OrderDirection;
import de.starface.integration.uci.v30.client.UciServiceRequests;
import de.starface.integration.uci.v30.client.utils.EventsProvider;

import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import javax.swing.table.TableModel;

public class CallListViewModel implements UciGroupEvents, UciCdrDataEvents, UciCallListEvents {
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////

	private PhoneToUseProvider phoneToUseProvider;

	public CallListViewModel(EventsProvider<UciGroupEvents> groupEventsProvider,
							 PhoneToUseProvider phoneToUseProvider) {
		groupEventsProvider.subscribeEvents(this);
		this.phoneToUseProvider = phoneToUseProvider;
	}

	private UciCallListRequests callListRequests;

	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException {
		this.callListRequests = uciServiceRequests.getRequests(UciCallListRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciCdrDataEvents.class);
		uciServiceRequests.subscribeEventsAsync(this, UciCallListEvents.class);
	}

	@Override
	public void groupSettingsChanged(List<GroupSetting> groupSettings) {
		setGroupRestrictionValues(groupSettings);
	}

	/// View Actions
	////////////////////////////////////////////////////////////////
	private void doGetCallList() {
		if (callListRequests != null) {
			try {
				Date startBefore = this.startBefore != null ? this.startBefore : new Date();
				Date startAfter = this.startAfter != null ? this.startAfter : new Date(0);

				String groupRestriction = this.groupRestriction instanceof GroupSetting ? ((GroupSetting) this.groupRestriction).getId()
					: (String) this.groupRestriction;

				// Retrieve call list sending getCallList request
				CallList callList = callListRequests.getCallList(startBefore, startAfter, directionRestriction,
					resultRestriction, groupRestriction, orderProperty, orderDirection, countOffset, countLimit);

				callListTableModel.setCallList(callList);
			} catch (UciException e) {
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}

	private void doDeleteCallListEntry() {
		withSelectedRow((row, entry) -> {
			// delete call list entry sending deleteCallListEntry request
			callListRequests.deleteCallListEntry(entry.getId());
			callListTableModel.removeEntry(row);
		});
	}

	private void doCallbackCallListEntry() {
		withSelectedRow((row, entry) -> {
			// delete call list entry sending deleteCallListEntry request
			callListRequests.placeCallbackCallWithPhone(entry.getId(), phoneToUseProvider.getPhoneToUse(),
					UUID.randomUUID().toString());
		});
	}

	private void doSetCalledBackCallListEntry() {
		withSelectedRow((row, entry) -> callListRequests.setCallListEntryCalledBack(entry.getId(), true));
	}

	private void doClearCalledBackCallListEntry() {
		withSelectedRow((row, entry) -> callListRequests.clearCallListEntryCalledBack(entry.getId()));
	}

	private void doSetCommentCallListEntry() {
		withSelectedRow((row, entry) -> callListRequests.setCallListEntryComment(entry.getId(), commentText));
	}

	private void doClearCommentCallListEntry() {
		withSelectedRow((row, entry) -> callListRequests.clearCallListEntryComment(entry.getId()));
	}


	private interface ExceptionalConsumer<T> {
		void accept(int row, T t) throws UciException;
	}

	private void withSelectedRow(ExceptionalConsumer<CallListEntry> callListEntryConsumer) {
		if (selectedRows.size() != 1 || callListRequests == null) {
			return;
		}

		int targetRow = selectedRows.get(0);
		CallListEntry entry = callListTableModel.getEntry(targetRow);
		try {
			callListEntryConsumer.accept(targetRow, entry);
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private void doDeleteCallListEntries() {
		if (selectedRows.isEmpty() || callListRequests == null) {
			return;
		}

		List<String> idsToDelete = computeSelectedEntryIds();
		try {
			callListRequests.deleteCallListEntries(idsToDelete);
			callListTableModel.removeEntries(selectedRows);
		} catch (UciException e) {
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}

	private List<String> computeSelectedEntryIds() {
		List<String> entryIds = new ArrayList<>(selectedRows.size());
		for (Integer i : selectedRows) {
			entryIds.add(callListTableModel.getEntry(i).getId());
		}
		return entryIds;
	}

	private final ActionListener getCallListActionListener = e -> doGetCallList();

	private final ActionListener deleteCallListEntryActionListener = e -> doDeleteCallListEntry();

	private final ActionListener callbackCallListEntryActionListener = e -> doCallbackCallListEntry();

	private final ActionListener setCalledBackCallListEntryActionListener = e -> doSetCalledBackCallListEntry();

	private final ActionListener setCommentCallListEntryActionListener = e -> doSetCommentCallListEntry();

	private final ActionListener clearCalledBackCallListEntryActionListener = e -> doClearCalledBackCallListEntry();

	private final ActionListener clearCommentCallListEntryActionListener = e -> doClearCommentCallListEntry();

	private final ActionListener deleteCallListEntriesActionListener = e -> doDeleteCallListEntries();

	/// View Data
	////////////////////////////////////////////////////////////////

	private final CallListTableModel callListTableModel = new CallListTableModel();

	private Date startBefore = null;

	private Date startAfter = null;

	private CallListEntryDirection directionRestriction = null;

	private CallListEntryResult resultRestriction = null;

	private List<Object> groupRestrictionValues;

	{
		setGroupRestrictionValues(Collections.<GroupSetting>emptyList());
	}

	private Object groupRestriction = "";

	private CallListEntryProperties orderProperty = null;

	private OrderDirection orderDirection = null;

	private int countOffset = 0;

	private int countLimit = 17;

	private String commentText;

	private List<Integer> selectedRows = new ArrayList<>();

	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

	public void addPropertyChangeListener(PropertyChangeListener x) {
		changeSupport.addPropertyChangeListener(x);
	}

	public void removePropertyChangeListener(PropertyChangeListener x) {
		changeSupport.removePropertyChangeListener(x);
	}

	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////

	public TableModel getCallListTableModel() {
		return callListTableModel;
	}

	public Date getStartBefore() {
		return startBefore;
	}

	public static final String PROPERTY_START_BEFORE = "startBefore";

	public void setStartBefore(Date startBefore) {
		Date old = this.startBefore;
		this.startBefore = startBefore;
		changeSupport.firePropertyChange(PROPERTY_START_BEFORE, old, startBefore);
	}

	public Date getStartAfter() {
		return startAfter;
	}

	public static final String PROPERTY_START_AFTER = "startAfter";

	public void setStartAfter(Date startAfter) {
		Date old = this.startAfter;
		this.startAfter = startAfter;
		changeSupport.firePropertyChange(PROPERTY_START_AFTER, old, startAfter);
	}

	public CallListEntryDirection getDirectionRestriction() {
		return directionRestriction;
	}

	public static final String PROPERTY_DIRECTION_RESTRICTION = "directionRestriction";

	public void setDirectionRestriction(CallListEntryDirection directionRestriction) {
		CallListEntryDirection old = this.directionRestriction;
		this.directionRestriction = directionRestriction;
		changeSupport.firePropertyChange(PROPERTY_DIRECTION_RESTRICTION, old, directionRestriction);
	}

	public CallListEntryDirection[] getDirectionRestrictionValues() {
		return Utils.createArrayWithLeadingNull(CallListEntryDirection.values());
	}

	public CallListEntryResult getResultRestriction() {
		return resultRestriction;
	}

	public static final String PROPERTY_RESULT_RESTRICTION = "resultRestriction";

	public void setResultRestriction(CallListEntryResult resultRestriction) {
		CallListEntryResult old = this.resultRestriction;
		this.resultRestriction = resultRestriction;
		changeSupport.firePropertyChange(PROPERTY_RESULT_RESTRICTION, old, resultRestriction);
	}

	public CallListEntryResult[] getResultRestrictionValues() {
		return Utils.createArrayWithLeadingNull(CallListEntryResult.values());
	}

	public static final String PROPERTY_GROUP_RESTRICTION = "groupRestriction";

	public Object getGroupRestriction() {
		return groupRestriction;
	}

	public void setGroupRestriction(Object groupRestriction) {
		Object old = this.groupRestriction;
		this.groupRestriction = groupRestriction;
		changeSupport.firePropertyChange(PROPERTY_GROUP_RESTRICTION, old, groupRestriction);
	}

	public static final String PROPERTY_GROUP_RESTRICTION_VALUES = "groupRestrictionValues";

	public List<Object> getGroupRestrictionValues() {
		return groupRestrictionValues;
	}

	private void setGroupRestrictionValues(List<GroupSetting> groupSettings) {
		List<Object> old = groupRestrictionValues;
		groupRestrictionValues = new ArrayList<>();
		groupRestrictionValues.add("");
		groupRestrictionValues.add(GroupRestriction.NON_GROUP);
		groupRestrictionValues.addAll(groupSettings);
		if (changeSupport != null) {
			changeSupport.firePropertyChange(PROPERTY_GROUP_RESTRICTION_VALUES, old, groupRestrictionValues);
		}
	}

	public CallListEntryProperties getOrderProperty() {
		return orderProperty;
	}

	public static final String PROPERTY_ORDER_PROPERTY = "orderProperty";

	public void setOrderProperty(CallListEntryProperties orderProperty) {
		CallListEntryProperties old = this.orderProperty;
		this.orderProperty = orderProperty;
		changeSupport.firePropertyChange(PROPERTY_ORDER_PROPERTY, old, orderProperty);
	}

	public CallListEntryProperties[] getOrderPropertyValues() {
		return Utils.createArrayWithLeadingNull(CallListEntryProperties.values());
	}

	public OrderDirection getOrderDirection() {
		return orderDirection;
	}

	public static final String PROPERTY_ORDER_DIRECTION = "orderDirection";

	public void setOrderDirection(OrderDirection orderDirection) {
		OrderDirection old = this.orderDirection;
		this.orderDirection = orderDirection;
		changeSupport.firePropertyChange(PROPERTY_ORDER_DIRECTION, old, orderDirection);
	}

	public OrderDirection[] getOrderDirectionValues() {
		return Utils.createArrayWithLeadingNull(OrderDirection.values());
	}

	public int getCountOffset() {
		return countOffset;
	}

	public static final String PROPERTY_COUNT_OFFSET = "countOffset";

	public void setCountOffset(int countOffset) {
		int old = this.countOffset;
		this.countOffset = countOffset;
		changeSupport.firePropertyChange(PROPERTY_COUNT_OFFSET, old, countOffset);
	}

	public int getCountLimit() {
		return countLimit;
	}

	public static final String PROPERTY_COUNT_LIMIT = "countLimit";

	public void setCountLimit(int countLimit) {
		int old = this.countLimit;
		this.countLimit = countLimit;
		changeSupport.firePropertyChange(PROPERTY_COUNT_LIMIT, old, countLimit);
	}

	public String getCommentText() {
		return commentText;
	}

	public static final String PROPERTY_COMMENT_TEXT = "commentText";

	public void setCommentText(String commentText) {
		String old = this.commentText;
		this.commentText = commentText;
		changeSupport.firePropertyChange(PROPERTY_COMMENT_TEXT, old, commentText);
	}

	public List<Integer> getSelectedRows() {
		return selectedRows;
	}

	public ActionListener getGetCallListActionListener() {
		return getCallListActionListener;
	}

	public ActionListener getDeleteCallListEntryActionListener() {
		return deleteCallListEntryActionListener;
	}

	public ActionListener getCallbackCallListEntryActionListener() {
		return callbackCallListEntryActionListener;
	}

	public ActionListener getSetCalledBackCallListEntryActionListener() {
		return setCalledBackCallListEntryActionListener;
	}

	public ActionListener getSetCommentCallListEntryActionListener() {
		return setCommentCallListEntryActionListener;
	}

	public ActionListener getClearCalledBackCallListEntryActionListener() {
		return clearCalledBackCallListEntryActionListener;
	}

	public ActionListener getClearCommentCallListEntryActionListener() {
		return clearCommentCallListEntryActionListener;
	}

	public ActionListener getDeleteCallListEntriesActionListener() {
		return deleteCallListEntriesActionListener;
	}

	@Override
	@RpcAsynchronous
	public void callCdrEvent(CdrDataInfo cdrDataInfo) {
		System.out.println(cdrDataInfo.toString());
	}

	@Override
	@RpcAsynchronous
	public void callCdrUpdatedEvent(CdrUpdateType cdrUpdateType, CdrDataInfo cdrDataInfo) {
		System.out.printf("CDR Updated Event: %s %s%n", cdrUpdateType.toString(), cdrDataInfo.toString());
	}

	@Override
	public void newCallListEntry(CallListEntry callListEntry) {
		callListTableModel.addEntry(callListEntry);
	}

	@Override
	public void updatedCallListEntry(CallListEntry callListEntry) {
		for (int i = 0; i < callListTableModel.getRowCount(); i++) {
			CallListEntry entry = callListTableModel.getEntry(i);
			if (entry.getId().equals(callListEntry.getId())) {
				callListTableModel.replaceEntry(i, callListEntry);
			}
		}
	}

	@Override
	public void deletedCallListEntries(List<String> deletedCallCdrDataIds) {
		List<Integer> rowsToDelete = new ArrayList<>();
		for (int i = 0; i < callListTableModel.getRowCount(); i++) {
			CallListEntry entry = callListTableModel.getEntry(i);
			if (deletedCallCdrDataIds.contains(entry.getId())) {
				rowsToDelete.add(i);
			}
		}
		callListTableModel.removeEntries(rowsToDelete);
	}
}
