/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client;

import de.starface.integration.uci.v30.client.impl.UciProxyFacade;
import de.starface.integration.uci.v30.client.impl.UciProxyFactoryBase;
import de.starface.integration.uci.v30.client.impl.UciServiceRequestsFacade;
import de.starface.integration.uci.v30.client.impl.UcpConnectionManager;
import de.starface.integration.uci.v30.client.transport.UcpTransport;
import de.starface.integration.uci.v30.client.transport.UcpTransportFactory;
import de.starface.integration.uci.v30.client.transport.UcpTransportFactory.Configurator;

import java.io.File;
import java.util.Map;

/**
 * This is a factory class that can be used to create {@link UciProxy} instances for multiple
 * STARFACE users.
 *
 * @author tilmann
 *
 */
public class UciProxyFactory extends UciProxyFactoryBase {
	private UciProxyFactory() throws UciProxyConfigurationFailedException {
		super();
	}

	private UciProxyFactory(UcpTransportFactory ucpTransportFactory) {
		super(ucpTransportFactory);
	}

	private UciProxyFactory(String filename) throws UciProxyConfigurationFailedException {
		super(filename);
	}

	private UciProxyFactory(File file) throws UciProxyConfigurationFailedException {
		super(file);
	}

	private UciProxyFactory(Map<String, String> properties) throws UciProxyConfigurationFailedException {
		super(properties);
	}

	/**
	 * Creates a new {@link UciProxyFactory} obtaining its connections from the given
	 * {@link UcpTransportFactory}. Thus select the proper factory to use depending on the transport
	 * mechanism to use for the connections.
	 *
	 * @param ucpTransportFactory
	 *        the {@link UcpTransportFactory} to be used
	 * @return the newly created UciProxyFactory
	 * @throws IllegalArgumentException
	 *         if null ucpTransportFactory given
	 */
	public static UciProxyFactory createWithTransportFactory(UcpTransportFactory ucpTransportFactory) {
		return new UciProxyFactory(ucpTransportFactory);
	}

	/**
	 * Creates a new UciProxyFactory and tries to read its configuration from the default properties
	 * file with the name "uci-proxy.properties".
	 *
	 * There must exist one mandatory property in the file with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the file.
	 *
	 * @return the newly created and configured {@link UciProxyFactory}
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while accessing the file or creating or executing the specified
	 *         {@link Configurator} implementation.
	 */
	public static UciProxyFactory createFromDefaultPropertiesFile() throws UciProxyConfigurationFailedException {
		return new UciProxyFactory();
	}

	/**
	 * Creates a new UciProxyFactory and tries to read its configuration from the given file.
	 *
	 * There must exist one mandatory property in the file with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the file.
	 *
	 * @param filename
	 *        the Java properties file that contains the configuration properties
	 * @return the newly created and configured {@link UciProxyFactory}
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while accessing the file or creating or executing the specified
	 *         {@link Configurator} implementation.
	 * @throws IllegalArgumentException
	 *         if null or empty filename given
	 */
	public static UciProxyFactory createFromPropertiesFile(String filename)
		throws UciProxyConfigurationFailedException {
		return new UciProxyFactory(filename);
	}

	/**
	 * Creates a new UciProxyFactory and tries to read its configuration from the given file.
	 *
	 * There must exist one mandatory property in the file with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the file.
	 *
	 * @param file
	 *        the Java properties file that contains the configuration properties
	 * @return the newly created and configured {@link UciProxyFactory}
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while accessing the file or creating or executing the specified
	 *         {@link Configurator} implementation.
	 * @throws IllegalArgumentException
	 *         if null file given
	 */
	public static UciProxyFactory createFromPropertiesFile(File file) throws UciProxyConfigurationFailedException {
		return new UciProxyFactory(file);
	}

	/**
	 * Creates a new UciProxyFactory and tries to read its configuration from the given {@link Map}.
	 *
	 * There must exist one mandatory property in the map with the key
	 * "ucpTransportFactory.configuratorClass" that contains the fully qualified Java class name of
	 * a {@link Configurator} implementation. Additional properties that are required by the
	 * specified {@link Configurator} implementation must also be present in the map.
	 *
	 * @param properties
	 *        the configuration properties
	 * @return the newly created and configured {@link UciProxyFactory}
	 * @throws UciProxyConfigurationFailedException
	 *         if a problem occurs while creating or executing the specified {@link Configurator}
	 *         implementation.
	 * @throws IllegalArgumentException
	 *         if null properties given
	 */
	public static UciProxyFactory createFromPropertiesMap(Map<String, String> properties)
		throws UciProxyConfigurationFailedException {
		return new UciProxyFactory(properties);
	}

	/**
	 * Creates a {@link UciProxy} for the STARFACE user identified by the given login id and
	 * password. Use this method only if you are not interested in receiving any events sent back by
	 * the UCP server.
	 *
	 * @param loginId
	 *        the login id of the STARFACE user
	 * @param password
	 *        the password of the STARFACE user
	 * @return The {@link UciProxy} tied to the STARFACE user.
	 * @throws UcpConnectionFailedException
	 * @throws IllegalArgumentException
	 *         if null or empty loginId or null password given
	 */
	public UciProxy createUciProxy(String loginId, String password) {
		UcpTransport transport = super.createUcpTransport(loginId, password);

		UcpConnectionManager connectionManager = new UcpConnectionManager(transport);
		UciServiceRequests serviceRequests = new UciServiceRequestsFacade(connectionManager);

		return new UciProxyFacade(connectionManager, serviceRequests);
	}
}
