/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;

import java.util.List;

/**
 * The methods in this interface are used obtain services that communicate with STARFACE. Two types
 * of services exist: Event services that can be used to be notified of events from STARFACE and
 * request services that can be used to send requests to STARFACE.
 *
 * @author tilmann
 */
public interface UciServiceRequests {
	/**
	 * Retrieve a list of strings with the service names of the events provided by STARFACE.
	 *
	 * @return the service names of available events
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public List<String> getProvidedEvents() throws UciException;

	/**
	 * Checks whether STARFACE provides the events with the given service name.
	 *
	 * @param serviceName
	 *        the service name of the events to ckeck for
	 * @return {@code true} if the requested events are provided by STARFACE
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public boolean providesEvents(String serviceName) throws UciException;

	/**
	 * Checks whether STARFACE provides the events defined by the given service interface.
	 *
	 * @param <Events>
	 *        the type of service to check for
	 * @param serviceInterface
	 *        the interface of the service to check for
	 * @return {@code true} if the requested events are provided by STARFACE
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public <Events> boolean providesEvents(Class<Events> serviceInterface) throws UciException;

	/**
	 * Subscribes a listener to the events defined by the given service interface. This method will
	 * only work if the proxy is currently connected to STARFACE. Otherwise a
	 * {@link UcpNotConnectedException} is thrown. Event subscriptions that are successfully
	 * established are automatically resubscribed after the connection to STARFACE has been lost and
	 * reestablished.
	 *
	 * @param <Events>
	 *        the type of service listen to
	 * @param listener
	 *        the event listener to be subscribed
	 * @param serviceInterface
	 *        the interface of the service to listen to
	 * @return {@code true} if the subscription was successful and {@code false} if the requested
	 *         events are not provided by STARFACE
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public <Events> boolean subscribeEvents(Events listener, Class<Events> serviceInterface) throws UciException;

	/**
	 * Asynchronously subscribes a listener to the events defined by the given service interface.
	 * This method will also work if the proxy is currently not connected to STARFACE. Event
	 * subscriptions are automatically subscribed when the connection to STARFACE is initially
	 * established and are resubscribed after the connection to STARFACE has been lost and
	 * reestablished.
	 *
	 * @param <Events>
	 *        the type of service to listen to
	 * @param listener
	 *        the event listener to be subscribed
	 * @param serviceInterface
	 *        the interface of the service to listen to
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}).
	 */
	public <Events> void subscribeEventsAsync(Events listener, Class<Events> serviceInterface) throws UciException;

	/**
	 * Unsubscribes a listener from the events it has previously subscribed for.
	 *
	 * @param <Events>
	 *        the type of service to no longer listen to
	 * @param listener
	 *        the event listener to be unsubscribed
	 * @param serviceInterface
	 *        the interface of the service to no longer listen to
	 */
	public <Events> void unsubscibeEvents(Events listener, Class<Events> serviceInterface);

	/**
	 * Retrieve a list of strings with the service names of the requests provided by STARFACE.
	 *
	 * @return the service names of available request
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public List<String> getProvidedRequests() throws UciException;

	/**
	 * Checks whether STARFACE provides the requests with the given service name.
	 *
	 * @param serviceName
	 *        the service name of the requests to ckeck for
	 * @return {@code true} if the requested requests are provided by STARFACE
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public boolean providesRequests(String serviceName) throws UciException;

	/**
	 * Checks whether STARFACE provides the requests defined by the given service interface.
	 *
	 * @param <Requests>
	 *        the type of service to check for
	 * @param serviceInterface
	 *        the interface of the service to check for
	 * @return {@code true} if the requested requests are provided by STARFACE
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public <Requests> boolean providesRequests(Class<Requests> serviceInterface) throws UciException;

	/**
	 * Retrieves a requests service implementation that can be used to send requests to STARFACE.
	 *
	 * @param <Requests>
	 *        the type of service to retrieve
	 * @param serviceInterface
	 *        the interface of the service to retrieve
	 * @return the implementation of the requested requests service
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public <Requests> Requests getRequests(Class<Requests> serviceInterface) throws UciException;
}
