/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.impl;

import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpServiceRequests;
import de.starface.integration.uci.v30.client.UciConnectionEvents;
import de.starface.integration.uci.v30.client.UcpConnectionFailedException;
import de.starface.integration.uci.v30.client.transport.UcpTransport;
import de.starface.integration.uci.v30.client.utils.EventsProviderSupport;

import org.apache.commons.lang3.Validate;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.HashMap;
import java.util.Map;

/**
 * The UciEventsRegistry manages all client event listeners subscribed to STARFACE events
 *
 * @author tilmann
 */
class UciEventsRegistry implements UciConnectionEvents {
	private static final Logger log = LogManager.getLogger(UciEventsRegistry.class);

	private Map<String, EventsProviderSupport<?>> eventsProviders = new HashMap<>();

	private UcpConnectionManager connectionManager;

	UciEventsRegistry(UcpConnectionManager connectionManager) {
		this.connectionManager = connectionManager;
		connectionManager.addConnectionEventListener(this);
	}

	private void resubscribeAllEvents() {
		try {
			UcpServiceRequests serviceRequests = connectionManager.getServiceRequests();

			synchronized (eventsProviders) {
				log.debug("resubscribeAllEvents: Resubscribing all UCP events...");
				for (String serviceName : eventsProviders.keySet()) {
					boolean subscribed = serviceRequests.subscribeEvents(serviceName);
					if (!subscribed) {
						log.warn("resubscribeAllEvents:  The server does not support the requested events "
							+ serviceName);
					}
				}
				log.debug("resubscribeAllEvents: Finished resubscribing events.");
			}
		} catch (UciException e) {
			log.warn("resubscribeAllEvents: Error while resubscribing UCP events.", e);
		}
	}

	<Events> boolean subscribeEvents(Events listener, Class<Events> serviceInterface) throws UciException {
		return subscribeEvents(listener, serviceInterface, false);
	}

	<Events> void subscribeEventsAsync(Events listener, Class<Events> serviceInterface) throws UciException {
		subscribeEvents(listener, serviceInterface, true);
	}

	<Events> boolean subscribeEvents(Events listener, Class<Events> serviceInterface, boolean async)
		throws UciException {
		Validate.notNull(listener, "listener=null");
		Validate.notNull(serviceInterface, "serviceInterface=null");

		UcpTransport transport = connectionManager.getTransport();
		String serviceName = UciServices.getServiceName(serviceInterface);

		boolean subscribed = false;

		synchronized (eventsProviders) {
			if (async) {
				log.debug("subscribeEvents: Asynchronously subscribing events of service interface "
					+ serviceInterface.getName() + "...");
			} else {
				log.debug("subscribeEvents: Synchronously trying to subscribe events of service interface "
					+ serviceInterface.getName() + "...");
			}
			@SuppressWarnings("unchecked")
			EventsProviderSupport<Events> eventsProvider = (EventsProviderSupport<Events>) eventsProviders.get(serviceName);
			try {
				if (eventsProvider == null) {
					eventsProvider = new EventsProviderSupport<>(serviceInterface, log);
					eventsProviders.put(serviceName, eventsProvider);
					transport.registerUcpEventListener(eventsProvider.getEventsDistributor(), serviceInterface);
					if (async) {
						subscribeEventsAtServerAsync(serviceName);
						subscribed = true;
					} else {
						subscribed = subscribeEventsAtServer(serviceName);
					}
				} else {
					if (!serviceInterface.equals(eventsProvider.getEventsInterface())) {
						throw new IllegalArgumentException(
							"It is not allowed to register different Interfaces for the same UCI service name. Trying to register: "
								+ serviceInterface.getName() + ", already registered: "
								+ eventsProvider.getEventsInterface().getName());
					}
				}
				if (subscribed) {
					eventsProvider.subscribeEvents(listener);
					log.debug("subscribeEvents: Successfully subscribed events of service interface "
						+ serviceInterface.getName());
				}
			} finally {
				if (!subscribed) {
					log.debug("subscribeEvents: Could not subscribe events of service interface "
						+ serviceInterface.getName());
					if (eventsProvider != null && !eventsProvider.hasSubscribers()) {
						transport.unregisterUcpEventListener(serviceInterface);
						eventsProviders.remove(serviceName);
					}
				}
			}
		}
		return subscribed;
	}

	private void subscribeEventsAtServerAsync(String serviceName) {
		if (connectionManager.getConnectionState()) {
			try {
				boolean subscribed = subscribeEventsAtServer(serviceName);
				if (!subscribed) {
					log.warn("asyncSubscribeEventsAtServer: The server does not support the requested events "
						+ serviceName);
				}
			} catch (UciException e) {
				log.info("asyncSubscribeEventsAtServer: Could not subscribe events at server.", e);
			}
		}
	}

	private boolean subscribeEventsAtServer(String serviceName) throws UciException {
		UcpServiceRequests serviceRequests = connectionManager.getServiceRequests();
		return serviceRequests.subscribeEvents(serviceName);
	}

	<Events> void unsubscibeEvents(Events listener, Class<Events> serviceInterface) {
		Validate.notNull(listener, "listener=null");
		Validate.notNull(serviceInterface, "serviceInterface=null");

		UcpTransport transport = connectionManager.getTransport();
		String serviceName = UciServices.getServiceName(serviceInterface);

		synchronized (eventsProviders) {
			@SuppressWarnings("unchecked")
			EventsProviderSupport<Events> eventsProvider = (EventsProviderSupport<Events>) eventsProviders.get(serviceName);
			if (eventsProvider != null) {
				log.debug("unsubscibeEvents: Trying to unsubscribe events of service interface "
					+ serviceInterface.getName() + "...");
				eventsProvider.unsubscribeEvents(listener);
				if (!eventsProvider.hasSubscribers()) {
					unsubscribeEventsAtServerAsync(serviceName);
					eventsProviders.remove(serviceName);
					transport.unregisterUcpEventListener(serviceInterface);
				}
				log.debug("unsubscibeEvents: Successfully unsubscribed events of service interface "
					+ serviceInterface.getName());
			}
		}
	}

	private void unsubscribeEventsAtServerAsync(String serviceName) {
		if (connectionManager.getConnectionState()) {
			try {
				unsubscribeEventsAtServer(serviceName);
			} catch (UciException e) {
				log.info("unsubscribeEventsAtServerAsync: Could not unsubscribe events at server.", e);
			}
		}
	}

	private void unsubscribeEventsAtServer(String serviceName) throws UciException {
		UcpServiceRequests serviceRequests = connectionManager.getServiceRequests();
		serviceRequests.unsubscribeEvents(serviceName);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionEvents#receiveConnectionState(boolean)
	 */
	@Override
	public void receiveConnectionState(boolean connectionAvailable) {
		if (connectionAvailable) {
			resubscribeAllEvents();
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionEvents#receiveConnectionError(de.starface.integration.uci.v30.client.UcpConnectionFailedException)
	 */
	@Override
	public void receiveConnectionError(UcpConnectionFailedException e) {
		// Nothing to do.
	}
}
