/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.impl;

import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.ucp.messages.events.UcpConnectionEvents;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpConnectionRequests;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpServiceRequests;
import de.starface.integration.uci.v30.client.UciServiceRequests;
import de.starface.integration.uci.v30.client.transport.UcpTransport;

import org.apache.commons.lang3.Validate;

import java.util.List;

/**
 * This class implements the {@link UciServiceRequests} interface hiding the complexity of the
 * underlying connection manager and events registry.
 *
 * @author tilmann
 */
public class UciServiceRequestsFacade implements UciServiceRequests {
	private UcpConnectionManager connectionManager;

	private UciEventsRegistry eventsRegistry;

	/**
	 * Creates a new facade for the given connection manager
	 *
	 * @param connectionManager
	 *        the connection manager to be associated with this facade
	 */
	public UciServiceRequestsFacade(UcpConnectionManager connectionManager) {
		Validate.notNull(connectionManager, "connectionManager=null");

		this.connectionManager = connectionManager;
		this.eventsRegistry = new UciEventsRegistry(connectionManager);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#getRequests(java.lang.Class)
	 */
	@Override
	public <Requests> Requests getRequests(Class<Requests> serviceInterface) throws UciException {
		Validate.notNull(serviceInterface, "serviceInterface=null");

		String serviceName = UciServices.getServiceName(serviceInterface);
		if (UcpConnectionRequests.SERVICE_NAME.equals(serviceName)) {
			throw new IllegalArgumentException("It is currently not allowed to create a proxy for the requests "
				+ UcpConnectionRequests.SERVICE_NAME);
		}
		connectionManager.checkConnectionState();
		UcpTransport transport = connectionManager.getTransport();
		return transport.getUcpRequests(serviceInterface);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#providesEvents(java.lang.Class)
	 */
	@Override
	public <Events> boolean providesEvents(Class<Events> serviceInterface) throws UciException {
		Validate.notNull(serviceInterface, "serviceInterface=null");

		UcpServiceRequests serviceRequests = connectionManager.getServiceRequests();
		String serviceName = UciServices.getServiceName(serviceInterface);
		return serviceRequests.providesEvents(serviceName);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#providesRequests(java.lang.Class)
	 */
	@Override
	public <Requests> boolean providesRequests(Class<Requests> serviceInterface) throws UciException {
		Validate.notNull(serviceInterface, "serviceInterface=null");

		UcpServiceRequests serviceRequests = connectionManager.getServiceRequests();
		boolean providesRequests = true;
		List<String> serviceNames = UciServices.getServiceNames(serviceInterface);
		for (String serviceName : serviceNames) {
			providesRequests = providesRequests && serviceRequests.providesRequests(serviceName);
		}
		return providesRequests;
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#getProvidedEvents()
	 */
	@Override
	public List<String> getProvidedEvents() throws UciException {
		UcpServiceRequests serviceRequests = connectionManager.getServiceRequests();
		return serviceRequests.getProvidedEvents();
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#getProvidedRequests()
	 */
	@Override
	public List<String> getProvidedRequests() throws UciException {
		UcpServiceRequests serviceRequests = connectionManager.getServiceRequests();
		return serviceRequests.getProvidedRequests();
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#providesEvents(java.lang.String)
	 */
	@Override
	public boolean providesEvents(String serviceName) throws UciException {
		Validate.notEmpty(serviceName, "serviceName=empty");

		UcpServiceRequests serviceRequests = connectionManager.getServiceRequests();
		return serviceRequests.providesEvents(serviceName);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#providesRequests(java.lang.String)
	 */
	@Override
	public boolean providesRequests(String serviceName) throws UciException {
		Validate.notEmpty(serviceName, "serviceName=empty");

		UcpServiceRequests serviceRequests = connectionManager.getServiceRequests();
		return serviceRequests.providesRequests(serviceName);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#subscribeEvents(java.lang.Object,
	 *      java.lang.Class)
	 */
	@Override
	public <Events> boolean subscribeEvents(Events listener, Class<Events> serviceInterface) throws UciException {
		return subscribeEvents(listener, serviceInterface, false);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#subscribeEventsAsync(java.lang.Object,
	 *      java.lang.Class)
	 */
	@Override
	public <Events> void subscribeEventsAsync(Events listener, Class<Events> serviceInterface) throws UciException {
		subscribeEvents(listener, serviceInterface, true);
	}

	private <Events> boolean subscribeEvents(Events listener, Class<Events> serviceInterface, boolean async)
		throws UciException {
		Validate.notNull(listener, "listener=null");
		Validate.notNull(serviceInterface, "serviceInterface=null");

		String serviceName = UciServices.getServiceName(serviceInterface);
		if (UcpConnectionEvents.SERVICE_NAME.equals(serviceName)) {
			throw new IllegalArgumentException("It is currently not allowed to subscribe to the events "
				+ UcpConnectionEvents.SERVICE_NAME);
		}

		return eventsRegistry.subscribeEvents(listener, serviceInterface, async);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciServiceRequests#unsubscibeEvents(java.lang.Object,
	 *      java.lang.Class)
	 */
	@Override
	public <Events> void unsubscibeEvents(Events listener, Class<Events> serviceInterface) {
		Validate.notNull(listener, "listener=null");
		Validate.notNull(serviceInterface, "serviceInterface=null");

		eventsRegistry.unsubscibeEvents(listener, serviceInterface);
	}
}
