/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.impl;

import de.starface.com.rpc.RpcException;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpCommunicationException;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.ucp.messages.events.UcpConnectionEvents;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpConnectionRequests;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpServiceRequests;
import de.starface.integration.uci.v30.client.UciConnectionEvents;
import de.starface.integration.uci.v30.client.UciConnectionRequests;
import de.starface.integration.uci.v30.client.UcpConnectionFailedException;
import de.starface.integration.uci.v30.client.transport.UcpTransport;
import de.starface.integration.uci.v30.client.transport.UcpTransportEvents;

import org.apache.commons.lang3.Validate;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.lang.reflect.Method;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.CopyOnWriteArrayList;

/**
 * The UcpConnectionManager is responsible for the proxy's connection to STARFACE. It is used to
 * connect to STARFACE and reconnect if the connection has been lost.
 */
public class UcpConnectionManager implements UciConnectionRequests, UcpConnectionEvents {
	private static final Logger log = LogManager.getLogger(UcpConnectionManager.class);

	private static final long PROBE_INTERVAL = 30000;

	private UcpTransport transport;

	private Timer reconnectTimer;

	private boolean attached;

	private boolean started;

	private int failedAttempts;

	private UcpConnectionRequests connectionRequests;

	private UcpServiceRequests serviceRequests;

	private List<UciConnectionEvents> connectionEventListeners = new CopyOnWriteArrayList<>();

	private boolean getLogConnectionError() {
		return failedAttempts == 0;
	}

	/**
	 * Sets the connection to the UCP server to either available or unavailable. Signaling the state
	 * change to the listeners
	 *
	 * @param value
	 *        <code>true</code> when logged in at UCP server, <code>false</code> otherwise
	 */
	private synchronized void setConnectionState(boolean value) {
		boolean oldValue = attached;
		attached = value;
		if (attached != oldValue) {
			sendReceiveConnectionState(attached);
		}
	}

	/**
	 * Creates a new UcpConnectionManager that manages a UCP connection with the given UcpTransport
	 * instance
	 *
	 * @param ucpTransport
	 *        the transport to use for the UCP protocol
	 */
	public UcpConnectionManager(UcpTransport ucpTransport) {
		Validate.notNull(ucpTransport, "ucpTransport=null");

		this.transport = ucpTransport;
		subscribeUcpTransportEvents();
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#connect()
	 */
	@Override
	public synchronized void connect() throws UcpConnectionFailedException {
		if (!started) {
			try {
				openTransportAndAttach();
			} finally {
				startReconnectTimer(true);
				started = true;
			}
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#connectAsync()
	 */
	@Override
	public synchronized void connectAsync() {
		if (!started) {
			startReconnectTimer(false);
			started = true;
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#disconnect()
	 */
	@Override
	public synchronized void disconnect() {
		if (started) {
			started = false;
			stopReconnectTimer();
			closeTransportAndDetach();
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#getConnectionState()
	 */
	@Override
	public boolean getConnectionState() {
		return attached;
	}

	private synchronized void onReconnectTimer() {
		if (attached) {
			probe();
		}
		if (!attached) {
			try {
				openTransportAndAttach();
			} catch (UcpConnectionFailedException e) {
				// Discard exception. Logging is already done in openTransportAndAttach().
			}
		}
	}

	private void startReconnectTimer(boolean initialDelay) {
		if (reconnectTimer == null) {
			log.debug("startReconnectTimer: Activating reconnect timer.");

			final long delay = initialDelay ? PROBE_INTERVAL : 0;

			reconnectTimer = new Timer("UciProxyImpl.reconnectTimer", true);
			reconnectTimer.schedule(new TimerTask() {
				@Override
				public void run() {
					try {
						onReconnectTimer();
					} catch (Throwable t) {
						log.error("reconnectTimer.TimerTask.run: Caught unexpected exception/error.", t);
					}
				}
			}, delay, PROBE_INTERVAL);
		}
	}

	private void stopReconnectTimer() {
		if (reconnectTimer != null) {
			log.debug("stopReconnectTimer: Deactivating reconnect timer.");

			reconnectTimer.cancel();
			reconnectTimer = null;
		}
	}

	/**
	 * Checks whether the connection to the UCP server is ready to receive requests. If not a
	 * connection attempt is made and if that also fails a {@link UcpConnectionFailedException} is
	 * thrown.
	 *
	 * @throws UcpNotConnectedException
	 *         if a connection to the UCP server is not available and cannot be established
	 */
	synchronized void checkConnectionState() throws UcpNotConnectedException {
		if (!started) {
			connect();
		}
		if (!attached) {
			openTransportAndAttach();
		}
		if (!attached) {
			throw new UcpNotConnectedException("UCI Proxy is not connected to UCP server.");
		}
	}

	private void openTransportAndAttach() throws UcpConnectionFailedException {
		log.debug("openTransportAndAttach: Connecting to UCP server...");
		try {
			transport.open();
			connectionRequests = transport.getUcpRequests(UcpConnectionRequests.class);
			serviceRequests = transport.getUcpRequests(UcpServiceRequests.class);
			attach();
			onConnectionEstablished();
		} catch (UcpConnectionFailedException e) {
			onConnectionFailed(e);
			throw e;
		}
	}

	private void closeTransportAndDetach() {
		log.debug("closeTransportAndDetach: Disconnecting from UCP server...");

		detach();
		transport.close();
		connectionRequests = null;
		serviceRequests = null;

		log.info("closeTransportAndDetach: Successfully disconected from UCP server.");
	}

	private void onConnectionFailed(UcpConnectionFailedException e) {
		if (getLogConnectionError()) {
			log.error("onConnectionFailed: Connecting to UCP server failed.", e);
		} else {
			log.debug("onConnectionFailed: Connection to UCP server is not available.");
		}
		failedAttempts++;
		if (failedAttempts >= 60) {
			failedAttempts = 0;
		}
		sendReceiveConnectionError(e);
	}

	private void onConnectionEstablished() {
		failedAttempts = 0;
		log.info("onConnectionEstablished: Connection to UCP server successfully initialized.");
	}

	private void probe() {
		if (attached) {
			try {
				setConnectionState(connectionRequests.keepAlive());
			} catch (UcpCommunicationException e) {
				setConnectionState(false);
				if (getLogConnectionError()) {
					log.debug("probe: Exception while communicating with UCP sever.", e);
				}
			}
		}
	}

	private void attach() throws UcpConnectionFailedException {
		if (!attached) {
			try {
				boolean ok = connectionRequests.login();
				if (ok) {
					try {
						ok = serviceRequests.subscribeEvents(UciServices.getServiceName(UcpConnectionEvents.class));
						if (ok) {
							transport.unregisterUcpEventListener(UcpConnectionEvents.class);
							transport.registerUcpEventListener(this, UcpConnectionEvents.class);
						}
					} catch (Exception e) {
						detach();
						throw new UcpConnectionFailedException(
							UcpConnectionFailedException.Codes.SUBSCRIBE_EVENTS_FAILED,
							"Could not subscribe to events " + UcpConnectionEvents.class.getName()
								+ " at UCP server.", e);
					}
				} else {
					throw new UcpConnectionFailedException(UcpConnectionFailedException.Codes.LOGIN_FAILED,
						"Could not login at UCP server.");
				}

				// finally we might be really attached :)
				setConnectionState(ok);
			} catch (UciIllegalAccessException e) {
				throw new UcpConnectionFailedException(UcpConnectionFailedException.Codes.LOGIN_FAILED,
					"The user does not have the right to log in to UCI.", e);
			} catch (UcpCommunicationException e) {
				Throwable cause = e.getCause();
				if (cause instanceof RpcException) {
					switch (((RpcException) cause).getCode()) {
						case RpcException.Codes.Server.AUTH_UNKNOWN: {
							throw new UcpConnectionFailedException(
								UcpConnectionFailedException.Codes.AUTHENTICATION_FAILED,
								"Authentication failed at UCP server.", cause);
						}
						case RpcException.Codes.Server.PROCEDURE_NOT_AVAILABLE: {
							throw new UcpConnectionFailedException(
								UcpConnectionFailedException.Codes.NO_UCP_SERVER,
								"Transport is connected but there is no UCP server responding.", cause);
						}
					}
				}
				throw new UcpConnectionFailedException(UcpConnectionFailedException.Codes.TRANSPORT_FAILURE,
					"Failure during xml-rpc call.", cause != null ? cause : e);
			}
		}
	}

	private void detach() {
		setConnectionState(false);
		if (connectionRequests != null) {
			try {
				connectionRequests.logout();
			} catch (Exception e) {
				// We are cleaning up anyway
				// log.debug("detach: UCP server is no longer available", e);
			}
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#addConnectionEventListener(de.starface.integration.uci.v30.client.UciConnectionEvents)
	 */
	@Override
	public void addConnectionEventListener(UciConnectionEvents listener) {
		Validate.notNull(listener, "listener=null");

		connectionEventListeners.add(listener);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.UciConnectionRequests#removeConnectionEventListener(de.starface.integration.uci.v30.client.UciConnectionEvents)
	 */
	@Override
	public void removeConnectionEventListener(UciConnectionEvents listener) {
		Validate.notNull(listener, "listener=null");

		connectionEventListeners.remove(listener);
	}

	private void sendReceiveConnectionState(boolean connectionAvailable) {
		try {
			for (UciConnectionEvents listener : connectionEventListeners) {
				listener.receiveConnectionState(connectionAvailable);
			}
		} catch (Exception e) {
			log.warn("sendReceiveConnectionState: Error while sending event to client.", e);
		}
	}

	private void sendReceiveConnectionError(UcpConnectionFailedException ex) {
		try {
			for (UciConnectionEvents listener : connectionEventListeners) {
				listener.receiveConnectionError(ex);
			}
		} catch (Exception e) {
			log.warn("sendReceiveConnectionState: Error while sending event to client.", e);
		}
	}

	UcpServiceRequests getServiceRequests() throws UcpNotConnectedException {
		checkConnectionState();
		return serviceRequests;
	}

	UcpTransport getTransport() {
		return transport;
	}

	/**
	 * @see getCallContextMapper().ucp.messages.events.UcpConnectionEvents#reset()
	 */
	@Override
	public void reset() {
		setConnectionState(false);
	}

	private void subscribeUcpTransportEvents() {
		transport.subscribeUcpTransportEvents(new UcpTransportEvents() {
			@Override
			public void ucpRequestFailed(Throwable t, Class<?> rpcInterface, Method rpcMethod) {
				if (!UcpConnectionRequests.class.equals(rpcInterface) && t instanceof RpcException) {
					setConnectionState(false);
				}
			}
		});
	}
}
