/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.transport.http;

import de.starface.com.rpc.common.authentication.RpcAuthToken;
import de.starface.com.rpc.xmlrpc.http.URL;
import de.starface.com.rpc.xmlrpc.http.URL.Protocol;
import de.starface.integration.uci.v30.client.utils.EncryptionUtils;

import net.sf.json.JSONObject;
import org.apache.commons.lang3.Validate;
import org.apache.http.HttpEntity;
import org.apache.http.client.HttpRequestRetryHandler;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.conn.ConnectionKeepAliveStrategy;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.TrustAllStrategy;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.DefaultHttpRequestRetryHandler;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContexts;
import org.apache.http.util.EntityUtils;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.Objects;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;

/**
 * An {@link XmlRpcAuthToken} that is used to map between a {@link URL} and a STARFACE login ID. And
 * is used to distinguish between the corresponding UCP clients for incoming server events.
 *
 * @author tilmann
 */
class LoginUrlAuthToken implements RpcAuthToken {
	private static final String ENDPOINT = "/xml-rpc";

	private String loginId;

	private String authHash;

	private URL url;

	String getLoginId() {
		return loginId;
	}

	/**
	 * Creates a new {@link LoginUrlAuthToken} for the user with the given login credentials for
	 * connection to the given server URL.
	 *
	 * @param loginId   the login ID of the STARFACE user
	 * @param password  the password of the STARFACE user
	 * @param serverUrl the URL of the STARFACE server. This URL will be enriched with an authentication token
	 * @throws IllegalArgumentException if null or empty loginId or null password or null serverUrl given
	 */
	LoginUrlAuthToken(String serverHost, int serverPort, boolean useSsl, String loginId, String password) {
		Validate.notEmpty(loginId, "loginId=empty");
		Validate.notNull(password, "password=null");
		Validate.notEmpty(serverHost, "serverHost=empty");
		if (serverPort <= 0) {
			throw new IllegalArgumentException("serverPort must not be less or equal to zero");
		}

		boolean activeDirectoryIsActive = false;

		String uri = "https://" + serverHost + ":" + serverPort + "/ucp-free/info/serverinformations/";
		if (!useSsl) {
			uri = "http://" + serverHost + ":" + serverPort + "/ucp-free/info/serverinformations/";
		}
		HostnameVerifier hostnameVerifier = new NoopHostnameVerifier();
		SSLContext sslContext = null;
		try {
			sslContext = SSLContexts.custom().loadTrustMaterial(new TrustAllStrategy()).build();
		} catch (NoSuchAlgorithmException | KeyManagementException | KeyStoreException e) {
			e.printStackTrace();
		}
		SSLConnectionSocketFactory socketFactory =
				new SSLConnectionSocketFactory(Objects.requireNonNull(sslContext), hostnameVerifier);
		final ConnectionKeepAliveStrategy keepAliveStrategy = (response, context) -> 1000;
		final HttpRequestRetryHandler retryHandler = new DefaultHttpRequestRetryHandler(10, false);
		final RequestConfig requestConfig = RequestConfig.custom()
				.setConnectTimeout(2000)
				.setSocketTimeout(1000)
				.build();

		try (CloseableHttpClient httpClient = HttpClients.custom()
				.setSSLSocketFactory(socketFactory)
				.setKeepAliveStrategy(keepAliveStrategy)
				.setRetryHandler(retryHandler)
				.setDefaultRequestConfig(requestConfig)
				.build()) {
			final HttpGet httpGet = new HttpGet(uri);

			try (CloseableHttpResponse response = httpClient.execute(httpGet)) {
				HttpEntity httpEntity = response.getEntity();
				activeDirectoryIsActive = isActiveDirectoryActivated(httpEntity);
				EntityUtils.consume(httpEntity);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		this.loginId = loginId;
		try {
			if (activeDirectoryIsActive) {
				String toEncode = loginId + "*" + password;
				authHash = java.util.Base64.getEncoder().encodeToString(toEncode.getBytes());
			} else {
				String passwordHash = EncryptionUtils.encryptFromString(password);
				authHash = loginId + ":"
						+ EncryptionUtils.encryptFromString(loginId + "*" + passwordHash);
			}
		} catch (NoSuchAlgorithmException | UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		url = createServerUrl(serverHost, serverPort, useSsl);
		url.setAuth(authHash);
		url.setUser(loginId);
	}

	private boolean isActiveDirectoryActivated(HttpEntity httpEntity) throws IOException {
		String body = "";
		DataInputStream reader = new DataInputStream(httpEntity.getContent());
		BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(reader));
		String line;
		while ((line = bufferedReader.readLine()) != null) {
			body += line;
		}
		JSONObject jsonObject = JSONObject.fromObject(body);
		return jsonObject.getBoolean("activeDirectory");
	}

	URL getUrl() {
		return url;
	}

	/**
	 * Computes the hash code depending on the loginId given at construction time
	 *
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((loginId == null) ? 0 : loginId.hashCode());
		return result;
	}

	/**
	 * Computes the equality depending on the loginId given at construction time
	 *
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		LoginUrlAuthToken other = (LoginUrlAuthToken) obj;
		if (loginId == null) {
			return other.loginId == null;
		} else {
			return loginId.equals(other.loginId);
		}
	}

	private URL createServerUrl(String serverHost, int serverPort, boolean useSsl) {
		if (useSsl) {
			return new URL(Protocol.https, serverHost, serverPort, ENDPOINT);
		} else {
			return new URL(Protocol.http, serverHost, serverPort, ENDPOINT);
		}
	}
}
