/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.transport.xmpp;

import de.starface.com.rpc.RpcException;
import de.starface.com.rpc.server.RpcObjectRegistry;
import de.starface.com.rpc.xmlrpc.smack.XmppXmlRpcClient;
import de.starface.com.rpc.xmlrpc.smack.XmppXmlRpcServer;
import de.starface.com.xmpp.smack.XmppConnectionException;
import de.starface.integration.uci.v30.client.UcpConnectionFailedException;
import de.starface.integration.uci.v30.client.transport.ExceptionConverter;
import de.starface.integration.uci.v30.client.transport.UcpTransport;
import de.starface.integration.uci.v30.client.transport.UcpTransportEvents;

/**
 * This implementation of {@link UcpTransport} provides UCP connectivity over XMPP
 *
 * @author tilmann
 */
public class XmppUcpTransport implements UcpTransport {
	private static final String UCI_XMPP_USER = "starface.xmpp.uci";
	private static final String UCI_XMPP_RESOURCE = "STARFACE";

	private final String applicationName;
	private final String loginId;
	private final String password;
	private final String host;
	private final int port;
	private final boolean useSsl;

	private final ExceptionConverter exceptionConverter = new ExceptionConverter();
	private RpcObjectRegistry rpcObjectRegistry = new RpcObjectRegistry();
	private XmppXmlRpcServer xmppXmlRpcServer;
	private XmppXmlRpcClient xmppXmlRpcClient;

	XmppUcpTransport(String applicationName, String host, int port, boolean useSsl, String loginId, String password) {
		this.applicationName = applicationName;
		this.host = host;
		this.port = port;
		this.loginId = loginId;
		this.password = password;
		this.useSsl = useSsl;
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#close()
	 */
	@Override
	public synchronized void close() {
		closeServer();
		closeClient();

		rpcObjectRegistry.clear();
	}

	private void closeServer() {
		if (xmppXmlRpcServer != null) {
			try {
				xmppXmlRpcServer.shutdown();
			} catch (RpcException e) {
				// We are currently exiting anyway
			} finally {
				xmppXmlRpcServer = null;
			}
		}
	}

	private void closeClient() {
		if (xmppXmlRpcClient != null) {
			xmppXmlRpcClient.close();
			xmppXmlRpcClient = null;
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#open()
	 */
	@Override
	public synchronized void open() throws UcpConnectionFailedException {
		openClient();

		if (!rpcObjectRegistry.isEmpty()) {
			openServer();
		}
	}

	private void openClient() throws UcpConnectionFailedException {
		if (xmppXmlRpcClient == null) {
			try {
				xmppXmlRpcClient = XmppXmlRpcClient.createWithNewConnection("UCI-" + loginId, applicationName,
					password, host, port, useSsl, true, false);
				xmppXmlRpcClient.setExceptionConverter(new ExceptionConverter());
				xmppXmlRpcClient.setDefaultXmlRpcServerJid(UCI_XMPP_USER, UCI_XMPP_RESOURCE);
				xmppXmlRpcClient.open();
			} catch (XmppConnectionException e) {
				close();
				int code = UcpConnectionFailedException.Codes.TRANSPORT_FAILURE;
				switch (e.getCode()) {
					case XmppConnectionException.Codes.AUTHENTICATION_FAILED: {
						code = UcpConnectionFailedException.Codes.AUTHENTICATION_FAILED;
						break;
					}
					case XmppConnectionException.Codes.UNKNOWN_HOST: {
						code = UcpConnectionFailedException.Codes.UNKNOWN_HOST;
						break;
					}
				}
				throw new UcpConnectionFailedException(code, "Could not start XMPP transport.", e);
			}
		}
	}

	private void openServer() throws UcpConnectionFailedException {
		ensureOpen();
		if (xmppXmlRpcServer == null) {
			try {
				xmppXmlRpcServer = XmppXmlRpcServer.createWithExistingConnection(xmppXmlRpcClient.getConnection());
				xmppXmlRpcServer.setRegistry(rpcObjectRegistry);
				xmppXmlRpcServer.startup();
			} catch (RpcException e) {
				close();
				throw new UcpConnectionFailedException(UcpConnectionFailedException.Codes.TRANSPORT_FAILURE,
					"Could not create XML-RPC server", e);
			}
		}
	}

	private void ensureOpen() {
		if (xmppXmlRpcClient == null) {
			throw new IllegalStateException("This UcpTransport is not open");
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#registerUcpEventListener(java.lang.Object,
	 *      java.lang.Class)
	 */
	@Override
	public synchronized <Events> void registerUcpEventListener(Events listener, Class<Events> eventsInterface)
		throws UcpConnectionFailedException {
		rpcObjectRegistry.registerRpcObject(listener, eventsInterface);

		if (xmppXmlRpcClient != null) {
			openServer();
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#unregisterUcpEventListener(java.lang.Class)
	 */
	@Override
	public synchronized <Events> void unregisterUcpEventListener(Class<Events> eventsInterface) {
		rpcObjectRegistry.unregisterRpcObject(eventsInterface);

		if (rpcObjectRegistry.isEmpty()) {
			closeServer();
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#getUcpRequests(java.lang.Class)
	 */
	@Override
	public <Requests> Requests getUcpRequests(Class<Requests> requestsInterface)
		throws UcpConnectionFailedException {
		ensureOpen();
		try {
			return xmppXmlRpcClient.createRpcProxy(requestsInterface);
		} catch (RpcException e) {
			throw new UcpConnectionFailedException(UcpConnectionFailedException.Codes.TRANSPORT_FAILURE,
				"Could not create XML-RPC proxy for interface " + requestsInterface.getName(), e);
		}
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#subscribeUcpTransportEvents(de.starface.integration.uci.v30.client.transport.UcpTransportEvents)
	 */
	@Override
	public void subscribeUcpTransportEvents(UcpTransportEvents listener) {
		exceptionConverter.subscribeEvents(listener);
	}

	/**
	 * @see de.starface.integration.uci.v30.client.transport.UcpTransport#unsubscribeUcpTransportEvents(de.starface.integration.uci.v30.client.transport.UcpTransportEvents)
	 */
	@Override
	public void unsubscribeUcpTransportEvents(UcpTransportEvents listener) {
		exceptionConverter.unsubscribeEvents(listener);
	}
}
