/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.v30.client.transport.xmpp;

import de.starface.integration.uci.v30.client.UciProxyConfigurationFailedException;
import de.starface.integration.uci.v30.client.transport.UcpTransportFactory;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;

import java.util.Map;

/**
 * This implementation of {@link UcpTransportFactory} creates instances of {@link XmppUcpTransport}
 * that can be used to connect to the UCP server over XMPP
 *
 * @author tilmann
 */
public class XmppUcpTransportFactory extends UcpTransportFactory {
	private String applicationName;

	private String host;

	private int port;

	private boolean useSsl;

	/**
	 * Creates a new XmppUcpTransportFactory that uses the given connection information to create
	 * {@link XmppUcpTransport} instances that connect to the UCP server over XMPP.
	 *
	 * @param applicationName
	 *        a unique name that identifies the application using the factory. XMPP connections use
	 *        this as a resource to identify the communication end point so for a given user and
	 *        application combination only one client can be logged in at the server side. May
	 *        contain only word characters, hyphen and dot.
	 * @param serverHost
	 *        The host name of the XMPP server
	 * @param serverPort
	 *        The port of the XMPP server
	 * @param useSsl
	 *        Set to true to use encrypted connection
	 * @throws IllegalArgumentException
	 *         if null or empty applicationName or host given or if given port is less than 0
	 */
	public XmppUcpTransportFactory(String applicationName, String serverHost, int serverPort, boolean useSsl) {
		Validate.notEmpty(applicationName, "applicationName=empty");
		if (!applicationName.matches("\\A[\\-\\w\\.]+\\z")) {
			throw new IllegalArgumentException(
				"applicationName must not contain any characters other than word characters, the hyphen and the dot");
		}
		Validate.notEmpty(serverHost, "host=empty");
		if (serverPort <= 0) {
			throw new IllegalArgumentException("port must not be less or equal to zero");
		}

		this.applicationName = applicationName;
		this.host = serverHost;
		this.port = serverPort;
		this.useSsl = useSsl;
	}

	/**
	 * Creates a new {@link XmppUcpTransport} for the STARFACE user identified by the given loginId
	 * and password.
	 *
	 * @param loginId
	 *        the login ID of the STARFACE user
	 * @param password
	 *        the password of the STARFACE user
	 * @return an instance of {@link XmppUcpTransport} that can be used to connect to the UCP server
	 *         over XMPP
	 * @throws IllegalArgumentException
	 *         if null or empty loginId or null password given
	 */
	@Override
	public XmppUcpTransport createUcpTransport(String loginId, String password) {
		Validate.notEmpty(loginId, "loginId=empty");
		Validate.notNull(password, "password=null");

		return new XmppUcpTransport(applicationName, host, port, useSsl, loginId, password);
	}

	/**
	 * This {@link Configurator} implementation is used to create and configure
	 * {@link XmppUcpTransportFactory} instances from configuration properties.
	 *
	 * There are four properties to set for this Configurator:
	 * <ul>
	 * <li>xmppUcpTransportFactory.serverHost</li>
	 * <li>xmppUcpTransportFactory.serverPort</li>
	 * <li>xmppUcpTransportFactory.useSsl</li>
	 * <li>xmppUcpTransportFactory.applicationName</li>
	 * </ul>
	 * The values to be set are the same as the corresponding constructor parameters of
	 * {@link XmppUcpTransportFactory#XmppUcpTransportFactory(String, String, int, boolean)}
	 *
	 * @author tilmann
	 *
	 */
	public static class Configurator implements UcpTransportFactory.Configurator {

		private static final String SERVER_HOST_PROPERTY = "xmppUcpTransportFactory.serverHost";
		private static final String SERVER_PORT_PROPERTY = "xmppUcpTransportFactory.serverPort";
		private static final String USE_SSL_PROPERTY = "xmppUcpTransportFactory.useSsl";
		private static final String APPLICATION_NAME_PROPERTY = "xmppUcpTransportFactory.applicationName";

		/**
		 * Creates a new {@link XmppUcpTransportFactory} instance and configures it from the values
		 * in the given {@link Map}
		 *
		 * @param configuration
		 *        the configuration properties to be used
		 * @return a newly created {@link XmppUcpTransportFactory} instance
		 * @throws UciProxyConfigurationFailedException
		 *         if a problem occurs while creating or configuring the new instance
		 */
		@Override
		public XmppUcpTransportFactory configureFromMap(Map<String, String> configuration)
			throws UciProxyConfigurationFailedException {

			String serverHost = configuration.get(SERVER_HOST_PROPERTY);
			String serverPortString = configuration.get(SERVER_PORT_PROPERTY);
			String useSslString = configuration.get(USE_SSL_PROPERTY);
			String applicationName = configuration.get(APPLICATION_NAME_PROPERTY);
			int serverPort;
			boolean useSsl;

			if (StringUtils.isEmpty(serverHost)) {
				throw new UciProxyConfigurationFailedException("configuration property " + SERVER_HOST_PROPERTY
					+ " is not set");
			}
			if (StringUtils.isEmpty(serverPortString)) {
				throw new UciProxyConfigurationFailedException("configuration property " + SERVER_PORT_PROPERTY
					+ " is not set");
			}
			if (StringUtils.isEmpty(useSslString)) {
				throw new UciProxyConfigurationFailedException("configuration property " + USE_SSL_PROPERTY
					+ " is not set");
			}
			if (StringUtils.isEmpty(applicationName)) {
				throw new UciProxyConfigurationFailedException("configuration property "
					+ APPLICATION_NAME_PROPERTY + " is not set");
			}

			try {
				serverPort = Integer.parseInt(serverPortString);
			} catch (NumberFormatException e) {
				throw new UciProxyConfigurationFailedException("configuration property " + SERVER_PORT_PROPERTY
					+ " is not set to a port number");
			}
			useSsl = Boolean.parseBoolean(useSslString);

			return new XmppUcpTransportFactory(applicationName, serverHost, serverPort, useSsl);
		}
	}
}
