/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.events;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.types.QueueAgentState;
import de.starface.integration.uci.java.v30.types.QueueSetting;
import de.starface.integration.uci.java.v30.types.QueueStatistics;
import de.starface.integration.uci.java.v30.types.UciQueueCall;
import de.starface.integration.uci.java.v30.types.UserState;

import java.util.List;

/**
 * This interface defines the Queue events that can be received by a UCI client implementation in
 * order to handle Queue changes.
 *
 * @author Mojo
 */
@UciService(UciQueueEvents.SERVICE_NAME)
@RpcInterface(UciQueueEvents.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciQueueEvents {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.events.queue";

	/**
	 * This event allows the UCI client to get informed when the state or number of the queue
	 * settings change.
	 *
	 * @param queueSetting
	 *        the new state of the Queue setting with values for the properties defined in
	 *        {@link QueueSetting}.
	 */
	@RpcAsynchronous
	public void queueSettingsChanged(List<QueueSetting> queueSetting);

	/**
	 * This event allows the UCI client to get informed when the state or number of the queue
	 * setting changes.
	 *
	 * @param queueAccountId
	 *        the new state of the Queue setting with values for the properties defined in
	 *        {@link Integer}.
	 */
	@RpcAsynchronous
	public void queueSettingChanged(Integer queueAccountId);

	/**
	 * One of the agents states has changed. This can be the {@link UserState} or the loginstate at
	 * the queue, as well as a name change.
	 *
	 * @param agentState
	 */
	@RpcAsynchronous
	public void queueAgentStateChanged(QueueAgentState agentState);

	/**
	 * The queue has a new agent or an agent was removed. Please reload the agent list for this
	 * queue.
	 *
	 * @param groupAccountId
	 */
	@RpcAsynchronous
	public void queueAgentChangedEvent(Integer groupAccountId);

	/**
	 * A new queue has been created (parameter newGroup = true) or a queue was deleted (parameter
	 * newGroup = false)
	 *
	 * @param newGroup
	 *        true if a queuegroup was created, false if a queuegroup was deleted
	 * @param groupAccountId
	 *        the accountid of the queuegroup
	 */
	@RpcAsynchronous
	public void queueGroupChangedEvent(boolean newGroup, Integer groupAccountId);

	/**
	 * Updates the UCI client about a new queue call or a statechange of a queue call.
	 *
	 * @param call
	 *        contains all the new/updated information about this queue call.
	 */
	@RpcAsynchronous
	public void queueCallEvent(UciQueueCall call);

	/**
	 * During a call, the id of the call can change (e.g. during a transfer). If this happens, the
	 * existing call must be updated. Ignoring this event may lead to "visually frozen" calls in
	 * your client, because all new events will reference the new id.
	 *
	 * @param oldId
	 *        the former id of the call
	 * @param newId
	 *        the new id of the call
	 */
	@RpcAsynchronous
	public void queueCallChangedIdEvent(String oldId, String newId);

	/**
	 * The statistics of an queue changed
	 *
	 * @param queueStatistics
	 *        informations of the current statistic
	 */
	@RpcAsynchronous
	public void queueStatisticsEvent(QueueStatistics queueStatistics);
}
