/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.types.FaxList;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpFaxListRequests;
import de.starface.integration.uci.java.v30.values.FaxListEntryFolder;
import de.starface.integration.uci.java.v30.values.FaxListEntryProperties;
import de.starface.integration.uci.java.v30.values.GroupRestriction;
import de.starface.integration.uci.java.v30.values.OrderDirection;

import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * This interface contains the UCI Communication Fax List Requests
 *
 * @author tilmann
 */
@UciService(UcpFaxListRequests.SERVICE_NAME)
@RpcInterface(UcpFaxListRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION, automaticFileTransfer = true)
public interface UciFaxListRequests {

	/**
	 * Retrieve a list of received and sent faxes of the current user. The number of entries
	 * retrieved can be restricted by several parameters of this procedure call. The results are
	 * returned as {@link List} of {@link Map} containing the values for the keys defined in
	 * {@link FaxListEntryProperties}.
	 *
	 * @param receivedBefore
	 *        limit returned results to faxes received/sent before the given point of time
	 * @param receivedAfter
	 *        limit returned results to faxes received/sent after the given point of time
	 * @param folderRestriction
	 *        restrict returned results to faxes in the given folder defined in
	 *        {@link FaxListEntryFolder}. If null is given no restriction is used.
	 * @param groupRestriction
	 *        restrict returned results to those conducted by a specified group. Possible choices
	 *        are the String {@link GroupRestriction#NON_GROUP} to return all faxes that do not
	 *        belong to any group, the id of a group to restrict the list to the faxes belonging to
	 *        that group and the empty String to return all faxes without restriction.
	 * @param orderProperty
	 *        give one of the {@link FaxListEntryProperties} to order results by the value of the
	 *        property, if null is given the default {@link FaxListEntryProperties#receptionTime} is
	 *        used.
	 * @param orderDirection
	 *        give one of the {@link OrderDirection}s to define the ordering. If null is given the
	 *        default order direction of the property will be used
	 * @param countOffset
	 *        the offset of the first returned value in the result list after the ordering has been
	 *        applied
	 * @param countLimit
	 *        the count of items returned in the result list
	 * @return a {@link List} containing the result elements
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public FaxList getFaxList(Date receivedBefore, Date receivedAfter, FaxListEntryFolder folderRestriction,
							  String groupRestriction, FaxListEntryProperties orderProperty,
							  OrderDirection orderDirection, int countOffset, int countLimit) throws UciException;

	/**
	 * Creates and returns a temporary URL that can be used to download the fax file for the next 15
	 * minutes.
	 *
	 * @param faxId
	 *        the id of the fax list entry to retrieve the file for
	 * @return The download URL as {@link String}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String getFaxFileUrl(String faxId) throws UciException;

	/**
	 * Downloads and returns the fax file for the given fax id.
	 *
	 * @param faxId
	 *        the id of the fax list entry to retrieve the file for
	 * @return The fax file
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public File getFaxFile(String faxId) throws UciException;

	/**
	 * Remove a fax list entry from the list
	 *
	 * @param faxId
	 *        the id of the fax list entry to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteFaxListEntry(String faxId) throws UciException;

	/**
	 * Remove multiple fax list entries from the list
	 *
	 * @param faxIds
	 *        the ids of the fax list entries to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteFaxListEntries(List<String> faxIds) throws UciException;

	/**
	 * Move a fax list entry to another folder
	 *
	 * @param faxId
	 *        the id of the fax list entry to move
	 * @param destinationFolder
	 *        one of the following {@link FaxListEntryFolder} {@link FaxListEntryFolder#NEW},
	 *        {@link FaxListEntryFolder#OLD} or {@link FaxListEntryFolder#PRIVATE} -
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void moveFaxListEntry(String faxId, FaxListEntryFolder destinationFolder) throws UciException;

	/**
	 * Move multiple fax list entries to another folder
	 *
	 * @param faxIds
	 *        the ids of the fax list entries to move
	 * @param destinationFolder
	 *        one of the following {@link FaxListEntryFolder} {@link FaxListEntryFolder#NEW},
	 *        {@link FaxListEntryFolder#OLD} or {@link FaxListEntryFolder#PRIVATE} -
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void moveFaxListEntries(List<String> faxIds, FaxListEntryFolder destinationFolder) throws UciException;
}
