/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.types.QueueAgentState;
import de.starface.integration.uci.java.v30.types.QueueSetting;
import de.starface.integration.uci.java.v30.types.QueueStatistics;
import de.starface.integration.uci.java.v30.types.UciQueueCall;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;

import java.util.List;

@UciService(UciQueueRequests.SERVICE_NAME)
@RpcInterface(UciQueueRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciQueueRequests {

	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.requests.queue";

	/**
	 * This request allows the UCI client to query the settings of all queues the user is an agent
	 * of.
	 *
	 * @return a {@link List} with a {@link QueueSetting} for each Queue the user is an agent of.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public abstract List<QueueSetting> getQueueSettings() throws UciException;

	/**
	 * Returns the settings of a queue as defined in {@link QueueSetting}
	 *
	 * @param queueAccountId
	 *        the accountid of the queue
	 * @return a {@link QueueSetting} for the requested queue.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public abstract QueueSetting getQueueSetting(int queueAccountId) throws UciException;

	/**
	 * Returns a List of QueueAgents with their current states as defined in {@link QueueAgentState}
	 * of the given queue, if the user is an agent of it.
	 *
	 * @param queueAccountId
	 *        the accountid of the queue
	 * @return A list of Agents
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public abstract List<QueueAgentState> getQueueAgents(int queueAccountId) throws UciException;

	/**
	 * Returns the waiting, ringin, and ongoing calls of the given queue as defined in
	 * {@link UciQueueCall}
	 *
	 * @return A list of calls, if there are any.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public List<UciQueueCall> getQueueCalls() throws UciException;

	/**
	 * Grabs a queue caller from the caller list 
	 *
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public void grabQueueCaller(int queueAccountId, String phone, String callId) throws UciException;

	/**
	 * Returns the statistics of the queue identified by queueAccountId
	 * 
	 * @return QueueStatistics
	 *  
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 * 
	 */
	public QueueStatistics getQueueStatistics(int queueAccountId) throws UciException;
}
