/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.types.RedirectSetting;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpRedirectRequests;
import de.starface.integration.uci.java.v30.values.GroupRestriction;
import de.starface.integration.uci.java.v30.values.OrderDirection;
import de.starface.integration.uci.java.v30.values.RedirectSettingProperties;
import de.starface.integration.uci.java.v30.values.RedirectSettingType;

import java.util.List;

/**
 * This interface contains the UCI Communication Redirect Requests
 *
 * @author tilmann
 */
@RpcInterface(UcpRedirectRequests.SERVICE_NAME)
@UciService(UcpRedirectRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciRedirectRequests {
	/**
	 * Retrieve the redirect settings for the current user.
	 *
	 * @param typeRestriction
	 *        if one of the constants in {@link RedirectSettingType} is given the settings returned
	 *        are restricted to those of the given type. If null is given no restriction is made.
	 * @param groupRestriction
	 *        restrict returned results to those belonging a specified group. Possible choices are
	 *        the String {@link GroupRestriction#NON_GROUP} to return redirect settings that do not
	 *        belong to any group, the id of a group to restrict the list to the calls belonging to
	 *        that group and the empty String to return all calls without restriction.
	 * @param orderProperty
	 *        give the name of one of the {@link RedirectSettingProperties} to order results by the
	 *        value of the property, if null is given the default
	 *        {@link RedirectSettingProperties#calledNumber} is used.
	 * @param orderDirection
	 *        give the name of one of the {@link OrderDirection}s to define the ordering. If null is
	 *        given the default order direction of the property will be used
	 * @return a {@link List} of {@link RedirectSetting} with the current settings of the user
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public List<RedirectSetting> getRedirectSettings(RedirectSettingType typeRestriction, String groupRestriction,
													 RedirectSettingProperties orderProperty,
													 OrderDirection orderDirection) throws UciException;

	/**
	 * Save the given redirect setting. The value for the property
	 * {@link RedirectSettingProperties#mailboxes} is ignored this is read only information given by
	 * the server.
	 *
	 * @param redirectSetting
	 *        the setting to be saved
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void setRedirectSetting(RedirectSetting redirectSetting) throws UciException;

	/**
	 * Save the given redirect settings. The value for the properties
	 * {@link RedirectSettingProperties#mailboxes} are ignored. This is read only information given
	 * by the server.
	 *
	 * @param redirectSettings
	 *        the settings to be saved
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void setRedirectSettings(List<RedirectSetting> redirectSettings) throws UciException;
}
