/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.types.Image;
import de.starface.integration.uci.java.v30.types.UserState;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpUserStateRequests;
import de.starface.integration.uci.java.v30.values.UserStateProperties;

import java.io.File;

/**
 * This interface defines the requests used to alter the user's state provided by the UCI server.
 *
 * @author tilmann
 */
@RpcInterface(UcpUserStateRequests.SERVICE_NAME)
@UciService(UcpUserStateRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION, automaticFileTransfer = true)
public interface UciUserStateRequests {
	/**
	 * This request can be used by the UCI client to obtain the current user's state with
	 * information about his DND, telefony, chat and avatar state.
	 *
	 * @return the user's current state information
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public UserState getUserState() throws UciException;

	/**
	 * This request can be used by the UCI client to set the user do not disturb setting. If DND is
	 * active all phone calls to that user are denied by the server.
	 *
	 * @param active
	 *        the new state for the do not disturb setting
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public void setDoNotDisturbSetting(boolean active) throws UciException;

	/**
	 * This request can be used by the UCI client to set the chat presence message of the user. The
	 * presence status itself cannot be set. It is determined from the presence states of the user's
	 * jabber/XMPP clients.
	 *
	 * @param message
	 *        the new presence message to be set
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void setChatPresenceMessage(String message) throws UciException;

	/**
	 * This request can be used by the UCI client to retrieve the user's avatar image. If the user
	 * has an avatar image set the {@link UserStateProperties#avatarHash} property holds an non
	 * empty hash of the image.
	 *
	 * @return the avatar image of the user
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public Image getAvatarImage() throws UciException;

	/**
	 * This request can be used by the UCI client to set a new avatar image for the user.
	 *
	 * @param image
	 *        the new image to be set.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void setAvatarImage(Image image) throws UciException;

	/**
	 * This request can be used by the UCI client to set a new avatar image for the user.
	 *
	 * @param file
	 *        the file of the new image to be set.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void setAvatarImageFile(File file) throws UciException;

	/**
	 * This request can be used by the UCI client to get the name of the current user.
	 *
	 * @return the current name
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String getUserName() throws UciException;
}
