/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.messages.requests;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.types.Mailbox;
import de.starface.integration.uci.java.v30.types.VoicemailList;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpVoicemailListRequests;
import de.starface.integration.uci.java.v30.values.MailboxProperties;
import de.starface.integration.uci.java.v30.values.OrderDirection;
import de.starface.integration.uci.java.v30.values.VoicemailListEntryFolder;
import de.starface.integration.uci.java.v30.values.VoicemailListEntryProperties;

import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * This interface contains the UCI Communication Voicemail List Requests
 *
 * @author tilmann
 */
@UciService(UcpVoicemailListRequests.SERVICE_NAME)
@RpcInterface(UcpVoicemailListRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION, automaticFileTransfer = true)
public interface UciVoicemailListRequests {
	/**
	 * Retrieve a list of voicemails of the current user. The number of entries retrieved can be
	 * restricted by several parameters of this procedure call. The results are returned as
	 * {@link List} of {@link Map} containing the values for the keys defined in
	 * {@link VoicemailListEntryProperties}.
	 *
	 * @param startBefore
	 *        limit returned results to voicemails started before the given point of time
	 * @param startAfter
	 *        limit returned results to voicemails started after the given point of time
	 * @param mailboxRestriction
	 *        restrict returned results to voicemails in the mailbox with the given id. Available
	 *        mailboxes for the current user may be retrieved with a call to {@link #getMailboxes()}
	 *        . If an empty String is given no restriction is used.
	 * @param folderRestriction
	 *        restrict returned results to voicemails in the given folder defined in
	 *        {@link VoicemailListEntryFolder}. If null is given no restriction is used.
	 * @param orderProperty
	 *        give one of the {@link VoicemailListEntryProperties} to order results by the value of
	 *        the property. Allowed values are {@link VoicemailListEntryProperties#startTime},
	 *        {@link VoicemailListEntryProperties#duration},
	 *        {@link VoicemailListEntryProperties#callerNumber} and
	 *        {@link VoicemailListEntryProperties#calledNumber}. When null or an invalid property is
	 *        given the default {@link VoicemailListEntryProperties#startTime} is used.
	 * @param orderDirection
	 *        give one of the {@link OrderDirection}s to define the ordering. If null is given the
	 *        default order direction of the property will be used
	 * @param countOffset
	 *        the offset of the first returned value in the result list after the ordering has been
	 *        applied
	 * @param countLimit
	 *        the count of items returned in the result list
	 * @return a {@link List} containing the result elements
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public VoicemailList getVoicemailList(Date startBefore, Date startAfter, String mailboxRestriction,
										  VoicemailListEntryFolder folderRestriction,
										  VoicemailListEntryProperties orderProperty,
										  OrderDirection orderDirection, int countOffset, int countLimit)
		throws UciException;

	/**
	 * Remove a voicemail list entry from the list
	 *
	 * @param voicemailId
	 *        the id of the voicemail list entry to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteVoicemailListEntry(String voicemailId) throws UciException;

	/**
	 * Remove multiple voicemail list entries from the list
	 *
	 * @param voicemailIds
	 *        the ids of the voicemail list entries to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteVoicemailListEntries(List<String> voicemailIds) throws UciException;

	/**
	 * Move a voicemail list entry to another folder
	 *
	 * @param voicemailId
	 *        the id of the voicemail list entry to move
	 * @param destinationFolder
	 *        the name of the folder to move the entry to as defined in
	 *        {@link VoicemailListEntryFolder}.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void moveVoicemailListEntry(String voicemailId, VoicemailListEntryFolder destinationFolder)
		throws UciException;

	/**
	 * Move mutliple voicemail list entries to another folder
	 *
	 * @param voicemailIds
	 *        the ids of the voicemail list entries to move
	 * @param destinationFolder
	 *        the name of the folder to move the entry to as defined in
	 *        {@link VoicemailListEntryFolder}.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void moveVoicemailListEntries(List<String> voicemailIds, VoicemailListEntryFolder destinationFolder)
		throws UciException;

	/**
	 * Creates and returns a temporary URL that can be used to download the voicemail file for the
	 * next 15 minutes.
	 *
	 * @param voicemailId
	 *        the id of the voicemail list entry to retrieve the file for
	 * @return The download URL as {@link String}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String getVoicemailFileUrl(String voicemailId) throws UciException;

	/**
	 * Downloads and returns the voicemail for the given voicemail id
	 *
	 * @param voicemailId
	 *        the id of the voicemail list entry to retrieve the file for
	 * @return the voicemail file
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public File getVoicemailFile(String voicemailId) throws UciException;

	/**
	 * Retrieve a list with all mailboxes of the current user
	 *
	 * @return a {@link List} with one {@link Map} for each mailbox containing the properties as
	 *         defined in {@link MailboxProperties}.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public List<Mailbox> getMailboxes() throws UciException;

	/**
	 * Initiate a phone call to the given phone number that when answered is connected to one of the
	 * current users voicemail boxes
	 *
	 * @param mailboxId
	 *        the id of the mailbox to connect to
	 * @param originatingNumber
	 *        the phone number to be called
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void callMailboxWithNumber(String mailboxId, String originatingNumber) throws UciException;

	/**
	 * Initiate a phone call to the given voicemail box using the given phone
	 *
	 * @param mailboxId
	 *        the id of the mailbox to connect to
	 * @param phoneId
	 *        the id of the phone used to initiate the call with as returned by
	 *        {@link UciPhoneRequests#getPhoneIds()} If not set the primary phone will be used.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void callMailboxWithPhone(String mailboxId, String phoneId) throws UciException;

	/**
	 * Initiate a phone call to the given phone number that when answered plays one of the current
	 * users voicemails.
	 *
	 * @param voicemailId
	 *        the id of the voicemail to be played
	 * @param originatingNumber
	 *        the phone number to be called
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void callVoicemailWithNumber(String voicemailId, String originatingNumber) throws UciException;

	/**
	 * Initiate a call with the given phone that plays one of the current users voicemails.
	 *
	 * @param voicemailId
	 *        the id of the voicemail to be played
	 * @param phoneId
	 *        the id of the phone used to initiate the call with as returned by
	 *        {@link UciPhoneRequests#getPhoneIds()} If not set the primary phone will be used.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void callVoicemailWithPhone(String voicemailId, String phoneId) throws UciException;
}
