/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.events.UciCallEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciCallListRequests;
import de.starface.integration.uci.java.v30.messages.requests.UciCallRequests;
import de.starface.integration.uci.java.v30.values.CallListEntryDirection;
import de.starface.integration.uci.java.v30.values.CallListEntryResult;

import java.io.Serializable;
import java.util.Date;
import java.util.Objects;

/**
 * This object holds the properties of the call list entries as returned by
 * {@link UciCallListRequests#getCallList(Date, Date, CallListEntryDirection, CallListEntryResult, String, de.starface.integration.uci.java.v30.values.CallListEntryProperties, de.starface.integration.uci.java.v30.values.OrderDirection, int, int)}
 *
 * @author tilmann
 */
@RpcValueObject
public class CallListEntry implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * The id of the call list entry
	 *
	 * ATTENTION: This is not the same id as used in the call functions of {@link UciCallRequests}
	 * and {@link UciCallEvents}
	 */
	@RpcValue
	private String id;

	/** The number that was called, if known */
	@RpcValue
	private String calledNumber = "";

	/** The number of the caller, if known */
	@RpcValue
	private String callerNumber = "";

	/** A description of the call (usually the CLIP), if known */
	@RpcValue
	private String callDescription = "";

	/**
	 * The start time of the call. Per default dates will be ordered newest first
	 */
	@RpcValue
	private Date startTime;

	/** The duration of the call in seconds */
	@RpcValue
	private int duration;

	/**
	 * The direction of the call
	 */
	@RpcValue
	private CallListEntryDirection direction;

	/**
	 * The result of the call
	 */
	@RpcValue
	private CallListEntryResult result;

	/**
	 * The id of a voicemail that was recorded for the call if existing (the empty string otherwise)
	 */
	@RpcValue
	private String voicemailId = "";

	/**
	 * The name of the group if this call was received by a group (the empty string otherwise)
	 */
	@RpcValue(minVersion = 22)
	private String groupId = "";

	/**
	 * The Jabber-ID of the caller if the caller is known, empty String otherwise
	 */
	@RpcValue
	private String callerJabberId = "";

	/**
	 * The Account-ID of the call partner if the partner is known, empty String otherwise
	 */
	@RpcValue
	private String callPartnerAccountId = "";

	/**
	 * Human-readable string identifying the entity that caused the call result (especially, answered a group call).
	 */
	@RpcValue
	private String answeredBy = "";

	@RpcValue
	private String comment = "";

	@RpcValue
	private String commentAuthor = "";

	@RpcValue
	private Date commentModified;

	@RpcValue
	private boolean calledBack;

	@RpcValue
	private String calledBackAuthor = "";

	@RpcValue
	private Date calledBackModified;

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getCalledNumber() {
		return calledNumber;
	}

	public void setCalledNumber(String calledNumber) {
		this.calledNumber = calledNumber;
	}

	public String getCallerNumber() {
		return callerNumber;
	}

	public void setCallerNumber(String callerNumber) {
		this.callerNumber = callerNumber;
	}

	public String getCallDescription() {
		return callDescription;
	}

	public void setCallDescription(String callDescription) {
		this.callDescription = callDescription;
	}

	public Date getStartTime() {
		return startTime;
	}

	public void setStartTime(Date startTime) {
		this.startTime = startTime;
	}

	public int getDuration() {
		return duration;
	}

	public void setDuration(int duration) {
		this.duration = duration;
	}

	public CallListEntryDirection getDirection() {
		return direction;
	}

	public void setDirection(CallListEntryDirection direction) {
		this.direction = direction;
	}

	public CallListEntryResult getResult() {
		return result;
	}

	public void setResult(CallListEntryResult result) {
		this.result = result;
	}

	public String getVoicemailId() {
		return voicemailId;
	}

	public void setVoicemailId(String voicemailId) {
		this.voicemailId = voicemailId;
	}

	public String getGroupId() {
		return groupId;
	}

	public void setGroupId(String groupId) {
		this.groupId = groupId;
	}

	public boolean isGroupCall() {
		return groupId != null && !"".equals(groupId);
	}

	public String getCallerJabberId() {
		return callerJabberId;
	}

	public void setCallerJabberId(final String callerJabberId) {
		this.callerJabberId = callerJabberId;
	}

	public String getAnsweredBy() {
		return answeredBy;
	}

	public void setAnsweredBy(String answeredBy) {
		this.answeredBy = answeredBy;
	}

	public String getCallPartnerAccountId() {
		return callPartnerAccountId;
	}

	public void setCallPartnerAccountId(final String callPartnerAccountId) {
		this.callPartnerAccountId = callPartnerAccountId;
	}

	public String getComment() {
		return comment;
	}

	public void setComment(String comment) {
		this.comment = comment;
	}

	public String getCommentAuthor() {
		return commentAuthor;
	}

	public void setCommentAuthor(String commentAuthor) {
		this.commentAuthor = commentAuthor;
	}

	public Date getCommentModified() {
		return commentModified;
	}

	public void setCommentModified(Date commentModified) {
		this.commentModified = commentModified;
	}

	public boolean isCalledBack() {
		return calledBack;
	}

	public void setCalledBack(boolean calledBack) {
		this.calledBack = calledBack;
	}

	public String getCalledBackAuthor() {
		return calledBackAuthor;
	}

	public void setCalledBackAuthor(String calledBackAuthor) {
		this.calledBackAuthor = calledBackAuthor;
	}

	public Date getCalledBackModified() {
		return calledBackModified;
	}

	public void setCalledBackModified(Date calledBackModified) {
		this.calledBackModified = calledBackModified;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null || getClass() != o.getClass()) {
			return false;
		}
		CallListEntry that = (CallListEntry) o;
		return duration == that.duration && calledBack == that.calledBack && Objects.equals(id, that.id) &&
				Objects.equals(calledNumber, that.calledNumber) &&
				Objects.equals(callerNumber, that.callerNumber) &&
				Objects.equals(callDescription, that.callDescription) &&
				Objects.equals(startTime, that.startTime) && direction == that.direction &&
				result == that.result && Objects.equals(voicemailId, that.voicemailId) &&
				Objects.equals(groupId, that.groupId) &&
				Objects.equals(callerJabberId, that.callerJabberId) &&
				Objects.equals(callPartnerAccountId, that.callPartnerAccountId) &&
				Objects.equals(comment, that.comment) &&
				Objects.equals(commentAuthor, that.commentAuthor) &&
				Objects.equals(commentModified, that.commentModified) &&
				Objects.equals(calledBackAuthor, that.calledBackAuthor) &&
				Objects.equals(calledBackModified, that.calledBackModified) &&
				Objects.equals(answeredBy, that.answeredBy);
	}

	@Override
	public int hashCode() {
		return Objects.hash(id, calledNumber, callerNumber, callDescription, startTime, duration, direction, result,
				voicemailId, groupId, callerJabberId, callPartnerAccountId, comment, commentAuthor, commentModified,
				calledBack, calledBackAuthor, calledBackModified, answeredBy);
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder("CallListEntry [");
		sb.append("id=").append(id);
		sb.append(", calledNumber=").append(calledNumber);
		sb.append(", callerNumber=").append(callerNumber);
		sb.append(", callDescription=").append(callDescription);
		sb.append(", startTime=").append(startTime);
		sb.append(", duration=").append(duration);
		sb.append(", direction=").append(direction);
		sb.append(", result=").append(result);
		sb.append(", voicemailId=").append(voicemailId);
		sb.append(", groupId=").append(groupId);
		sb.append(", callerJabberId=").append(callerJabberId);
		sb.append(", callPartnerAccountId=").append(callPartnerAccountId);
		sb.append(", comment=").append(comment);
		sb.append(", commentAuthor=").append(commentAuthor);
		sb.append(", commentModified=").append(commentModified);
		sb.append(", calledBack=").append(calledBack);
		sb.append(", calledBackAuthor=").append(calledBackAuthor);
		sb.append(", calledBackModified=").append(calledBackModified);
		sb.append(", answeredBy=").append(answeredBy);
		sb.append(']');
		return sb.toString();
	}

}
