/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.com.rpc.annotation.RpcValues;
import de.starface.integration.uci.java.v30.messages.events.UciFaxEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciFaxListRequests;
import de.starface.integration.uci.java.v30.messages.requests.UciFaxRequests;
import de.starface.integration.uci.java.v30.values.FaxListEntryFolder;
import de.starface.integration.uci.java.v30.values.FaxListEntryProperties;
import de.starface.integration.uci.java.v30.values.OrderDirection;

import java.io.Serializable;
import java.util.Date;

/**
 * This object holds the properties of the fax list entries as returned by
 * {@link UciFaxListRequests#getFaxList(Date, Date, FaxListEntryFolder, String, FaxListEntryProperties, OrderDirection, int, int)}
 *
 * @author tilmann
 */
@RpcValueObject
public class FaxListEntry implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * The id of the fax list entry
	 *
	 * ATTENTION: This is not the same id as used in the fax functions of {@link UciFaxRequests} and
	 * {@link UciFaxEvents}
	 */
	@RpcValue
	private String id;

	/** The number that was called, if known */
	@RpcValue
	private String calledNumber = "";

	/** The number of the sender, if known */
	@RpcValue
	private String callerNumber = "";

	/** A description of the fax call (usually the CLIP), if known */
	@RpcValue
	private String callDescription = "";

	/**
	 * The folder the fax belongs to holding one of the values defined in {@link FaxListEntryFolder}
	 * .
	 */
	@RpcValue
	private FaxListEntryFolder folder;

	/**
	 * The time when the fax was received. Dates will be ordered newest first per default.
	 */
	@RpcValue
	private Date receptionTime;

	/** A flag indicating that the fax was not transmitted completely */

	@RpcValues({@RpcValue(value = "isPartial", minVersion = 22), @RpcValue(maxVersion = 21)})
	private boolean partial;

	/**
	 * The name of the group if this fax was received by a group (the empty string otherwise)
	 */
	@RpcValue(minVersion = 22)
	private String groupId = "";

	/**
	 * The Jabber-ID of the caller if the caller is known, empty String otherwise
	 */
	@RpcValue
	private String callerJabberId = "";

	/**
	 * The Account-ID of the call partner if the partner is known, empty String otherwise
	 */
	@RpcValue
	private String callPartnerAccountId = "";

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getCalledNumber() {
		return calledNumber;
	}

	public void setCalledNumber(String calledNumber) {
		this.calledNumber = calledNumber;
	}

	public String getCallerNumber() {
		return callerNumber;
	}

	public void setCallerNumber(String callerNumber) {
		this.callerNumber = callerNumber;
	}

	public String getCallDescription() {
		return callDescription;
	}

	public void setCallDescription(String callDescription) {
		this.callDescription = callDescription;
	}

	public FaxListEntryFolder getFolder() {
		return folder;
	}

	public void setFolder(FaxListEntryFolder folder) {
		this.folder = folder;
	}

	public Date getReceptionTime() {
		return receptionTime;
	}

	public void setReceptionTime(Date receptionTime) {
		this.receptionTime = receptionTime;
	}

	public boolean isPartial() {
		return partial;
	}

	public void setPartial(boolean partial) {
		this.partial = partial;
	}

	public String getGroupId() {
		return groupId;
	}

	public void setGroupId(String groupId) {
		this.groupId = groupId;
	}

	public boolean isGroupFax() {
		return groupId != null && !"".equals(groupId);
	}

	public boolean isIncomingFax() {
		return folder != FaxListEntryFolder.SENT;
	}

	public String getCallerJabberId() {
		return callerJabberId;
	}

	public void setCallerJabberId(final String callerJabberId) {
		this.callerJabberId = callerJabberId;
	}

	@Override
	public boolean equals(final Object o) {
		if (this == o) {
			return true;
		}
		if (!(o instanceof FaxListEntry)) {
			return false;
		}

		final FaxListEntry that = (FaxListEntry) o;

		if (partial != that.partial) {
			return false;
		}
		if (id != null ? !id.equals(that.id) : that.id != null) {
			return false;
		}
		if (calledNumber != null ? !calledNumber.equals(that.calledNumber) : that.calledNumber != null) {
			return false;
		}
		if (callerNumber != null ? !callerNumber.equals(that.callerNumber) : that.callerNumber != null) {
			return false;
		}
		if (callDescription != null ? !callDescription.equals(that.callDescription) : that.callDescription != null) {
			return false;
		}
		if (folder != that.folder) {
			return false;
		}
		if (receptionTime != null ? !receptionTime.equals(that.receptionTime) : that.receptionTime != null) {
			return false;
		}
		if (groupId != null ? !groupId.equals(that.groupId) : that.groupId != null) {
			return false;
		}
		if (callerJabberId != null ? !callerJabberId.equals(that.callerJabberId) : that.callerJabberId != null) {
			return false;
		}
		return callPartnerAccountId != null ? callPartnerAccountId.equals(that.callPartnerAccountId) : that.callPartnerAccountId == null;

	}

	@Override
	public int hashCode() {
		int result = id != null ? id.hashCode() : 0;
		result = 31 * result + (calledNumber != null ? calledNumber.hashCode() : 0);
		result = 31 * result + (callerNumber != null ? callerNumber.hashCode() : 0);
		result = 31 * result + (callDescription != null ? callDescription.hashCode() : 0);
		result = 31 * result + (folder != null ? folder.hashCode() : 0);
		result = 31 * result + (receptionTime != null ? receptionTime.hashCode() : 0);
		result = 31 * result + (partial ? 1 : 0);
		result = 31 * result + (groupId != null ? groupId.hashCode() : 0);
		result = 31 * result + (callerJabberId != null ? callerJabberId.hashCode() : 0);
		result = 31 * result + (callPartnerAccountId != null ? callPartnerAccountId.hashCode() : 0);
		return result;
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder("FaxListEntry [");
		sb.append("id=").append(id);
		sb.append(", calledNumber=").append(calledNumber);
		sb.append(", callerNumber=").append(callerNumber);
		sb.append(", callDescription=").append(callDescription);
		sb.append(", folder=").append(folder);
		sb.append(", receptionTime=").append(receptionTime);
		sb.append(", partial=").append(partial);
		sb.append(", groupId=").append(groupId);
		sb.append(", callerJabberId=").append(callerJabberId);
		sb.append(", callPartnerAccountId=").append(callPartnerAccountId);
		sb.append(']');
		return sb.toString();
	}

	public String getCallPartnerAccountId() {
		return callPartnerAccountId;
	}

	public void setCallPartnerAccountId(final String callPartnerAccountId) {
		this.callPartnerAccountId = callPartnerAccountId;
	}
}
