/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.requests.UciVoicemailListRequests;
import de.starface.integration.uci.java.v30.values.OrderDirection;
import de.starface.integration.uci.java.v30.values.VoicemailListEntryFolder;
import de.starface.integration.uci.java.v30.values.VoicemailListEntryProperties;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * This object holds the properties of the voicemail list as returned by
 * {@link UciVoicemailListRequests#getVoicemailList(Date, Date, String, VoicemailListEntryFolder, VoicemailListEntryProperties, OrderDirection, int, int)}
 *
 * @author tilmann
 */
@RpcValueObject
public class VoicemailList implements Serializable {
	private static final long serialVersionUID = 1L;

	/** The entries start before this point of time */
	@RpcValue
	private Date startBefore;

	/** The entries start after this point of time */
	@RpcValue
	private Date startAfter;

	/**
	 * The entries are restricted to voicemails in the mailbox with the given id. If an empty String
	 * is given no restriction is used.
	 */
	@RpcValue
	private String mailboxRestriction = "";

	/**
	 * The entries are restricted to voicemails in the given folder defined in
	 * {@link VoicemailListEntryFolder}. If an empty String is given no restriction is used.
	 */
	@RpcValue
	private VoicemailListEntryFolder folderRestriction;

	/**
	 * The entries are ordered by the given property. This may be one of the properties in
	 * {@link VoicemailListEntryProperties}
	 */
	@RpcValue
	private VoicemailListEntryProperties orderProperty;

	/**
	 * The entries are ordered in the given direction. This may be one of the values in
	 * {@link OrderDirection}
	 */
	@RpcValue
	private OrderDirection orderDirection;

	/**
	 * The offset used for the retrieval of this list regarding the given time constraints and
	 * restrictions
	 */
	@RpcValue
	private int countOffset;

	/**
	 * The limit used for the retrieval of this list regarding the given time constraints and
	 * restrictions
	 */
	@RpcValue
	private int countLimit;

	/**
	 * The total count of available entries on the server regarding the given time constraints and
	 * restrictions
	 */
	@RpcValue
	private int totalCount;

	/**
	 * The call list entries in this list
	 */
	@RpcValue
	private List<VoicemailListEntry> entries;

	@Override
	public String toString() {
		return "VoicemailList [startBefore=" + startBefore + ", startAfter=" + startAfter
			+ ", mailboxRestriction=" + mailboxRestriction + ", folderRestriction="
			+ folderRestriction + ", orderProperty=" + orderProperty + ", orderDirection="
			+ orderDirection + ", countOffset=" + countOffset + ", countLimit=" + countLimit
			+ ", totalCount=" + totalCount + ", entries=" + entries + "]";
	}

	public Date getStartBefore() {
		return startBefore;
	}

	public void setStartBefore(Date startBefore) {
		this.startBefore = startBefore;
	}

	public Date getStartAfter() {
		return startAfter;
	}

	public void setStartAfter(Date startAfter) {
		this.startAfter = startAfter;
	}

	public String getMailboxRestriction() {
		return mailboxRestriction;
	}

	public void setMailboxRestriction(String mailboxRestriction) {
		this.mailboxRestriction = mailboxRestriction;
	}

	public VoicemailListEntryFolder getFolderRestriction() {
		return folderRestriction;
	}

	public void setFolderRestriction(VoicemailListEntryFolder folderRestriction) {
		this.folderRestriction = folderRestriction;
	}

	public VoicemailListEntryProperties getOrderProperty() {
		return orderProperty;
	}

	public void setOrderProperty(VoicemailListEntryProperties orderProperty) {
		this.orderProperty = orderProperty;
	}

	public OrderDirection getOrderDirection() {
		return orderDirection;
	}

	public void setOrderDirection(OrderDirection orderDirection) {
		this.orderDirection = orderDirection;
	}

	public int getCountOffset() {
		return countOffset;
	}

	public void setCountOffset(int countOffset) {
		this.countOffset = countOffset;
	}

	public int getCountLimit() {
		return countLimit;
	}

	public void setCountLimit(int countLimit) {
		this.countLimit = countLimit;
	}

	public int getTotalCount() {
		return totalCount;
	}

	public void setTotalCount(int totalCount) {
		this.totalCount = totalCount;
	}

	public List<VoicemailListEntry> getEntries() {
		return entries;
	}

	public void setEntries(List<VoicemailListEntry> entries) {
		this.entries = entries;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + countLimit;
		result = prime * result + countOffset;
		result = prime * result + ((entries == null) ? 0 : entries.hashCode());
		result = prime * result + ((folderRestriction == null) ? 0 : folderRestriction.hashCode());
		result = prime * result
			+ ((mailboxRestriction == null) ? 0 : mailboxRestriction.hashCode());
		result = prime * result + ((orderDirection == null) ? 0 : orderDirection.hashCode());
		result = prime * result + ((orderProperty == null) ? 0 : orderProperty.hashCode());
		result = prime * result + ((startAfter == null) ? 0 : startAfter.hashCode());
		result = prime * result + ((startBefore == null) ? 0 : startBefore.hashCode());
		result = prime * result + totalCount;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		VoicemailList other = (VoicemailList) obj;
		if (countLimit != other.countLimit) {
			return false;
		}
		if (countOffset != other.countOffset) {
			return false;
		}
		if (entries == null) {
			if (other.entries != null) {
				return false;
			}
		} else if (!entries.equals(other.entries)) {
			return false;
		}
		if (folderRestriction != other.folderRestriction) {
			return false;
		}
		if (mailboxRestriction == null) {
			if (other.mailboxRestriction != null) {
				return false;
			}
		} else if (!mailboxRestriction.equals(other.mailboxRestriction)) {
			return false;
		}
		if (orderDirection != other.orderDirection) {
			return false;
		}
		if (orderProperty != other.orderProperty) {
			return false;
		}
		if (startAfter == null) {
			if (other.startAfter != null) {
				return false;
			}
		} else if (!startAfter.equals(other.startAfter)) {
			return false;
		}
		if (startBefore == null) {
			if (other.startBefore != null) {
				return false;
			}
		} else if (!startBefore.equals(other.startBefore)) {
			return false;
		}
		if (totalCount != other.totalCount) {
			return false;
		}
		return true;
	}

}
