/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.types;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.requests.UciVoicemailListRequests;
import de.starface.integration.uci.java.v30.values.OrderDirection;
import de.starface.integration.uci.java.v30.values.VoicemailListEntryFolder;
import de.starface.integration.uci.java.v30.values.VoicemailListEntryProperties;

import java.io.Serializable;
import java.util.Date;

/**
 * This object holds the properties of the voicemail list entries as returned by
 * {@link UciVoicemailListRequests#getVoicemailList(Date, Date, String, VoicemailListEntryFolder, VoicemailListEntryProperties, OrderDirection, int, int)}
 *
 * @author tilmann
 */
@RpcValueObject
public class VoicemailListEntry implements Serializable {
	private static final long serialVersionUID = 1L;

	/** The id of the voicemail list entry */
	@RpcValue
	private String id;

	/** The id of the corresponding CDR entry */
	@RpcValue
	private String cdrId;

	/** The id of the mailbox of the voicemail */
	@RpcValue
	private String mailboxId;

	/** The display name of the mailbox of the voicemail */
	@RpcValue
	private String mailboxName;

	/**
	 * The folder the voicemail belongs to holding one of the values defined in
	 * {@link VoicemailListEntryFolder}.
	 */
	@RpcValue
	private VoicemailListEntryFolder folder;

	/** The number that was called, if known */
	@RpcValue
	private String calledNumber;

	/** The number of the caller, if known */
	@RpcValue
	private String callerNumber;

	/** A description of the call (usually the CLIP), if known */
	@RpcValue
	private String callDescription;

	/**
	 * The start time of the call. Per default dates will be ordered newest first
	 */
	@RpcValue
	private Date startTime;

	/** The duration of the call in seconds */
	@RpcValue
	private int duration;

	/**
	 * The name of the group if this voicemail was received by a group (the empty string otherwise)
	 */
	@RpcValue(minVersion = 22)
	private String groupId = "";

	/**
	 * The Jabber-ID of the caller if the caller is known, empty String otherwise
	 */
	@RpcValue
	private String callerJabberId = "";

	/**
	 * The Account-ID of the call partner if the partner is known, empty String otherwise
	 */
	@RpcValue
	private String callPartnerAccountId = "";

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getMailboxId() {
		return mailboxId;
	}

	public void setMailboxId(String mailboxId) {
		this.mailboxId = mailboxId;
	}

	public String getMailboxName() {
		return mailboxName;
	}

	public void setMailboxName(String mailboxName) {
		this.mailboxName = mailboxName;
	}

	public VoicemailListEntryFolder getFolder() {
		return folder;
	}

	public void setFolder(VoicemailListEntryFolder folder) {
		this.folder = folder;
	}

	public String getCalledNumber() {
		return calledNumber;
	}

	public void setCalledNumber(String calledNumber) {
		this.calledNumber = calledNumber;
	}

	public String getCallerNumber() {
		return callerNumber;
	}

	public void setCallerNumber(String callerNumber) {
		this.callerNumber = callerNumber;
	}

	public String getCallDescription() {
		return callDescription;
	}

	public void setCallDescription(String callDescription) {
		this.callDescription = callDescription;
	}

	public Date getStartTime() {
		return startTime;
	}

	public void setStartTime(Date startTime) {
		this.startTime = startTime;
	}

	public int getDuration() {
		return duration;
	}

	public void setDuration(int duration) {
		this.duration = duration;
	}

	public String getGroupId() {
		return groupId;
	}

	public void setGroupId(String groupId) {
		this.groupId = groupId;
	}

	public boolean isGroupCall() {
		return groupId != null && !"".equals(groupId);
	}

	public String getCdrId() {
		return cdrId;
	}

	public void setCdrId(String cdrId) {
		this.cdrId = cdrId;
	}

	public String getCallerJabberId() {
		return callerJabberId;
	}

	public void setCallerJabberId(final String callerJabberId) {
		this.callerJabberId = callerJabberId;
	}

	@Override
	public boolean equals(final Object o) {
		if (this == o) {
			return true;
		}
		if (!(o instanceof VoicemailListEntry)) {
			return false;
		}

		final VoicemailListEntry that = (VoicemailListEntry) o;

		if (duration != that.duration) {
			return false;
		}
		if (id != null ? !id.equals(that.id) : that.id != null) {
			return false;
		}
		if (cdrId != null ? !cdrId.equals(that.cdrId) : that.cdrId != null) {
			return false;
		}
		if (mailboxId != null ? !mailboxId.equals(that.mailboxId) : that.mailboxId != null) {
			return false;
		}
		if (mailboxName != null ? !mailboxName.equals(that.mailboxName) : that.mailboxName != null) {
			return false;
		}
		if (folder != that.folder) {
			return false;
		}
		if (calledNumber != null ? !calledNumber.equals(that.calledNumber) : that.calledNumber != null) {
			return false;
		}
		if (callerNumber != null ? !callerNumber.equals(that.callerNumber) : that.callerNumber != null) {
			return false;
		}
		if (callDescription != null ? !callDescription.equals(that.callDescription) : that.callDescription != null) {
			return false;
		}
		if (startTime != null ? !startTime.equals(that.startTime) : that.startTime != null) {
			return false;
		}
		if (groupId != null ? !groupId.equals(that.groupId) : that.groupId != null) {
			return false;
		}
		if (callerJabberId != null ? !callerJabberId.equals(that.callerJabberId) : that.callerJabberId != null) {
			return false;
		}
		return callPartnerAccountId != null ? callPartnerAccountId.equals(that.callPartnerAccountId) : that.callPartnerAccountId == null;

	}

	@Override
	public int hashCode() {
		int result = id != null ? id.hashCode() : 0;
		result = 31 * result + (cdrId != null ? cdrId.hashCode() : 0);
		result = 31 * result + (mailboxId != null ? mailboxId.hashCode() : 0);
		result = 31 * result + (mailboxName != null ? mailboxName.hashCode() : 0);
		result = 31 * result + (folder != null ? folder.hashCode() : 0);
		result = 31 * result + (calledNumber != null ? calledNumber.hashCode() : 0);
		result = 31 * result + (callerNumber != null ? callerNumber.hashCode() : 0);
		result = 31 * result + (callDescription != null ? callDescription.hashCode() : 0);
		result = 31 * result + (startTime != null ? startTime.hashCode() : 0);
		result = 31 * result + duration;
		result = 31 * result + (groupId != null ? groupId.hashCode() : 0);
		result = 31 * result + (callerJabberId != null ? callerJabberId.hashCode() : 0);
		result = 31 * result + (callPartnerAccountId != null ? callPartnerAccountId.hashCode() : 0);
		return result;
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder("VoicemailListEntry [");
		sb.append("id=").append(id);
		sb.append(", cdrId=").append(cdrId);
		sb.append(", mailboxId=").append(mailboxId);
		sb.append(", mailboxName=").append(mailboxName);
		sb.append(", folder=").append(folder);
		sb.append(", calledNumber=").append(calledNumber);
		sb.append(", callerNumber=").append(callerNumber);
		sb.append(", callDescription=").append(callDescription);
		sb.append(", startTime=").append(startTime);
		sb.append(", duration=").append(duration);
		sb.append(", groupId=").append(groupId);
		sb.append(", callerJabberId=").append(callerJabberId);
		sb.append(", callPartnerAccountId=").append(callPartnerAccountId);
		sb.append(']');
		return sb.toString();
	}

	public String getCallPartnerAccountId() {
		return callPartnerAccountId;
	}

	public void setCallPartnerAccountId(final String callPartnerAccountId) {
		this.callPartnerAccountId = callPartnerAccountId;
	}
}
