/*
 *  Copyright (C) by STARFACE GmbH, Karlsruhe, Germany. All rights reserved.
 *
 *  NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY STARFACE GMBH. UNAUTHORIZED ACCESS,
 *  USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 */
package de.starface.integration.uci.java.v30.values;

import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpRedirectRequests;

import java.util.List;
import java.util.Map;

/**
 * This enum holds constants with the property names of the redirect settings as returned by
 * {@link UcpRedirectRequests#getRedirectSettings(String, String, String, String)}
 *
 * @author tilmann
 */
public enum RedirectSettingProperties {
	/**
	 * The type of the redirect setting as {@link String} with one of the values of
	 * {@link RedirectSettingType}. The type together with the {@link #calledNumber} uniquely
	 * identify the redirect setting.
	 */
	type(OrderDirection.ASCENDING, String.class),

	/**
	 * The incoming number for which the redirect holds as {@link String}. The calledNumber together
	 * with the {@link #type} uniquely identify the redirect setting.
	 */
	calledNumber(OrderDirection.ASCENDING, String.class),

	/**
	 * The incoming number for which the redirect holds in its distinct format. If multiple external trunk numbers
	 * with different prefixes (national-code or area-code) are configured, it's not possible to distinguish the
	 * RedirectSetting by just its trunk number. The distinctCalledNumber hence 
	 * <ul>
	 *     <li>is equals to the calledNumber if no overlapping external trunk numbers are configured (i.e. "123456789"),</li>
	 *     <li>is prefixed by the area-code (local prefix) if there are overlapping trunk numbers (i.e. "(721) 123456789"),</li>
	 *     <li>is prefixed by the area-code and national-code if there are overlapping trunk numbers with identical
	 *     area-code (i.e. "(0049) (721) 123456789")</li>
	 * </ul>
	 *
	 */
	distinctCalledNumber(OrderDirection.ASCENDING, String.class),

	/** The id of a group as {@link String} if the number belongs to a user group */
	groupId(OrderDirection.ASCENDING, String.class),

	/** A {@link Boolean} flag indicating if the redirect is enabled/active */
	enabled(OrderDirection.ASCENDING, Boolean.class),

	/**
	 * The type of the destination of the redirect setting as {@link String} with one of the values
	 * of {@link RedirectSettingDestinationType}
	 */
	destinationType(OrderDirection.ASCENDING, String.class),

	/**
	 * The destination of the redirect as {@link String}. This may either be a phone number if the
	 * {@link #destinationType} is {@link RedirectSettingDestinationType#PHONENUMBER} or the id of
	 * one of the mailboxes given in {@link #mailboxes} if the {@link #destinationType} is
	 * {@link RedirectSettingDestinationType#VOICEMAIL}
	 */
	destination(OrderDirection.ASCENDING, String.class),

	/**
	 * A {@link List} of {@link Map} that hold the information about the mailboxes that are valid
	 * destinations for the current redirect setting. The {@link List} contains one entry for every
	 * valid mailbox that is a {@link Map} with the properties defined in {@link MailboxProperties}.
	 */
	mailboxes(OrderDirection.ASCENDING, List.class),

	/**
	 * The timeout in seconds as {@link Integer} that will be used as a trigger for the redirect if
	 * the {@link #type} is set to {@link RedirectSettingType#TIMEOUT}.
	 */
	timeout(OrderDirection.ASCENDING, Integer.class),

	/**
	 * The last known selected destination voicemailbox as {@link String}. The value will be set by
	 * the Server and cannot be changed explicitly by the Client
	 */
	lastDestinationMailbox(OrderDirection.ASCENDING, String.class),

	/**
	 * The last known selected destination phonenumber as {@link String}. The value will be set by
	 * the Server and cannot be changed explicitly by the Client
	 */
	lastDestinationNumber(OrderDirection.ASCENDING, String.class);

	private OrderDirection defaultOrderDirection;

	private Class<?> clazz;

	private RedirectSettingProperties(OrderDirection defaultOrderDirection, Class<?> type) {
		this.defaultOrderDirection = defaultOrderDirection;
		this.clazz = type;
	}

	/**
	 * @return the default {@link OrderDirection} for the current property
	 */
	public OrderDirection getDefaultOrderDirection() {
		return defaultOrderDirection;
	}

	/**
	 * @return the java type of the property
	 */
	public Class<?> getType() {
		return clazz;
	}
}
